@@ -28,10 +28,16 @@
 #     Runs the "slow" version of the unit tests. Set to 0 to
 #     run the tests more quickly.
 #
-#   TEST_TMPDIR   Default: /tmp/kudutest-$UID
-#     Specifies the temporary directory where tests should write their
-#     data. It is expected that following the completion of all tests, this
-#     directory is empty (i.e. every test cleaned up after itself).
+#   TEST_TMPDIR   Default: /tmp/$BUILD_TAG if BUILD_TAG is defined (Jenkins),
+#                          /tmp/kudutest-$UID otherwise
+#     Specifies the temporary directory where tests should write their data
+#     and logs. It is expected that following successful completion
+#     (i.e. no failures) of all the tests, this directory is empty: every
+#     successfully completed test cleans up after itself. Failed tests leave
+#     relevant data and logs in the temporary directory upon exit: that's for
+#     post-mortem troubleshooting and debugging. In case of Jenkins builds
+#     (i.e. when the BUILD_TAG environment variable is defined),
+#     the whole TEST_TMPDIR directory is removed when this script exists.
 #
 #   RUN_FLAKY_ONLY    Default: 0
 #     Only runs tests which have failed recently, if this is 1.
@@ -155,7 +161,11 @@ BUILD_ROOT=$SOURCE_ROOT/build/$BUILD_TYPE_LOWER
 export KUDU_FLAKY_TEST_ATTEMPTS=${KUDU_FLAKY_TEST_ATTEMPTS:-1}
 export KUDU_ALLOW_SLOW_TESTS=${KUDU_ALLOW_SLOW_TESTS:-$DEFAULT_ALLOW_SLOW_TESTS}
 export KUDU_COMPRESS_TEST_OUTPUT=${KUDU_COMPRESS_TEST_OUTPUT:-1}
-export TEST_TMPDIR=${TEST_TMPDIR:-/tmp/kudutest-$UID}
+if [ -n "$BUILD_TAG" ]; then
+  export TEST_TMPDIR=${TEST_TMPDIR:-/tmp/$BUILD_TAG}
+else
+  export TEST_TMPDIR=${TEST_TMPDIR:-/tmp/kudutest-$UID}
+fi
 export PARALLEL=${PARALLEL:-$(getconf _NPROCESSORS_ONLN)}
 export PARALLEL_TESTS=${PARALLEL_TESTS:-$PARALLEL}
 export THIRDPARTY_DIR=${THIRDPARTY_DIR:-$SOURCE_ROOT/thirdparty}
@@ -197,12 +207,13 @@ TEST_LOGDIR="$BUILD_ROOT/test-logs"
 TEST_DEBUGDIR="$BUILD_ROOT/test-debug"
 
 cleanup() {
-  echo Cleaning up all build artifacts...
+  echo Cleaning up all build artifacts and temporary data...
   $SOURCE_ROOT/build-support/jenkins/post-build-clean.sh
 }
-# If we're running inside Jenkins (the BUILD_ID is set), then install
-# an exit handler which will clean up all of our build results.
-if [ -n "$BUILD_ID" ]; then
+# If we're running inside Jenkins (the BUILD_TAG is set), then install
+# an exit handler which will clean up all of our build results and temporary
+# data.
+if [ -n "$BUILD_TAG" ]; then
   trap cleanup EXIT
 fi
 
@@ -345,8 +356,7 @@ fi
 export EXTRA_GRADLE_FLAGS="--console=plain"
 EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --no-daemon"
 EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --continue"
-# Temporarily disable parallel builds for automated builds.
-EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --no-parallel"
+EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --build-cache"
 # KUDU-2524: temporarily disable scalafmt until we can work out its JDK
 # incompatibility issue.
 EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS -DskipFormat"
@@ -480,11 +490,6 @@ if [ "$BUILD_TYPE" != "ASAN" ]; then
   export KUDU_TEST_ULIMIT_CORE=unlimited
 fi
 
-# our tests leave lots of data lying around, clean up before we run
-if [ -d "$TEST_TMPDIR" ]; then
-  rm -Rf $TEST_TMPDIR/*
-fi
-
 # actually do the build
 echo
 echo Building C++ code.
@@ -546,6 +551,10 @@ if [ "$BUILD_JAVA" == "1" ]; then
   # Run the full Gradle build.
   # If we're running distributed Java tests, submit them asynchronously.
   if [ "$ENABLE_DIST_TEST" == "1" ]; then
+    if ! ./gradlew $EXTRA_GRADLE_FLAGS clean assemble; then
+      TESTS_FAILED=1
+      FAILURES="$FAILURES"$'Java Gradle build failed\n'
+    fi
     echo
     echo Submitting Java distributed-test job.
     echo ------------------------------------------------------------
@@ -641,6 +650,17 @@ if [ "$BUILD_PYTHON3" == "1" ]; then
   CC=$CLANG CXX=$CLANG++ python setup.py build_ext
   set +e
 
+  # A testing environment might have HTTP/HTTPS proxy configured to proxy
+  # requests even for 127.0.0.0/8 network or other quirks. Since some Python
+  # Kudu tests rely on Kudu components listening on 127.0.0.0/8 addresses,
+  # let's unset environment variables that might affect behavior of the
+  # Python libraries like urllib/urllib2.
+  unset ALL_PROXY
+  unset HTTP_PROXY
+  unset HTTPS_PROXY
+  unset http_proxy
+  unset https_proxy
+
   # Run the Python tests. This may also involve some compiler work.
   if ! CC=$CLANG CXX=$CLANG++ python setup.py test \
       --addopts="kudu --junit-xml=$TEST_LOGDIR/python3_client.xml" \
