@@ -34,14 +34,13 @@ function install_python_packages() {
   # We use get-pip.py to bootstrap pip outside of system packages.
   # This prevents issues with the platform package manager knowing
   # about only some of the python packages.
-  if [[ "$PYTHON_MAJOR" == "2" && "$PYTHON_MINOR" == "6" ]]; then
-    # Beginning with pip 10, Python 2.6 is no longer supported.
-    curl https://bootstrap.pypa.io/2.6/get-pip.py | python
+  if [[ "$PYTHON_MAJOR" == "2" && "$PYTHON_MINOR" == "7" ]]; then
+    # The standard get-pip.py URL no longer supports Python 2.7,
+    # so we need to use the version specific one.
+    curl https://bootstrap.pypa.io/pip/2.7/get-pip.py | python
   else
-    # Use a stable version of pip that works with the remaining
-    # versions of Python 2 and 3. pip 19.1 doesn't support Python 3.4,
-    # which is the version of Python 3 shipped with Ubuntu 14.04.
-    curl https://bootstrap.pypa.io/get-pip.py | python - "pip < 19.0"
+    # Use a stable version of pip that works with Python 2 and 3.
+    curl https://bootstrap.pypa.io/get-pip.py | python - "pip < 20.3.4"
   fi
   pip install --upgrade \
     cython \
