@@ -58,6 +58,7 @@ public abstract class AbstractKuduScannerBuilder
   long scanRequestTimeout;
   ReplicaSelection replicaSelection = ReplicaSelection.LEADER_ONLY;
   long keepAlivePeriodMs = AsyncKuduClient.DEFAULT_KEEP_ALIVE_PERIOD_MS;
+  String queryId = "";
 
   AbstractKuduScannerBuilder(AsyncKuduClient client, KuduTable table) {
     this.client = client;
@@ -96,7 +97,7 @@ public abstract class AbstractKuduScannerBuilder
   public S setFaultTolerant(boolean isFaultTolerant) {
     this.isFaultTolerant = isFaultTolerant;
     if (isFaultTolerant) {
-      readMode = AsyncKuduScanner.ReadMode.READ_AT_SNAPSHOT;
+      this.readMode = AsyncKuduScanner.ReadMode.READ_AT_SNAPSHOT;
     }
     return (S) this;
   }
@@ -412,5 +413,20 @@ public abstract class AbstractKuduScannerBuilder
     return (S) this;
   }
 
+  /**
+   * Set a query id for the scan to trace the whole scanning process.
+   * Query id is posted by the user or generated automatically by the
+   * client library code. It is used to trace the whole query process
+   * for debugging.
+   *
+   * @param queryId query id to trace a query.
+   * @return this instance
+   */
+  @SuppressWarnings("unchecked")
+  public S setQueryId(String queryId) {
+    this.queryId = queryId;
+    return (S) this;
+  }
+
   public abstract T build();
 }
