@@ -22,6 +22,7 @@ import static org.apache.kudu.test.ClientTestUtil.createBasicSchemaInsert;
 import static org.apache.kudu.test.ClientTestUtil.getBasicCreateTableOptions;
 import static org.apache.kudu.test.ClientTestUtil.getBasicSchema;
 import static org.apache.kudu.test.KuduTestHarness.DEFAULT_SLEEP;
+import static org.apache.kudu.test.junit.AssertHelpers.assertEventuallyTrue;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
@@ -40,6 +41,7 @@ import org.slf4j.LoggerFactory;
 import org.apache.kudu.Schema;
 import org.apache.kudu.WireProtocol.AppStatusPB;
 import org.apache.kudu.test.KuduTestHarness;
+import org.apache.kudu.test.junit.AssertHelpers.BooleanExpression;
 import org.apache.kudu.tserver.Tserver.TabletServerErrorPB;
 
 public class TestAsyncKuduSession {
@@ -387,6 +389,53 @@ public class TestAsyncKuduSession {
     assertEquals(2 * kNumOps, countRowsInTable(table));
   }
 
+  /**
+   * Test KuduSession supports configuring buffer space by data size.
+   */
+  @Test(timeout = 90000)
+  public void testFlushBySize() throws Exception {
+    AsyncKuduSession session = client.newSession();
+    final int kBufferSizeOps = 10;
+    // Considering the existence of buffers, we set a number of operations that is significantly
+    // larger than the number of buffers to ensure that the buffers are triggered to flush.
+    final int kNumOps = 100;
+    // Set a small buffer size so we should flush every time.
+    session.setMutationBufferSpace(kBufferSizeOps, 1);
+    // Set a large flush interval so if the flush by size function is not correctly implemented,
+    // the test will timeout.
+    session.setFlushInterval(60 * 60 * 1000);
+    session.setFlushMode(SessionConfiguration.FlushMode.AUTO_FLUSH_BACKGROUND);
+
+    for (int i = 0; i < kNumOps; i++) {
+      // If the client tries to buffer many more operations, it may receive a
+      // PleaseThrottleException. In this case, if the client simply waits for a flush notification
+      // on the Deferred returned with the exception, it can continue to buffer operations.
+      Insert insert = createInsert(i);
+      try {
+        session.apply(insert);
+      } catch (PleaseThrottleException ex) {
+        ex.getDeferred().join(DEFAULT_SLEEP);
+        session.apply(insert);
+      }
+    }
+    // There might be pending requests in the cache, but the above operation should not generate any
+    // errors.
+    assertEquals(0, session.countPendingErrors());
+    // Confirm that we can still make progress.
+    Insert insert = createInsert(kNumOps);
+    try {
+      session.apply(insert);
+    } catch (PleaseThrottleException ex) {
+      ex.getDeferred().join(DEFAULT_SLEEP);
+      session.apply(insert);
+    }
+
+    for (OperationResponse resp: session.flush().join(DEFAULT_SLEEP)) {
+      assertFalse(resp.hasRowError());
+    }
+    assertEquals(0, session.close().join(DEFAULT_SLEEP).size());
+  }
+
   // A helper just to make some lines shorter.
   private Insert createInsert(int key) {
     return createBasicSchemaInsert(table, key);
