@@ -28,7 +28,9 @@ import static org.junit.Assert.fail;
 
 import java.math.BigDecimal;
 import java.nio.ByteBuffer;
+import java.sql.Date;
 import java.sql.Timestamp;
+import java.time.LocalDate;
 
 import org.junit.Rule;
 import org.junit.Test;
@@ -37,6 +39,7 @@ import org.apache.kudu.ColumnSchema;
 import org.apache.kudu.Schema;
 import org.apache.kudu.Type;
 import org.apache.kudu.test.junit.RetryRule;
+import org.apache.kudu.util.DateUtil;
 
 public class TestPartialRow {
 
@@ -52,16 +55,22 @@ public class TestPartialRow {
     assertEquals(44, partialRow.getInt("int32"));
     assertEquals(45, partialRow.getLong("int64"));
     assertEquals(new Timestamp(1234567890), partialRow.getTimestamp("timestamp"));
+    assertEquals(Date.valueOf(LocalDate.ofEpochDay(0)), partialRow.getDate("date"));
     assertEquals(52.35F, partialRow.getFloat("float"), 0.0f);
     assertEquals(53.35, partialRow.getDouble("double"), 0.0);
     assertEquals("fun with ütf\0", partialRow.getString("string"));
-    assertArrayEquals(new byte[] { 0, 1, 2, 3, 4 }, partialRow.getBinaryCopy("binary-array"));
-    assertArrayEquals(new byte[] { 5, 6, 7, 8, 9 }, partialRow.getBinaryCopy("binary-bytebuffer"));
-    assertEquals(ByteBuffer.wrap(new byte[] { 0, 1, 2, 3, 4 }), partialRow.getBinary("binary-array"));
-    assertEquals(ByteBuffer.wrap(new byte[] { 5, 6, 7, 8, 9 }), partialRow.getBinary("binary-bytebuffer"));
+    assertArrayEquals(new byte[] { 0, 1, 2, 3, 4 },
+        partialRow.getBinaryCopy("binary-array"));
+    assertArrayEquals(new byte[] { 5, 6, 7, 8, 9 },
+        partialRow.getBinaryCopy("binary-bytebuffer"));
+    assertEquals(ByteBuffer.wrap(new byte[] { 0, 1, 2, 3, 4 }),
+        partialRow.getBinary("binary-array"));
+    assertEquals(ByteBuffer.wrap(new byte[] { 5, 6, 7, 8, 9 }),
+        partialRow.getBinary("binary-bytebuffer"));
     assertTrue(partialRow.isSet("null"));
     assertTrue(partialRow.isNull("null"));
-    assertEquals(BigDecimal.valueOf(12345, 3), partialRow.getDecimal("decimal"));
+    assertEquals(BigDecimal.valueOf(12345, 3),
+        partialRow.getDecimal("decimal"));
   }
 
   @Test
@@ -79,26 +88,33 @@ public class TestPartialRow {
     assertEquals((long) 45, partialRow.getObject("int64"));
     assertTrue(partialRow.getObject("timestamp") instanceof Timestamp);
     assertEquals(new Timestamp(1234567890), partialRow.getObject("timestamp"));
+    assertTrue(partialRow.getObject("date") instanceof Date);
+    assertEquals(Date.valueOf(LocalDate.ofEpochDay(0)), partialRow.getObject("date"));
     assertTrue(partialRow.getObject("float") instanceof Float);
     assertEquals(52.35F, (float) partialRow.getObject("float"), 0.0f);
     assertTrue(partialRow.getObject("double") instanceof Double);
     assertEquals(53.35, (double) partialRow.getObject("double"), 0.0);
     assertTrue(partialRow.getObject("string") instanceof String);
     assertEquals("fun with ütf\0", partialRow.getObject("string"));
+    assertTrue(partialRow.getObject("varchar") instanceof String);
+    assertEquals("árvíztűrő ", partialRow.getObject("varchar"));
     assertTrue(partialRow.getObject("binary-array") instanceof byte[]);
-    assertArrayEquals(new byte[] { 0, 1, 2, 3, 4 }, partialRow.getBinaryCopy("binary-array"));
+    assertArrayEquals(new byte[] { 0, 1, 2, 3, 4 },
+        partialRow.getBinaryCopy("binary-array"));
     assertTrue(partialRow.getObject("binary-bytebuffer") instanceof byte[]);
-    assertEquals(ByteBuffer.wrap(new byte[] { 5, 6, 7, 8, 9 }), partialRow.getBinary("binary-bytebuffer"));
+    assertEquals(ByteBuffer.wrap(new byte[] { 5, 6, 7, 8, 9 }),
+        partialRow.getBinary("binary-bytebuffer"));
     assertNull(partialRow.getObject("null"));
     assertTrue(partialRow.getObject("decimal") instanceof BigDecimal);
-    assertEquals(BigDecimal.valueOf(12345, 3), partialRow.getObject("decimal"));
+    assertEquals(BigDecimal.valueOf(12345, 3),
+        partialRow.getObject("decimal"));
   }
 
   @Test
   public void testAddObject() {
     Schema schema = getSchemaWithAllTypes();
     // Ensure we aren't missing any types
-    assertEquals(13, schema.getColumnCount());
+    assertEquals(15, schema.getColumnCount());
 
     PartialRow row = schema.newPartialRow();
     row.addObject("int8", (byte) 42);
@@ -106,10 +122,12 @@ public class TestPartialRow {
     row.addObject("int32", 44);
     row.addObject("int64", 45L);
     row.addObject("timestamp", new Timestamp(1234567890));
+    row.addObject("date", Date.valueOf(LocalDate.ofEpochDay(0)));
     row.addObject("bool", true);
     row.addObject("float", 52.35F);
     row.addObject("double", 53.35);
     row.addObject("string", "fun with ütf\0");
+    row.addObject("varchar", "árvíztűrő tükörfúrógép");
     row.addObject("binary-array", new byte[] { 0, 1, 2, 3, 4 });
     ByteBuffer binaryBuffer = ByteBuffer.wrap(new byte[] { 5, 6, 7, 8, 9 });
     row.addObject("binary-bytebuffer", binaryBuffer);
@@ -299,6 +317,13 @@ public class TestPartialRow {
     assertEquals("22.200", decimal.toString());
   }
 
+  @Test(expected = IllegalArgumentException.class)
+  public void testAddDateOutOfRange() {
+    PartialRow partialRow = getPartialRowWithAllTypes();
+    Date d = Date.valueOf(LocalDate.of(10000, 1, 1));
+    partialRow.addDate("date", d);
+  }
+
   @Test
   public void testToString() {
     Schema schema = getSchemaWithAllTypes();
@@ -334,6 +359,17 @@ public class TestPartialRow {
             "string string=\"fun with ütf\\0\", binary binary-bytebuffer=[2, 3, 4], " +
             "decimal(5, 3) decimal=12.345)",
         row.toString());
+
+    row.addVarchar("varchar", "árvíztűrő tükörfúrógép");
+    assertEquals("(int8 int8=42, int32 int32=42, double double=52.35, " +
+        "string string=\"fun with ütf\\0\", binary binary-bytebuffer=[2, 3, 4], " +
+        "decimal(5, 3) decimal=12.345, varchar(10) varchar=\"árvíztűrő \")",
+        row.toString());
+
+    PartialRow row2 = schema.newPartialRow();
+    assertEquals("()", row2.toString());
+    row2.addDate("date", Date.valueOf(LocalDate.ofEpochDay(0)));
+    assertEquals("(date date=1970-01-01)", row2.toString());
   }
 
   @Test
@@ -408,6 +444,20 @@ public class TestPartialRow {
     partialRow.addBinary(binaryIndex, new byte[] { 0, 1, 2, 3, 4 });
     assertTrue(partialRow.incrementColumn(binaryIndex));
     assertArrayEquals(new byte[] { 0, 1, 2, 3, 4, 0 }, partialRow.getBinaryCopy(binaryIndex));
+
+    // Varchar
+    int varcharIndex = getColumnIndex(partialRow, "varchar");
+    partialRow.addVarchar(varcharIndex, "hello");
+    assertTrue(partialRow.incrementColumn(varcharIndex));
+    assertEquals("hello\0", partialRow.getVarchar(varcharIndex));
+
+    // Date
+    int dateIndex = getColumnIndex(partialRow, "date");
+    partialRow.addDate(dateIndex, DateUtil.epochDaysToSqlDate(DateUtil.MAX_DATE_VALUE - 1));
+    assertTrue(partialRow.incrementColumn(dateIndex));
+    Date maxDate = DateUtil.epochDaysToSqlDate(DateUtil.MAX_DATE_VALUE);
+    assertEquals(maxDate, partialRow.getDate(dateIndex));
+    assertFalse(partialRow.incrementColumn(dateIndex));
   }
 
   @Test
@@ -422,9 +472,11 @@ public class TestPartialRow {
     assertEquals(Integer.MIN_VALUE, partialRow.getInt("int32"));
     assertEquals(Long.MIN_VALUE, partialRow.getLong("int64"));
     assertEquals(Long.MIN_VALUE, partialRow.getLong("timestamp"));
+    assertEquals(DateUtil.epochDaysToSqlDate(DateUtil.MIN_DATE_VALUE), partialRow.getDate("date"));
     assertEquals(-Float.MAX_VALUE, partialRow.getFloat("float"), 0.0f);
     assertEquals(-Double.MAX_VALUE, partialRow.getDouble("double"), 0.0);
     assertEquals("", partialRow.getString("string"));
+    assertEquals("", partialRow.getVarchar("varchar"));
     assertArrayEquals(new byte[0], partialRow.getBinaryCopy("binary-array"));
     assertArrayEquals(new byte[0], partialRow.getBinaryCopy("binary-bytebuffer"));
     assertEquals(BigDecimal.valueOf(-99999, 3), partialRow.getDecimal("decimal"));
@@ -449,7 +501,9 @@ public class TestPartialRow {
       case INT16: return partialRow.getShort(columnName);
       case INT32: return partialRow.getInt(columnName);
       case INT64: return partialRow.getLong(columnName);
+      case DATE: return partialRow.getDate(columnName);
       case UNIXTIME_MICROS: return partialRow.getTimestamp(columnName);
+      case VARCHAR: return partialRow.getVarchar(columnName);
       case STRING: return partialRow.getString(columnName);
       case BINARY: return partialRow.getBinary(columnName);
       case FLOAT: return partialRow.getFloat(columnName);
@@ -470,7 +524,9 @@ public class TestPartialRow {
       case INT16: return partialRow.getShort(columnIndex);
       case INT32: return partialRow.getInt(columnIndex);
       case INT64: return partialRow.getLong(columnIndex);
+      case DATE: return partialRow.getDate(columnIndex);
       case UNIXTIME_MICROS: return partialRow.getTimestamp(columnIndex);
+      case VARCHAR: return partialRow.getVarchar(columnIndex);
       case STRING: return partialRow.getString(columnIndex);
       case BINARY: return partialRow.getBinary(columnIndex);
       case FLOAT: return partialRow.getFloat(columnIndex);
@@ -484,17 +540,45 @@ public class TestPartialRow {
 
   private void callAddByName(PartialRow partialRow, String columnName, Type type) {
     switch (type) {
-      case INT8: partialRow.addByte(columnName, (byte) 42); break;
-      case INT16: partialRow.addShort(columnName, (short) 43); break;
-      case INT32: partialRow.addInt(columnName, 44); break;
-      case INT64: partialRow.addLong(columnName, 45); break;
-      case UNIXTIME_MICROS: partialRow.addTimestamp(columnName, new Timestamp(1234567890)); break;
-      case STRING: partialRow.addString(columnName, "fun with ütf\0"); break;
-      case BINARY: partialRow.addBinary(columnName, new byte[] { 0, 1, 2, 3, 4 }); break;
-      case FLOAT: partialRow.addFloat(columnName, 52.35F); break;
-      case DOUBLE: partialRow.addDouble(columnName, 53.35); break;
-      case BOOL: partialRow.addBoolean(columnName, true); break;
-      case DECIMAL: partialRow.addDecimal(columnName, BigDecimal.valueOf(12345, 3)); break;
+      case INT8:
+        partialRow.addByte(columnName, (byte) 42);
+        break;
+      case INT16:
+        partialRow.addShort(columnName, (short) 43);
+        break;
+      case INT32:
+        partialRow.addInt(columnName, 44);
+        break;
+      case INT64:
+        partialRow.addLong(columnName, 45);
+        break;
+      case UNIXTIME_MICROS:
+        partialRow.addTimestamp(columnName, new Timestamp(1234567890));
+        break;
+      case VARCHAR:
+        partialRow.addVarchar(columnName, "fun with ütf\0");
+        break;
+      case STRING:
+        partialRow.addString(columnName, "fun with ütf\0");
+        break;
+      case BINARY:
+        partialRow.addBinary(columnName, new byte[] { 0, 1, 2, 3, 4 });
+        break;
+      case FLOAT:
+        partialRow.addFloat(columnName, 52.35F);
+        break;
+      case DOUBLE:
+        partialRow.addDouble(columnName, 53.35);
+        break;
+      case BOOL:
+        partialRow.addBoolean(columnName, true);
+        break;
+      case DECIMAL:
+        partialRow.addDecimal(columnName, BigDecimal.valueOf(12345, 3));
+        break;
+      case DATE:
+        partialRow.addDate(columnName, DateUtil.epochDaysToSqlDate(0));
+        break;
       default:
         throw new UnsupportedOperationException();
     }
@@ -502,17 +586,45 @@ public class TestPartialRow {
 
   private void callAddByIndex(PartialRow partialRow, int columnIndex, Type type) {
     switch (type) {
-      case INT8: partialRow.addByte(columnIndex, (byte) 42); break;
-      case INT16: partialRow.addShort(columnIndex, (short) 43); break;
-      case INT32: partialRow.addInt(columnIndex, 44); break;
-      case INT64: partialRow.addLong(columnIndex, 45); break;
-      case UNIXTIME_MICROS: partialRow.addTimestamp(columnIndex, new Timestamp(1234567890)); break;
-      case STRING: partialRow.addString(columnIndex, "fun with ütf\0"); break;
-      case BINARY: partialRow.addBinary(columnIndex, new byte[] { 0, 1, 2, 3, 4 }); break;
-      case FLOAT: partialRow.addFloat(columnIndex, 52.35F); break;
-      case DOUBLE: partialRow.addDouble(columnIndex, 53.35); break;
-      case BOOL: partialRow.addBoolean(columnIndex, true); break;
-      case DECIMAL: partialRow.addDecimal(columnIndex, BigDecimal.valueOf(12345, 3)); break;
+      case INT8:
+        partialRow.addByte(columnIndex, (byte) 42);
+        break;
+      case INT16:
+        partialRow.addShort(columnIndex, (short) 43);
+        break;
+      case INT32:
+        partialRow.addInt(columnIndex, 44);
+        break;
+      case INT64:
+        partialRow.addLong(columnIndex, 45);
+        break;
+      case UNIXTIME_MICROS:
+        partialRow.addTimestamp(columnIndex, new Timestamp(1234567890));
+        break;
+      case VARCHAR:
+        partialRow.addVarchar(columnIndex, "fun with ütf\0");
+        break;
+      case STRING:
+        partialRow.addString(columnIndex, "fun with ütf\0");
+        break;
+      case BINARY:
+        partialRow.addBinary(columnIndex, new byte[] { 0, 1, 2, 3, 4 });
+        break;
+      case FLOAT:
+        partialRow.addFloat(columnIndex, 52.35F);
+        break;
+      case DOUBLE:
+        partialRow.addDouble(columnIndex, 53.35);
+        break;
+      case BOOL:
+        partialRow.addBoolean(columnIndex, true);
+        break;
+      case DECIMAL:
+        partialRow.addDecimal(columnIndex, BigDecimal.valueOf(12345, 3));
+        break;
+      case DATE:
+        partialRow.addDate(columnIndex, DateUtil.epochDaysToSqlDate(0));
+        break;
       default:
         throw new UnsupportedOperationException();
     }
