@@ -22,6 +22,7 @@
 #include <mutex>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -57,7 +58,7 @@
 #include "kudu/rpc/rpc_header.pb.h"
 #include "kudu/security/token.pb.h"
 #include "kudu/tserver/tserver.pb.h"
-#include "kudu/tserver/tserver_service.proxy.h"
+#include "kudu/tserver/tserver_service.proxy.h" // IWYU pragma: keep
 #include "kudu/util/logging.h"
 #include "kudu/util/pb_util.h"
 
@@ -218,7 +219,8 @@ class WriteRpc : public RetriableRpc<RemoteTabletServer, WriteRequestPB, WriteRe
            shared_ptr<Messenger> messenger,
            const string& tablet_id,
            uint64_t propagated_timestamp);
-  virtual ~WriteRpc();
+  ~WriteRpc() override;
+
   string ToString() const override;
 
   const KuduTable* table() const {
@@ -501,6 +503,15 @@ RetriableRpcStatus WriteRpc::AnalyzeResponse(const Status& rpc_cb_status) {
     result.status = StatusFromPB(resp_.error().status());
   }
 
+  // In a multi-client usage scenario, where one client is used for DDL ops
+  // and other is used for DML ops on the same range partition, tablet
+  // entry in metacache may become invalid, rendering it useless for
+  // subsequent operation. This check lets client know about the same.
+  if (result.status.IsInvalidArgument()) {
+    result.result = RetriableRpcStatus::NON_RETRIABLE_ERROR;
+    return result;
+  }
+
   // If we get TABLET_NOT_FOUND, the replica we thought was leader has been deleted.
   if (resp_.has_error() && resp_.error().code() == tserver::TabletServerErrorPB::TABLET_NOT_FOUND) {
     result.result = RetriableRpcStatus::RESOURCE_NOT_FOUND;
@@ -626,7 +637,7 @@ void Batcher::Abort() {
 
   vector<InFlightOp*> to_abort;
   for (InFlightOp* op : ops_) {
-    std::lock_guard<simple_spinlock> l(op->lock_);
+    std::lock_guard l(op->lock_);
     if (op->state == InFlightOp::kBufferedToTabletServer) {
       to_abort.push_back(op);
     }
@@ -655,18 +666,18 @@ Batcher::~Batcher() {
 }
 
 void Batcher::SetTimeout(const MonoDelta& timeout) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   timeout_ = timeout;
 }
 
 
 bool Batcher::HasPendingOperations() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return !ops_.empty();
 }
 
 int Batcher::CountBufferedOperations() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (state_ == kGatheringOps) {
     return ops_.size();
   } else {
@@ -679,7 +690,7 @@ int Batcher::CountBufferedOperations() const {
 void Batcher::CheckForFinishedFlush() {
   sp::shared_ptr<KuduSession> session;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     if (state_ != kFlushing || !ops_.empty()) {
       return;
     }
@@ -711,7 +722,7 @@ MonoTime Batcher::ComputeDeadlineUnlocked() const {
 
 void Batcher::FlushAsync(KuduStatusCallback* cb) {
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     CHECK_EQ(state_, kGatheringOps);
     state_ = kFlushing;
     flush_callback_ = cb;
@@ -765,7 +776,7 @@ Status Batcher::Add(KuduWriteOperation* write_op) {
 void Batcher::AddInFlightOp(InFlightOp* op) {
   DCHECK_EQ(op->state, InFlightOp::kLookingUpTablet);
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   CHECK_EQ(state_, kGatheringOps);
   InsertOrDie(&ops_, op);
   op->sequence_number_ = next_op_sequence_number_++;
@@ -781,7 +792,7 @@ bool Batcher::IsAbortedUnlocked() const {
 }
 
 void Batcher::MarkInFlightOpFailed(InFlightOp* op, const Status& s) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   MarkInFlightOpFailedUnlocked(op, s);
 }
 
@@ -830,7 +841,7 @@ void Batcher::TabletLookupFinished(InFlightOp* op, const Status& s) {
   }
 
   {
-    std::lock_guard<simple_spinlock> l2(op->lock_);
+    std::lock_guard l2(op->lock_);
     CHECK_EQ(op->state, InFlightOp::kLookingUpTablet);
     CHECK(op->tablet != NULL);
 
@@ -871,7 +882,7 @@ void Batcher::FlushBuffersIfReady() {
   // 2. All outstanding ops have finished lookup. Why? To avoid a situation
   //    where ops are flushed one by one as they finish lookup.
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     if (state_ != kFlushing) {
       VLOG(3) << "FlushBuffersIfReady: batcher not yet in flushing state";
       return;
@@ -991,7 +1002,7 @@ void Batcher::ProcessWriteResponse(const WriteRpc& rpc,
   //     calls CheckForFinishedFlush(), and wakes up the client thread
   //     from which the Flush() is being called.
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     for (InFlightOp* op : rpc.ops()) {
       CHECK_EQ(1, ops_.erase(op))
             << "Could not remove op " << op->ToString()
