@@ -512,6 +512,7 @@ Status KuduClient::Data::ListTablesWithInfo(KuduClient* client,
   }
   for (const auto& table : resp.tables()) {
     TableInfo info;
+    if (table.has_id()) info.id = table.id();
     info.table_name = table.name();
     info.live_row_count = table.has_live_row_count() ? table.live_row_count() : 0;
     info.num_tablets = table.has_num_tablets() ? table.num_tablets() : 0;
@@ -768,7 +769,7 @@ void KuduClient::Data::ConnectedToClusterCb(
 
   vector<StatusCallback> cbs;
   {
-    std::lock_guard<simple_spinlock> l(leader_master_lock_);
+    std::lock_guard l(leader_master_lock_);
     if (cred_policy == CredentialsPolicy::PRIMARY_CREDENTIALS) {
       leader_master_rpc_primary_creds_.reset();
       cbs.swap(leader_master_callbacks_primary_creds_);
@@ -936,33 +937,33 @@ void KuduClient::Data::RetrieveAuthzTokenAsync(const KuduTable* table,
 }
 
 HostPort KuduClient::Data::leader_master_hostport() const {
-  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  std::lock_guard l(leader_master_lock_);
   return leader_master_hostport_;
 }
 
 vector<HostPort> KuduClient::Data::master_hostports() const {
-  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  std::lock_guard l(leader_master_lock_);
   return master_hostports_;
 }
 
 string KuduClient::Data::location() const {
-  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  std::lock_guard l(leader_master_lock_);
   return location_;
 }
 
 string KuduClient::Data::cluster_id() const {
-  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  std::lock_guard l(leader_master_lock_);
   return cluster_id_;
 }
 
 shared_ptr<master::MasterServiceProxy> KuduClient::Data::master_proxy() const {
-  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  std::lock_guard l(leader_master_lock_);
   return master_proxy_;
 }
 
 std::shared_ptr<transactions::TxnManagerServiceProxy>
 KuduClient::Data::txn_manager_proxy() const {
-  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  std::lock_guard l(leader_master_lock_);
   return txn_manager_proxy_;
 }
 
