@@ -1277,8 +1277,8 @@ bool MetaCache::LookupEntryByKeyFastPath(const KuduTable* table,
 
   // Stale entries must be re-fetched.
   if (e->stale()) {
-    VLOG(2) << Substitute("Table $0: Stale entry for tablet $1 found, must be re-fetched.",
-                          table->name(), e->tablet()->tablet_id());
+    VLOG(2) << Substitute("Table $0: stale entry must be re-fetched: $1",
+                          table->name(), e->DebugString(table));
     return false;
   }
 
