@@ -14,19 +14,16 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
-#ifndef KUDU_CLIENT_TABLE_ALTERER_INTERNAL_H
-#define KUDU_CLIENT_TABLE_ALTERER_INTERNAL_H
+#pragma once
 
 #include <cstdint>
 #include <map>
 #include <memory>
+#include <optional>
 #include <string>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
-
 #include "kudu/client/client.h"
-#include "kudu/common/partial_row.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/master/master.pb.h"
 #include "kudu/util/monotime.h"
@@ -58,15 +55,12 @@ class KuduTableAlterer::Data {
     // [ADD|DROP|RENAME|ALTER]_COLUMN.
     KuduColumnSpec *spec;
 
-    // Lower and upper bound partition keys. Only set when the StepType is
+    // The Kudu range partition to add or drop. Only set when the StepType is
     // [ADD|DROP]_RANGE_PARTITION.
-    std::unique_ptr<KuduPartialRow> lower_bound;
-    std::unique_ptr<KuduPartialRow> upper_bound;
-    KuduTableCreator::RangePartitionBound lower_bound_type;
-    KuduTableCreator::RangePartitionBound upper_bound_type;
+    std::unique_ptr<KuduRangePartition> range_partition;
 
     // The dimension label for tablet. Only set when the StepType is ADD_RANGE_PARTITION.
-    boost::optional<std::string> dimension_label;
+    std::optional<std::string> dimension_label;
   };
   std::vector<Step> steps_;
 
@@ -74,19 +68,22 @@ class KuduTableAlterer::Data {
 
   bool wait_;
 
-  boost::optional<std::string> rename_to_;
-  boost::optional<std::string> set_owner_to_;
-  boost::optional<std::string> set_comment_to_;
-  boost::optional<int> set_replication_factor_to_;
+  std::optional<std::string> rename_to_;
+  std::optional<std::string> set_owner_to_;
+  std::optional<std::string> set_comment_to_;
+  std::optional<int> set_replication_factor_to_;
 
-  boost::optional<std::map<std::string, std::string>> new_extra_configs_;
+  std::optional<std::map<std::string, std::string>> new_extra_configs_;
 
-  boost::optional<int64_t> disk_size_limit_;
-  boost::optional<int64_t> row_count_limit_;
+  std::optional<int64_t> disk_size_limit_;
+  std::optional<int64_t> row_count_limit_;
 
   // Set to true if there are alter partition steps.
   bool has_alter_partitioning_steps = false;
 
+  // Set to true if a new range with custom hash schema is being added.
+  bool adding_range_with_custom_hash_schema = false;
+
   // Schema of add/drop range partition bound rows.
   const Schema* schema_;
 
@@ -100,5 +97,3 @@ class KuduTableAlterer::Data {
 
 } // namespace client
 } // namespace kudu
-
-#endif
