@@ -252,8 +252,7 @@ Status PartitionSchema::FromPB(
       schema, pb.hash_schema(), &partition_schema->hash_schema_));
 
   const auto custom_ranges_num = pb.custom_hash_schema_ranges_size();
-  vector<HashSchema> range_hash_schemas;
-  range_hash_schemas.resize(custom_ranges_num);
+  vector<HashSchema> range_hash_schemas(custom_ranges_num);
   vector<pair<KuduPartialRow, KuduPartialRow>> range_bounds;
   for (auto i = 0; i < custom_ranges_num; ++i) {
     const auto& range = pb.custom_hash_schema_ranges(i);
@@ -504,12 +503,15 @@ Status PartitionSchema::EncodeRangeBounds(
     const vector<pair<KuduPartialRow, KuduPartialRow>>& range_bounds,
     const vector<HashSchema>& range_hash_schemas,
     const Schema& schema,
-    RangesWithHashSchemas* bounds_with_hash_schemas) const {
+    RangesWithHashSchemas* bounds_with_hash_schemas,
+    bool allow_create_partition) const {
   DCHECK(bounds_with_hash_schemas);
   auto& bounds_whs = *bounds_with_hash_schemas;
   DCHECK(bounds_whs.empty());
   if (range_bounds.empty()) {
-    bounds_whs.emplace_back(RangeWithHashSchema{"", "", hash_schema_});
+    if (!allow_create_partition) {
+      bounds_whs.emplace_back(RangeWithHashSchema{"", "", hash_schema_});
+    }
     return Status::OK();
   }
 
@@ -622,14 +624,16 @@ Status PartitionSchema::CreatePartitions(
     const vector<KuduPartialRow>& split_rows,
     const vector<pair<KuduPartialRow, KuduPartialRow>>& range_bounds,
     const Schema& schema,
-    vector<Partition>* partitions) const {
+    vector<Partition>* partitions,
+    bool allow_empty_partition) const {
   DCHECK(partitions);
 
   RETURN_NOT_OK(CheckRangeSchema(schema));
 
   RangesWithHashSchemas bounds_with_hash_schemas;
   RETURN_NOT_OK(EncodeRangeBounds(range_bounds, {}, schema,
-                                  &bounds_with_hash_schemas));
+                                  &bounds_with_hash_schemas,
+                                  allow_empty_partition));
   vector<string> splits;
   RETURN_NOT_OK(EncodeRangeSplits(split_rows, schema, &splits));
   RETURN_NOT_OK(SplitRangeBounds(schema, splits, &bounds_with_hash_schemas));
@@ -856,7 +860,7 @@ Status PartitionSchema::DecodeRangeKey(Slice* encoded_key,
     if (column.type_info()->physical_type() == BINARY) {
       // Copy cell value into the 'partial_row', because in the decoding process above, we just make
       // row data a pointer to the memory allocated by arena.
-      partial_row->Set(column_idx, cont_row.cell_ptr(column_idx));
+      RETURN_NOT_OK(partial_row->Set(column_idx, cont_row.cell_ptr(column_idx)));
     }
   }
   if (!encoded_key->empty()) {
