@@ -24,6 +24,7 @@
 #include <mutex>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <vector>
 
 #include <gflags/gflags.h>
@@ -144,18 +145,18 @@ Peer::Peer(RaftPeerPB peer_pb,
 
 void Peer::Init() {
   {
-    std::lock_guard<simple_spinlock> l(peer_lock_);
+    std::lock_guard l(peer_lock_);
     queue_->TrackPeer(peer_pb_);
   }
 
   // Capture a weak_ptr reference into the functor so it can safely handle
   // outliving the peer.
-  weak_ptr<Peer> w = shared_from_this();
+  weak_ptr<Peer> w_this = shared_from_this();
   heartbeater_ = PeriodicTimer::Create(
       messenger_,
-      [w]() {
-        if (auto p = w.lock()) {
-          p->SignalRequest(true);
+      [w_this = std::move(w_this)]() {
+        if (auto p = w_this.lock()) {
+          WARN_NOT_OK(p->SignalRequest(true), "SignalRequest failed");
         }
       },
       MonoDelta::FromMilliseconds(FLAGS_raft_heartbeat_interval_ms));
@@ -169,7 +170,7 @@ Status Peer::SignalRequest(bool even_if_queue_empty) {
   // the implementation of SendNextRequest() checks for 'closed_' and
   // 'request_pending_' on its own.
   if (PREDICT_FALSE(closed_)) {
-    return Status::IllegalState("Peer was closed.");
+    return Status::IllegalState("peer closed");
   }
 
   // Only allow one request at a time. No sense waking up the
@@ -180,8 +181,8 @@ Status Peer::SignalRequest(bool even_if_queue_empty) {
 
   // Capture a weak_ptr reference into the submitted functor so that we can
   // safely handle the functor outliving its peer.
-  weak_ptr<Peer> w_this = shared_from_this();
-  return raft_pool_token_->Submit([even_if_queue_empty, w_this]() {
+  weak_ptr<Peer> w_this(shared_from_this());
+  return raft_pool_token_->Submit([even_if_queue_empty, w_this = std::move(w_this)]() {
     if (auto p = w_this.lock()) {
       p->SendNextRequest(even_if_queue_empty);
     }
@@ -334,7 +335,7 @@ void Peer::StartElection() {
 
 void Peer::ProcessResponse() {
   // Note: This method runs on the reactor thread.
-  std::unique_lock<simple_spinlock> lock(peer_lock_);
+  std::lock_guard lock(peer_lock_);
   if (PREDICT_FALSE(closed_)) {
     return;
   }
@@ -416,7 +417,7 @@ void Peer::DoProcessResponse() {
       queue_->ResponseFromPeer(peer_pb_.permanent_uuid(), response_);
 
   {
-    std::unique_lock<simple_spinlock> lock(peer_lock_);
+    std::lock_guard lock(peer_lock_);
     CHECK(request_pending_);
     failed_attempts_ = 0;
     request_pending_ = false;
@@ -457,12 +458,13 @@ void Peer::ProcessTabletCopyResponse() {
     queue_->UpdatePeerStatus(peer_pb_.permanent_uuid(), PeerStatus::OK, Status::OK());
   } else if (!tc_response_.has_error() ||
               tc_response_.error().code() != TabletServerErrorPB::TabletServerErrorPB::THROTTLED) {
+    const auto& response_str = controller_status.ok()
+        ? SecureShortDebugString(tc_response_) : controller_status.ToString();
+    lock.unlock();
     // THROTTLED is a common response after a tserver with many replicas fails;
     // logging it would generate a great deal of log spam.
     LOG_WITH_PREFIX_UNLOCKED(WARNING) << "Unable to start Tablet Copy on peer: "
-                                      << (controller_status.ok() ?
-                                          SecureShortDebugString(tc_response_) :
-                                          controller_status.ToString());
+                                      << response_str;
   }
 }
 
@@ -495,7 +497,7 @@ void Peer::ProcessResponseErrorUnlocked(const Status& status) {
 }
 
 bool Peer::CreateProxyIfNeeded() {
-  std::lock_guard<simple_spinlock> l(proxy_lock_);
+  std::lock_guard l(proxy_lock_);
   if (!proxy_) {
     unique_ptr<PeerProxy> peer_proxy;
     Status s = DCHECK_NOTNULL(peer_proxy_factory_)->NewProxy(peer_pb_, &peer_proxy);
@@ -521,7 +523,7 @@ void Peer::Close() {
     return;
   }
   {
-    std::lock_guard<simple_spinlock> lock(peer_lock_);
+    std::lock_guard lock(peer_lock_);
     closed_ = true;
   }
   VLOG_WITH_PREFIX_UNLOCKED(1) << "Closing peer: " << peer_pb_.permanent_uuid();
@@ -536,7 +538,7 @@ Peer::~Peer() {
   }
 
   // We don't own the ops (the queue does).
-  request_.mutable_ops()->ExtractSubrange(0, request_.ops_size(), nullptr);
+  request_.mutable_ops()->UnsafeArenaExtractSubrange(0, request_.ops_size(), nullptr);
 }
 
 RpcPeerProxy::RpcPeerProxy(HostPort hostport,
