@@ -52,7 +52,6 @@
 #include "kudu/consensus/ref_counted_replicate.h"
 #include "kudu/consensus/time_manager.h"
 #include "kudu/fs/fs_manager.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/util/async_util.h"
 #include "kudu/util/metrics.h"
@@ -96,7 +95,7 @@ class ConsensusQueueTest : public KuduTest {
         allow_status_msg_for_failed_peer_(false) {
   }
 
-  virtual void SetUp() OVERRIDE {
+  void SetUp() override {
     KuduTest::SetUp();
     fs_manager_.reset(new FsManager(env_, FsManagerOpts(GetTestPath("fs_root"))));
     ASSERT_OK(fs_manager_->CreateInitialFileSystemLayout());
@@ -131,8 +130,8 @@ class ConsensusQueueTest : public KuduTest {
         &allow_status_msg_for_failed_peer_));
   }
 
-  virtual void TearDown() OVERRIDE {
-    log_->WaitUntilAllFlushed();
+  void TearDown() override {
+    ASSERT_OK(log_->WaitUntilAllFlushed());
     queue_->Close();
   }
 
@@ -451,7 +450,7 @@ TEST_F(ConsensusQueueTest, TestStartTrackingAfterStart) {
   ASSERT_EQ(0, request.ops_size());
 
   // extract the ops from the request to avoid double free
-  request.mutable_ops()->ExtractSubrange(0, request.ops_size(), nullptr);
+  request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops_size(), nullptr);
 }
 
 // Tests that the peers gets the messages pages, with the size of a page
@@ -500,7 +499,7 @@ TEST_F(ConsensusQueueTest, TestGetPagedMessages) {
 
   SCOPED_CLEANUP({
     // Extract the ops from the request to avoid double free.
-    request.mutable_ops()->ExtractSubrange(0, request.ops_size(), nullptr);
+    request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops_size(), nullptr);
   });
 
   OpId last;
@@ -592,7 +591,7 @@ TEST_F(ConsensusQueueTest, TestPeersDontAckBeyondWatermarks) {
   ASSERT_EQ(queue_->GetAllReplicatedIndex(), expected.index());
 
   // extract the ops from the request to avoid double free
-  request.mutable_ops()->ExtractSubrange(0, request.ops_size(), nullptr);
+  request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops_size(), nullptr);
 }
 
 TEST_F(ConsensusQueueTest, TestQueueAdvancesCommittedIndex) {
@@ -791,7 +790,7 @@ TEST_F(ConsensusQueueTest, TestQueueLoadsOperationsForPeer) {
   ASSERT_EQ(request.ops_size(), 50);
 
   // The messages still belong to the queue so we have to release them.
-  request.mutable_ops()->ExtractSubrange(0, request.ops().size(), nullptr);
+  request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops().size(), nullptr);
 }
 
 // This tests that the queue is able to handle operation overwriting, i.e. when a
@@ -900,7 +899,7 @@ TEST_F(ConsensusQueueTest, TestQueueHandlesOperationOverwriting) {
   ASSERT_EQ(queue_->GetAllReplicatedIndex(), 21);
 
   // The messages still belong to the queue so we have to release them.
-  request.mutable_ops()->ExtractSubrange(0, request.ops().size(), nullptr);
+  request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops().size(), nullptr);
 }
 
 // Test for a bug where we wouldn't move any watermark back, when overwriting
@@ -911,7 +910,7 @@ TEST_F(ConsensusQueueTest, TestQueueMovesWatermarksBackward) {
   // Append a bunch of messages and update as if they were also appeneded to the leader.
   queue_->UpdateLastIndexAppendedToLeader(10);
   AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 10);
-  log_->WaitUntilAllFlushed();
+  ASSERT_OK(log_->WaitUntilAllFlushed());
 
   // Now rewrite some of the operations and wait for the log to append.
   Synchronizer synch;
@@ -996,14 +995,14 @@ TEST_F(ConsensusQueueTest, TestOnlyAdvancesWatermarkWhenPeerHasAPrefixOfOurLog)
 
   for (int i = 31; i <= 53; i++) {
     if (i <= 45) {
-      AppendReplicateMsg(72, i, 1024);
+      ASSERT_OK(AppendReplicateMsg(72, i, 1024));
       continue;
     }
     if (i <= 51) {
-      AppendReplicateMsg(73, i, 1024);
+      ASSERT_OK(AppendReplicateMsg(73, i, 1024));
       continue;
     }
-    AppendReplicateMsg(76, i, 1024);
+    ASSERT_OK(AppendReplicateMsg(76, i, 1024));
   }
 
   WaitForLocalPeerToAckIndex(53);
@@ -1031,7 +1030,7 @@ TEST_F(ConsensusQueueTest, TestOnlyAdvancesWatermarkWhenPeerHasAPrefixOfOurLog)
 
   // Another request for this peer should get another page of messages. Still not
   // on the queue's term (and thus without advancing watermarks).
-  request.mutable_ops()->ExtractSubrange(0, request.ops().size(), nullptr);
+  request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops().size(), nullptr);
   ASSERT_OK(queue_->RequestForPeer(kPeerUuid, &request, &refs, &needs_tablet_copy));
   ASSERT_FALSE(needs_tablet_copy);
   ASSERT_EQ(request.ops_size(), 9);
@@ -1049,7 +1048,7 @@ TEST_F(ConsensusQueueTest, TestOnlyAdvancesWatermarkWhenPeerHasAPrefixOfOurLog)
 
   // The last page of request should overwrite the peer's operations and the
   // response should finally advance the watermarks.
-  request.mutable_ops()->ExtractSubrange(0, request.ops().size(), nullptr);
+  request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops().size(), nullptr);
   ASSERT_OK(queue_->RequestForPeer(kPeerUuid, &request, &refs, &needs_tablet_copy));
   ASSERT_FALSE(needs_tablet_copy);
   ASSERT_EQ(request.ops_size(), 4);
@@ -1064,7 +1063,7 @@ TEST_F(ConsensusQueueTest, TestOnlyAdvancesWatermarkWhenPeerHasAPrefixOfOurLog)
   ASSERT_EQ(queue_->GetMajorityReplicatedIndexForTests(), expected_majority_replicated);
   ASSERT_EQ(queue_->GetAllReplicatedIndex(), expected_all_replicated);
 
-  request.mutable_ops()->ExtractSubrange(0, request.ops().size(), nullptr);
+  request.mutable_ops()->UnsafeArenaExtractSubrange(0, request.ops().size(), nullptr);
 }
 
 // Test that Tablet Copy is triggered when a "tablet not found" error occurs.
