@@ -29,7 +29,6 @@
 #include <utility>
 
 #include <gflags/gflags.h>
-#include <google/protobuf/stubs/port.h>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/timestamp.h"
@@ -152,7 +151,10 @@ PeerMessageQueue::TrackedPeer::TrackedPeer(RaftPeerPB peer_pb)
       wal_catchup_possible(true),
       remote_server_quiescing(false),
       last_overall_health_status(HealthReportPB::UNKNOWN),
-      status_log_throttler(std::make_shared<logging::LogThrottler>()),
+      status_log_throttler_lag(
+          std::make_shared<logging::LogThrottler>(3, "TrackedPeer Lag")),
+      status_log_throttler_wal_gc(
+          std::make_shared<logging::LogThrottler>(60, "TrackedPeer WAL GC")),
       last_seen_term_(0) {
 }
 
@@ -216,7 +218,7 @@ PeerMessageQueue::PeerMessageQueue(const scoped_refptr<MetricEntity>& metric_ent
 void PeerMessageQueue::SetLeaderMode(int64_t committed_index,
                                      int64_t current_term,
                                      const RaftConfigPB& active_config) {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   if (current_term != queue_state_.current_term) {
     CHECK_GT(current_term, queue_state_.current_term) << "Terms should only increase";
     queue_state_.first_index_in_current_term.reset();
@@ -245,7 +247,7 @@ void PeerMessageQueue::SetLeaderMode(int64_t committed_index,
 }
 
 void PeerMessageQueue::SetNonLeaderMode(const RaftConfigPB& active_config) {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   queue_state_.active_config.reset(new RaftConfigPB(active_config));
   queue_state_.mode = NON_LEADER;
   queue_state_.majority_size_ = -1;
@@ -262,7 +264,7 @@ void PeerMessageQueue::SetNonLeaderMode(const RaftConfigPB& active_config) {
 }
 
 void PeerMessageQueue::TrackPeer(const RaftPeerPB& peer_pb) {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   TrackPeerUnlocked(peer_pb);
 }
 
@@ -292,7 +294,7 @@ void PeerMessageQueue::TrackPeerUnlocked(const RaftPeerPB& peer_pb) {
 }
 
 void PeerMessageQueue::UntrackPeer(const string& uuid) {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   UntrackPeerUnlocked(uuid);
 }
 
@@ -330,7 +332,7 @@ void PeerMessageQueue::TrackLocalPeerUnlocked() {
 
 unordered_map<string, HealthReportPB> PeerMessageQueue::ReportHealthOfPeers() const {
   unordered_map<string, HealthReportPB> reports;
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   for (const auto& entry : peers_map_) {
     const string& peer_uuid = entry.first;
     const TrackedPeer* peer = entry.second;
@@ -383,7 +385,7 @@ void PeerMessageQueue::LocalPeerAppendFinished(const OpId& id,
   *fake_response.mutable_status()->mutable_last_received() = id;
   *fake_response.mutable_status()->mutable_last_received_current_leader() = id;
   {
-    std::lock_guard<simple_spinlock> lock(queue_lock_);
+    std::lock_guard lock(queue_lock_);
     fake_response.mutable_status()->set_last_committed_idx(queue_state_.committed_index);
   }
   ResponseFromPeer(local_peer_pb_.permanent_uuid(), fake_response);
@@ -443,7 +445,7 @@ Status PeerMessageQueue::AppendOperations(vector<ReplicateRefPtr> msgs,
       }));
   lock.lock();
   DCHECK(last_id.IsInitialized());
-  queue_state_.last_appended = last_id;
+  queue_state_.last_appended = std::move(last_id);
   UpdateMetricsUnlocked();
 
   return Status::OK();
@@ -457,7 +459,7 @@ void PeerMessageQueue::TruncateOpsAfter(int64_t index) {
                               LogPrefixUnlocked(),
                               index));
   {
-    std::unique_lock<simple_spinlock> lock(queue_lock_);
+    std::lock_guard lock(queue_lock_);
     DCHECK(op.IsInitialized());
     queue_state_.last_appended = op;
   }
@@ -465,13 +467,13 @@ void PeerMessageQueue::TruncateOpsAfter(int64_t index) {
 }
 
 OpId PeerMessageQueue::GetLastOpIdInLog() const {
-  std::unique_lock<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   DCHECK(queue_state_.last_appended.IsInitialized());
   return queue_state_.last_appended;
 }
 
 OpId PeerMessageQueue::GetNextOpId() const {
-  std::unique_lock<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   DCHECK(queue_state_.last_appended.IsInitialized());
   return MakeOpId(queue_state_.current_term,
                   queue_state_.last_appended.index() + 1);
@@ -651,7 +653,7 @@ Status PeerMessageQueue::RequestForPeer(const string& uuid,
   TrackedPeer peer_copy;
   MonoDelta unreachable_time;
   {
-    std::lock_guard<simple_spinlock> lock(queue_lock_);
+    std::lock_guard lock(queue_lock_);
     DCHECK_EQ(queue_state_.state, kQueueOpen);
     DCHECK_NE(uuid, local_peer_pb_.permanent_uuid());
 
@@ -663,7 +665,7 @@ Status PeerMessageQueue::RequestForPeer(const string& uuid,
     peer_copy = *peer;
 
     // Clear the requests without deleting the entries, as they may be in use by other peers.
-    request->mutable_ops()->ExtractSubrange(0, request->ops_size(), nullptr);
+    request->mutable_ops()->UnsafeArenaExtractSubrange(0, request->ops_size(), nullptr);
 
     // This is initialized to the queue's last appended op but gets set to the id of the
     // log entry preceding the first one in 'messages' if messages are found for the peer.
@@ -681,7 +683,7 @@ Status PeerMessageQueue::RequestForPeer(const string& uuid,
   bool wal_catchup_progress = false;
   bool wal_catchup_failure = false;
   SCOPED_CLEANUP({
-      std::lock_guard<simple_spinlock> lock(queue_lock_);
+      std::lock_guard lock(queue_lock_);
       TrackedPeer* peer = FindPtrOrNull(peers_map_, uuid);
       if (PREDICT_FALSE(peer == nullptr || queue_state_.mode == NON_LEADER)) {
         VLOG(1) << LogPrefixUnlocked() << "peer " << uuid
@@ -735,7 +737,7 @@ Status PeerMessageQueue::RequestForPeer(const string& uuid,
       // the leader has GCed its logs. The follower replica will hang around
       // for a while until it's evicted.
       if (PREDICT_TRUE(s.IsNotFound())) {
-        KLOG_EVERY_N_SECS_THROTTLER(INFO, 60, *peer_copy.status_log_throttler, "logs_gced")
+        KLOG_THROTTLER(INFO, *peer_copy.status_log_throttler_wal_gc)
             << LogPrefixUnlocked()
             << Substitute("The logs necessary to catch up peer $0 have been "
                           "garbage collected. The follower will never be able "
@@ -781,7 +783,7 @@ Status PeerMessageQueue::RequestForPeer(const string& uuid,
   if (request->ops_size() > 0) {
     int64_t last_op_sent = request->ops(request->ops_size() - 1).id().index();
     if (last_op_sent < request->committed_index()) {
-      KLOG_EVERY_N_SECS_THROTTLER(INFO, 3, *peer_copy.status_log_throttler, "lagging")
+      KLOG_THROTTLER(INFO, *peer_copy.status_log_throttler_lag)
           << LogPrefixUnlocked() << "Peer " << uuid << " is lagging by at least "
           << (request->committed_index() - last_op_sent)
           << " ops behind the committed index " << THROTTLE_MSG;
@@ -821,7 +823,7 @@ Status PeerMessageQueue::GetTabletCopyRequestForPeer(const string& uuid,
   TrackedPeer* peer = nullptr;
   int64_t current_term;
   {
-    std::lock_guard<simple_spinlock> lock(queue_lock_);
+    std::lock_guard lock(queue_lock_);
     DCHECK_EQ(queue_state_.state, kQueueOpen);
     DCHECK_NE(uuid, local_peer_pb_.permanent_uuid());
     peer = FindPtrOrNull(peers_map_, uuid);
@@ -927,19 +929,19 @@ void PeerMessageQueue::AdvanceQueueWatermark(const char* type,
 
 void PeerMessageQueue::BeginWatchForSuccessor(
     const optional<string>& successor_uuid) {
-  std::lock_guard<simple_spinlock> l(queue_lock_);
+  std::lock_guard l(queue_lock_);
   successor_watch_in_progress_ = true;
   designated_successor_uuid_ = successor_uuid;
 }
 
 void PeerMessageQueue::EndWatchForSuccessor() {
-  std::lock_guard<simple_spinlock> l(queue_lock_);
+  std::lock_guard l(queue_lock_);
   successor_watch_in_progress_ = false;
 }
 
 void PeerMessageQueue::UpdateFollowerWatermarks(int64_t committed_index,
                                                 int64_t all_replicated_index) {
-  std::lock_guard<simple_spinlock> l(queue_lock_);
+  std::lock_guard l(queue_lock_);
   DCHECK_EQ(queue_state_.mode, NON_LEADER);
   queue_state_.committed_index = committed_index;
   queue_state_.all_replicated_index = all_replicated_index;
@@ -947,7 +949,7 @@ void PeerMessageQueue::UpdateFollowerWatermarks(int64_t committed_index,
 }
 
 void PeerMessageQueue::UpdateLastIndexAppendedToLeader(int64_t last_idx_appended_to_leader) {
-  std::lock_guard<simple_spinlock> l(queue_lock_);
+  std::lock_guard l(queue_lock_);
   DCHECK_EQ(queue_state_.mode, NON_LEADER);
   queue_state_.last_idx_appended_to_leader = last_idx_appended_to_leader;
   UpdateLagMetricsUnlocked();
@@ -956,7 +958,7 @@ void PeerMessageQueue::UpdateLastIndexAppendedToLeader(int64_t last_idx_appended
 void PeerMessageQueue::UpdatePeerStatus(const string& peer_uuid,
                                         PeerStatus ps,
                                         const Status& status) {
-  std::unique_lock<simple_spinlock> l(queue_lock_);
+  std::lock_guard l(queue_lock_);
   TrackedPeer* peer = FindPtrOrNull(peers_map_, peer_uuid);
   if (PREDICT_FALSE(peer == nullptr || queue_state_.mode == NON_LEADER)) {
     VLOG(1) << LogPrefixUnlocked() << "peer " << peer_uuid
@@ -1010,7 +1012,7 @@ void PeerMessageQueue::UpdatePeerStatus(const string& peer_uuid,
 }
 
 void PeerMessageQueue::UpdateExchangeStatus(TrackedPeer* peer,
-                                            const TrackedPeer& prev_peer_state,
+                                            PeerStatus prev_last_exchange_status,
                                             const ConsensusResponsePB& response,
                                             bool* lmp_mismatch) {
   DCHECK(queue_lock_.is_locked());
@@ -1029,7 +1031,7 @@ void PeerMessageQueue::UpdateExchangeStatus(TrackedPeer* peer,
     case ConsensusErrorPB::PRECEDING_ENTRY_DIDNT_MATCH:
       peer->last_exchange_status = PeerStatus::LMP_MISMATCH;
       DCHECK(status.has_last_received());
-      if (prev_peer_state.last_exchange_status == PeerStatus::NEW) {
+      if (prev_last_exchange_status == PeerStatus::NEW) {
         LOG_WITH_PREFIX_UNLOCKED(INFO) << "Connected to new peer: " << peer->ToString();
       } else {
         LOG_WITH_PREFIX_UNLOCKED(INFO) << "Got LMP mismatch error from peer: "
@@ -1055,11 +1057,12 @@ void PeerMessageQueue::UpdateExchangeStatus(TrackedPeer* peer,
   }
 }
 
-void PeerMessageQueue::PromoteIfNeeded(TrackedPeer* peer, const TrackedPeer& prev_peer_state,
-                                       const ConsensusStatusPB& status) {
+void PeerMessageQueue::PromoteIfNeeded(const TrackedPeer& peer,
+                                       const ConsensusStatusPB& status,
+                                       int64_t prev_last_received_index) {
   DCHECK(queue_lock_.is_locked());
   if (queue_state_.mode != PeerMessageQueue::LEADER ||
-      peer->last_exchange_status != PeerStatus::OK) {
+      peer.last_exchange_status != PeerStatus::OK) {
     return;
   }
 
@@ -1067,7 +1070,7 @@ void PeerMessageQueue::PromoteIfNeeded(TrackedPeer* peer, const TrackedPeer& pre
   // TrackedPeer data structure.
   RaftPeerPB* peer_pb;
   Status s = GetRaftConfigMember(DCHECK_NOTNULL(queue_state_.active_config.get()),
-                                 peer->uuid(), &peer_pb);
+                                 peer.uuid(), &peer_pb);
   if (s.ok() &&
       peer_pb->member_type() == RaftPeerPB::NON_VOTER &&
       peer_pb->attrs().promote()) {
@@ -1078,20 +1081,23 @@ void PeerMessageQueue::PromoteIfNeeded(TrackedPeer* peer, const TrackedPeer& pre
 
     // If we had never previously contacted this peer, wait until the second
     // time we contact them to try to promote them.
-    if (prev_peer_state.last_received.index() == 0) return;
+    if (prev_last_received_index == 0) {
+      return;
+    }
 
-    int64_t last_batch_size =
-        std::max<int64_t>(0, peer->last_received.index() - prev_peer_state.last_received.index());
-    bool peer_caught_up =
-        !OpIdEquals(status.last_received_current_leader(), MinimumOpId()) &&
+    const int64_t last_batch_size = std::max<int64_t>(
+        0, peer.last_received.index() - prev_last_received_index);
+    if (status.last_received_current_leader() == MinimumOpId() ||
         status.last_received_current_leader().index() + last_batch_size
-            >= queue_state_.committed_index;
-    if (!peer_caught_up) return;
+            < queue_state_.committed_index) {
+      // The peer hasn't caught up yet.
+      return;
+    }
 
     // TODO(mpercy): Implement a SafeToPromote() check to ensure that we only
     // try to promote a NON_VOTER to VOTER if we will be able to commit the
     // resulting config change operation.
-    NotifyObserversOfPeerToPromote(peer->uuid());
+    NotifyObserversOfPeerToPromote(peer.uuid());
   }
 }
 
@@ -1127,10 +1133,9 @@ void PeerMessageQueue::TransferLeadershipIfNeeded(const TrackedPeer& peer,
     return;
   }
 
-  bool peer_caught_up =
-      !OpIdEquals(status.last_received_current_leader(), MinimumOpId()) &&
-      OpIdEquals(status.last_received_current_leader(), queue_state_.last_appended);
-  if (!peer_caught_up) {
+  if (status.last_received_current_leader() == MinimumOpId() ||
+      status.last_received_current_leader() != queue_state_.last_appended) {
+    // The peer hasn't caught up yet.
     return;
   }
 
@@ -1150,7 +1155,7 @@ bool PeerMessageQueue::ResponseFromPeer(const std::string& peer_uuid,
   optional<int64_t> updated_commit_index;
   Mode mode_copy;
   {
-    std::lock_guard<simple_spinlock> scoped_lock(queue_lock_);
+    std::lock_guard scoped_lock(queue_lock_);
 
     TrackedPeer* peer = FindPtrOrNull(peers_map_, peer_uuid);
     if (PREDICT_FALSE(queue_state_.state != kQueueOpen || peer == nullptr)) {
@@ -1176,14 +1181,16 @@ bool PeerMessageQueue::ResponseFromPeer(const std::string& peer_uuid,
     DCHECK(status.has_last_received_current_leader());
     DCHECK(status.has_last_committed_idx());
 
-    // Take a snapshot of the previously-recorded peer state.
-    const TrackedPeer prev_peer_state = *peer;
+    // Take a snapshot of a few fields from the previously-recorded peer state.
+    const OpId peer_last_received_op_id = peer->last_received;
+    const PeerStatus peer_last_exchange_status = peer->last_exchange_status;
 
     // Update the peer's last exchange status based on the response.
     // In this case, if there is a log matching property (LMP) mismatch, we
     // want to immediately send another request as we attempt to sync the log
     // offset between the local leader and the remote peer.
-    UpdateExchangeStatus(peer, prev_peer_state, response, &send_more_immediately);
+    UpdateExchangeStatus(
+        peer, peer_last_exchange_status, response, &send_more_immediately);
 
     // If the peer is hosted on a server that is quiescing, note that now.
     peer->remote_server_quiescing = response.has_server_quiescing() &&
@@ -1202,16 +1209,15 @@ bool PeerMessageQueue::ResponseFromPeer(const std::string& peer_uuid,
       peer->next_index = peer->last_received.index() + 1;
 
       // Check if the peer is a NON_VOTER candidate ready for promotion.
-      PromoteIfNeeded(peer, prev_peer_state, status);
+      PromoteIfNeeded(*peer, status, peer_last_received_op_id.index());
 
       TransferLeadershipIfNeeded(*peer, status);
-    } else if (!OpIdEquals(status.last_received_current_leader(), MinimumOpId())) {
+    } else if (status.last_received_current_leader() != MinimumOpId()) {
       // Their log may have diverged from ours, however we are in the process
       // of replicating our ops to them, so continue doing so. Eventually, we
       // will cause the divergent entry in their log to be overwritten.
       peer->last_received = status.last_received_current_leader();
       peer->next_index = peer->last_received.index() + 1;
-
     } else {
       // The peer is divergent and they have not (successfully) received
       // anything from us yet. Start sending from their last committed index.
@@ -1260,7 +1266,7 @@ bool PeerMessageQueue::ResponseFromPeer(const std::string& peer_uuid,
       // Advance the majority replicated index.
       AdvanceQueueWatermark("majority_replicated",
                             &queue_state_.majority_replicated_index,
-                            /*replicated_before=*/ prev_peer_state.last_received,
+                            /*replicated_before=*/ peer_last_received_op_id,
                             /*replicated_after=*/ peer->last_received,
                             /*num_peers_required=*/ queue_state_.majority_size_,
                             VOTER_REPLICAS,
@@ -1269,7 +1275,7 @@ bool PeerMessageQueue::ResponseFromPeer(const std::string& peer_uuid,
       // Advance the all replicated index.
       AdvanceQueueWatermark("all_replicated",
                             &queue_state_.all_replicated_index,
-                            /*replicated_before=*/ prev_peer_state.last_received,
+                            /*replicated_before=*/ peer_last_received_op_id,
                             /*replicated_after=*/ peer->last_received,
                             /*num_peers_required=*/ peers_map_.size(),
                             ALL_REPLICAS,
@@ -1332,34 +1338,34 @@ bool PeerMessageQueue::ResponseFromPeer(const std::string& peer_uuid,
 }
 
 PeerMessageQueue::TrackedPeer PeerMessageQueue::GetTrackedPeerForTests(const string& uuid) {
-  std::lock_guard<simple_spinlock> scoped_lock(queue_lock_);
+  std::lock_guard scoped_lock(queue_lock_);
   TrackedPeer* tracked = FindOrDie(peers_map_, uuid);
   return *tracked;
 }
 
 int64_t PeerMessageQueue::GetAllReplicatedIndex() const {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   return queue_state_.all_replicated_index;
 }
 
 int64_t PeerMessageQueue::GetCommittedIndex() const {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   return queue_state_.committed_index;
 }
 
 bool PeerMessageQueue::IsCommittedIndexInCurrentTerm() const {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   return queue_state_.first_index_in_current_term.has_value() &&
       queue_state_.committed_index >= *queue_state_.first_index_in_current_term;
 }
 
 bool PeerMessageQueue::IsInLeaderMode() const {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   return queue_state_.mode == Mode::LEADER;
 }
 
 int64_t PeerMessageQueue::GetMajorityReplicatedIndexForTests() const {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   return queue_state_.majority_replicated_index;
 }
 
@@ -1386,7 +1392,7 @@ void PeerMessageQueue::UpdateLagMetricsUnlocked() {
 }
 
 void PeerMessageQueue::DumpToStrings(vector<string>* lines) const {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   DumpToStringsUnlocked(lines);
 }
 
@@ -1404,7 +1410,7 @@ void PeerMessageQueue::DumpToStringsUnlocked(vector<string>* lines) const {
 void PeerMessageQueue::DumpToHtml(std::ostream& out) const {
   using std::endl;
 
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   out << "<h3>Watermarks</h3>" << endl;
   out << "<table>" << endl;
   out << "  <tr><th>Peer</th><th>Watermark</th></tr>" << endl;
@@ -1428,7 +1434,7 @@ void PeerMessageQueue::ClearUnlocked() {
 void PeerMessageQueue::Close() {
   raft_pool_observers_token_->Shutdown();
   {
-    std::lock_guard<simple_spinlock> lock(queue_lock_);
+    std::lock_guard lock(queue_lock_);
     ClearUnlocked();
   }
   log_cache_.Clear();
@@ -1441,7 +1447,7 @@ int64_t PeerMessageQueue::GetQueuedOperationsSizeBytesForTests() const {
 string PeerMessageQueue::ToString() const {
   // Even though metrics are thread-safe obtain the lock so that we get
   // a "consistent" snapshot of the metrics.
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   return ToStringUnlocked();
 }
 
@@ -1454,28 +1460,27 @@ string PeerMessageQueue::ToStringUnlocked() const {
 }
 
 void PeerMessageQueue::RegisterObserver(PeerMessageQueueObserver* observer) {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+  std::lock_guard lock(queue_lock_);
   auto iter = std::find(observers_.begin(), observers_.end(), observer);
   if (iter == observers_.end()) {
     observers_.push_back(observer);
   }
 }
 
-Status PeerMessageQueue::UnRegisterObserver(PeerMessageQueueObserver* observer) {
-  std::lock_guard<simple_spinlock> lock(queue_lock_);
+void PeerMessageQueue::UnRegisterObserver(PeerMessageQueueObserver* observer) {
+  std::lock_guard lock(queue_lock_);
   auto iter = std::find(observers_.begin(), observers_.end(), observer);
   if (iter == observers_.end()) {
-    return Status::NotFound("Can't find observer.");
+    return;
   }
   observers_.erase(iter);
-  return Status::OK();
 }
 
 bool PeerMessageQueue::IsOpInLog(const OpId& desired_op) const {
   OpId log_op;
   Status s = log_cache_.LookupOpId(desired_op.index(), &log_op);
   if (PREDICT_TRUE(s.ok())) {
-    return OpIdEquals(desired_op, log_op);
+    return desired_op == log_op;
   }
   if (PREDICT_TRUE(s.IsNotFound() || s.IsIncomplete())) {
     return false;
@@ -1552,7 +1557,7 @@ void PeerMessageQueue::NotifyObserversTask(
   MAYBE_INJECT_RANDOM_LATENCY(FLAGS_consensus_inject_latency_ms_in_notifications);
   std::vector<PeerMessageQueueObserver*> observers_copy;
   {
-    std::lock_guard<simple_spinlock> lock(queue_lock_);
+    std::lock_guard lock(queue_lock_);
     observers_copy = observers_;
   }
   for (PeerMessageQueueObserver* observer : observers_copy) {
