@@ -16,12 +16,14 @@
 // under the License.
 
 #include <algorithm>
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <memory>
 #include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
@@ -134,17 +136,29 @@ class RaftConsensusQuorumTest : public KuduTest {
       opts.wal_root = test_path;
       opts.data_roots = { test_path };
       unique_ptr<FsManager> fs_manager(new FsManager(env_, opts));
-      string server_key;
-      string server_key_iv;
-      string server_key_version;
-      GetEncryptionKey(&server_key, &server_key_iv, &server_key_version);
-      if (server_key.empty()) {
+      string tenant_name;
+      string tenant_id;
+      string encryption_key;
+      string encryption_key_iv;
+      string encryption_key_version;
+      GetEncryptionKey(&tenant_name, &tenant_id, &encryption_key,
+                       &encryption_key_iv, &encryption_key_version);
+      if (tenant_name.empty() && encryption_key.empty()) {
         RETURN_NOT_OK(fs_manager->CreateInitialFileSystemLayout());
+      } else if (tenant_name.empty()) {
+        RETURN_NOT_OK(fs_manager->CreateInitialFileSystemLayout(nullopt,
+                                                                nullopt,
+                                                                nullopt,
+                                                                encryption_key,
+                                                                encryption_key_iv,
+                                                                encryption_key_version));
       } else {
         RETURN_NOT_OK(fs_manager->CreateInitialFileSystemLayout(nullopt,
-                                                                server_key,
-                                                                server_key_iv,
-                                                                server_key_version));
+                                                                tenant_name,
+                                                                tenant_id,
+                                                                encryption_key,
+                                                                encryption_key_iv,
+                                                                encryption_key_version));
       }
       RETURN_NOT_OK(fs_manager->Open());
 
@@ -320,7 +334,7 @@ class RaftConsensusQuorumTest : public KuduTest {
     shared_ptr<RaftConsensus> peer;
     CHECK_OK(peers_->GetPeerByIdx(peer_idx, &peer));
     while (true) {
-      if (OpIdCompare(peer->queue_->GetLastOpIdInLog(), to_wait_for) >= 0) {
+      if (peer->queue_->GetLastOpIdInLog() >= to_wait_for) {
         return;
       }
       SleepFor(MonoDelta::FromMilliseconds(1));
@@ -408,7 +422,7 @@ class RaftConsensusQuorumTest : public KuduTest {
       ASSERT_OK(WaitForReplicate(round.get()));
       last_op_id->CopyFrom(round->id());
       if (commit_mode == COMMIT_ONE_BY_ONE) {
-        CommitDummyMessage(leader_idx, round.get(), commit_sync);
+        ASSERT_OK(CommitDummyMessage(leader_idx, round.get(), commit_sync));
       }
       rounds->push_back(round);
     }
@@ -431,7 +445,7 @@ class RaftConsensusQuorumTest : public KuduTest {
   void GatherLogEntries(int idx, const scoped_refptr<Log>& log,
                         LogEntries* entries) {
     ASSERT_OK(log->WaitUntilAllFlushed());
-    log->Close();
+    ASSERT_OK(log->Close());
     shared_ptr<LogReader> log_reader;
     ASSERT_OK(log::LogReader::Open(fs_managers_[idx].get(),
                                    /*index*/nullptr,
@@ -505,9 +519,22 @@ class RaftConsensusQuorumTest : public KuduTest {
     }
   }
 
-  void VerifyNoCommitsBeforeReplicates(const LogEntries& entries) {
-    std::unordered_set<OpId, OpIdHashFunctor, OpIdEqualsFunctor> replication_ops;
+  static void VerifyNoCommitsBeforeReplicates(const LogEntries& entries) {
+    // OpId equals functor for std::unordered_map.
+    struct OpIdEqualsFunctor {
+      bool operator()(const OpId& lhs, const OpId& rhs) const {
+        return lhs == rhs;
+      }
+    };
 
+    // OpId hash functor for std::unordered_map.
+    struct OpIdHashFunctor {
+      size_t operator()(const OpId& id) const {
+        return (id.term() + 31) ^ id.index();
+      }
+    };
+
+    std::unordered_set<OpId, OpIdHashFunctor, OpIdEqualsFunctor> replication_ops;
     for (const auto& entry : entries) {
       if (entry->has_replicate()) {
         ASSERT_TRUE(InsertIfNotPresent(&replication_ops, entry->replicate().id()))
@@ -730,7 +757,7 @@ TEST_F(RaftConsensusQuorumTest, TestConsensusStopsIfAMajorityFallsBehind) {
   // After we release the locks the operation should replicate to all replicas
   // and we commit.
   ASSERT_OK(WaitForReplicate(round.get()));
-  CommitDummyMessage(kLeaderIdx, round.get());
+  ASSERT_OK(CommitDummyMessage(kLeaderIdx, round.get()));
 
   // Assert that everything was ok
   WaitForReplicateIfNotAlreadyPresent(last_op_id, kFollower0Idx);
@@ -972,7 +999,7 @@ TEST_F(RaftConsensusQuorumTest, TestReplicasEnforceTheLogMatchingProperty) {
   // Appending this message to peer0 should work and update
   // its 'last_received' to 'id'.
   ASSERT_OK(follower->Update(&req, &resp));
-  ASSERT_TRUE(OpIdEquals(resp.status().last_received(), *id));
+  ASSERT_EQ(resp.status().last_received(), *id);
   ASSERT_EQ(0, follower->queue_->metrics_.num_ops_behind_leader->value());
 
   // Now skip one message in the same term. The replica should
@@ -1006,7 +1033,7 @@ TEST_F(RaftConsensusQuorumTest, TestRequestVote) {
   WaitForCommitIfNotAlreadyPresent(last_op_id.index(), 1, 2);
 
   // Ensure last-logged OpId is > (0,0).
-  ASSERT_TRUE(OpIdLessThan(MinimumOpId(), last_op_id));
+  ASSERT_LT(MinimumOpId(), last_op_id);
 
   const int kPeerIndex = 1;
   shared_ptr<RaftConsensus> peer;
