@@ -15,6 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include <atomic>
 #include <cmath>
 #include <cstdint>
 #include <cstring>
@@ -36,6 +37,7 @@
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/block_manager.h"
 #include "kudu/fs/data_dirs.h"
+#include "kudu/fs/dir_manager.h"
 #include "kudu/fs/error_manager.h"
 #include "kudu/fs/file_block_manager.h" // IWYU pragma: keep
 #include "kudu/fs/fs.pb.h"
@@ -46,7 +48,6 @@
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
-#include "kudu/util/atomic.h"
 #include "kudu/util/countdown_latch.h"
 #include "kudu/util/env.h"
 #include "kudu/util/faststring.h"
@@ -64,6 +65,7 @@
 DECLARE_bool(cache_force_single_shard);
 DECLARE_double(log_container_excess_space_before_cleanup_fraction);
 DECLARE_double(log_container_live_metadata_before_compact_ratio);
+DECLARE_string(block_manager);
 DECLARE_uint64(log_container_max_size);
 DECLARE_uint64(log_container_preallocate_bytes);
 
@@ -131,6 +133,7 @@ class BlockManagerStressTest : public KuduTest {
       total_blocks_read_(0),
       total_bytes_read_(0),
       total_blocks_deleted_(0) {
+    FLAGS_block_manager = T::name();
 
     // Increase the number of containers created.
     FLAGS_log_container_max_size = 1 * 1024 * 1024;
@@ -149,12 +152,12 @@ class BlockManagerStressTest : public KuduTest {
     // Defer block manager creation until after the above flags are set.
     bm_.reset(CreateBlockManager());
     CHECK_OK(file_cache_.Init());
-    CHECK_OK(bm_->Open(nullptr, nullptr, nullptr));
+    CHECK_OK(bm_->Open(nullptr, BlockManager::MergeReport::NOT_REQUIRED, nullptr, nullptr));
     CHECK_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
     CHECK_OK(dd_manager_->GetDataDirGroupPB(test_tablet_name_, &test_group_pb_));
   }
 
-  virtual void TearDown() override {
+  void TearDown() override {
     // If non-standard paths were provided we need to delete them in between
     // test runs.
     if (!FLAGS_block_manager_paths.empty()) {
@@ -186,8 +189,9 @@ class BlockManagerStressTest : public KuduTest {
       CHECK_OK(DataDirManager::OpenExistingForTests(env_, data_dirs,
           DataDirManagerOptions(), &dd_manager_));
     }
-    return new T(env_, dd_manager_.get(), &error_manager_,
-                 &file_cache_, BlockManagerOptions());
+    error_manager_ = new FsErrorManager();
+    return new T(env_, dd_manager_.get(), error_manager_,
+                 &file_cache_, BlockManagerOptions(), fs::kDefaultTenantName);
   }
 
   void RunTest(double secs) {
@@ -254,13 +258,13 @@ class BlockManagerStressTest : public KuduTest {
   // Protects written_blocks_.
   simple_spinlock lock_;
 
-  unique_ptr<DataDirManager> dd_manager_;
+  scoped_refptr<DataDirManager> dd_manager_;
 
-  FsErrorManager error_manager_;
+  scoped_refptr<FsErrorManager> error_manager_;
 
   FileCache file_cache_;
 
-  unique_ptr<BlockManager> bm_;
+  scoped_refptr<BlockManager> bm_;
 
   // Test group of disk to spread data across.
   DataDirGroupPB test_group_pb_;
@@ -273,13 +277,13 @@ class BlockManagerStressTest : public KuduTest {
 
   // Some performance counters.
 
-  AtomicInt<int64_t> total_blocks_written_;
-  AtomicInt<int64_t> total_bytes_written_;
+  std::atomic<int64_t> total_blocks_written_;
+  std::atomic<int64_t> total_bytes_written_;
 
-  AtomicInt<int64_t> total_blocks_read_;
-  AtomicInt<int64_t> total_bytes_read_;
+  std::atomic<int64_t> total_blocks_read_;
+  std::atomic<int64_t> total_bytes_read_;
 
-  AtomicInt<int64_t> total_blocks_deleted_;
+  std::atomic<int64_t> total_blocks_deleted_;
 };
 
 template <typename T>
@@ -348,15 +352,15 @@ void BlockManagerStressTest<T>::WriterThread() {
     CHECK_OK(creation_transaction->CommitCreatedBlocks());
     // Publish the now sync'ed blocks to readers and deleters.
     {
-      std::lock_guard<simple_spinlock> l(lock_);
+      std::lock_guard l(lock_);
       for (const auto& block : all_dirty_blocks) {
         InsertOrDie(&written_blocks_, block, 0);
       }
     }
   }
 
-  total_blocks_written_.IncrementBy(num_blocks_written);
-  total_bytes_written_.IncrementBy(num_bytes_written);
+  total_blocks_written_.fetch_add(num_blocks_written, std::memory_order_relaxed);
+  total_bytes_written_.fetch_add(num_bytes_written, std::memory_order_relaxed);
 }
 
 template <typename T>
@@ -369,7 +373,7 @@ void BlockManagerStressTest<T>::ReaderThread() {
     BlockId block_id;
     {
       // Grab a block at random.
-      std::lock_guard<simple_spinlock> l(lock_);
+      std::lock_guard l(lock_);
       if (written_blocks_.empty()) {
         continue;
       }
@@ -385,7 +389,7 @@ void BlockManagerStressTest<T>::ReaderThread() {
 
     // Done opening the block, make it available for deleting.
     {
-      std::lock_guard<simple_spinlock> l(lock_);
+      std::lock_guard l(lock_);
       int& openers = FindOrDie(written_blocks_, block_id);
       openers--;
     }
@@ -422,8 +426,8 @@ void BlockManagerStressTest<T>::ReaderThread() {
     num_bytes_read += block_size;
   }
 
-  total_blocks_read_.IncrementBy(num_blocks_read);
-  total_bytes_read_.IncrementBy(num_bytes_read);
+  total_blocks_read_.fetch_add(num_blocks_read, std::memory_order_relaxed);
+  total_bytes_read_.fetch_add(num_bytes_read, std::memory_order_relaxed);
 }
 
 template <typename T>
@@ -466,7 +470,7 @@ void BlockManagerStressTest<T>::DeleterThread() {
     num_blocks_deleted += deleted.size();
   }
 
-  total_blocks_deleted_.IncrementBy(num_blocks_deleted);
+  total_blocks_deleted_.fetch_add(num_blocks_deleted, std::memory_order_relaxed);
 }
 
 template <>
@@ -480,8 +484,8 @@ int BlockManagerStressTest<FileBlockManager>::GetMaxFdCount() const {
       FLAGS_num_reader_threads;
 }
 
-template <>
-int BlockManagerStressTest<LogBlockManager>::GetMaxFdCount() const {
+template <typename T>
+int BlockManagerStressTest<T>::GetMaxFdCount() const {
   return FLAGS_max_open_files +
       // If all containers are full, each open block could theoretically
       // result in a new container, which is two files briefly outside the
@@ -494,19 +498,24 @@ void BlockManagerStressTest<FileBlockManager>::InjectNonFatalInconsistencies() {
   // Do nothing; the FBM has no repairable inconsistencies.
 }
 
-template <>
-void BlockManagerStressTest<LogBlockManager>::InjectNonFatalInconsistencies() {
-  LBMCorruptor corruptor(env_, dd_manager_->GetDirs(), rand_seed_);
-  ASSERT_OK(corruptor.Init());
+template <typename T>
+void BlockManagerStressTest<T>::InjectNonFatalInconsistencies() {
+  auto corruptor = LBMCorruptor::Create(env_, dd_manager_.get(), rand_seed_);
+  ASSERT_OK(corruptor->Init());
 
   for (int i = 0; i < FLAGS_num_inconsistencies; i++) {
-    ASSERT_OK(corruptor.InjectRandomNonFatalInconsistency());
+    ASSERT_OK(corruptor->InjectRandomNonFatalInconsistency());
   }
 }
 
 // What kinds of BlockManagers are supported?
 #if defined(__linux__)
-typedef ::testing::Types<FileBlockManager, LogBlockManager> BlockManagers;
+#if defined(NO_ROCKSDB)
+typedef ::testing::Types<FileBlockManager, LogBlockManagerNativeMeta> BlockManagers;
+#else
+typedef ::testing::Types<FileBlockManager, LogBlockManagerNativeMeta, LogBlockManagerRdbMeta>
+    BlockManagers;
+#endif
 #else
 typedef ::testing::Types<FileBlockManager> BlockManagers;
 #endif
@@ -542,7 +551,7 @@ TYPED_TEST(BlockManagerStressTest, StressTest) {
     LOG(INFO) << "Running on populated block manager (restart #" << i << ")";
     this->bm_.reset(this->CreateBlockManager());
     FsReport report;
-    ASSERT_OK(this->bm_->Open(&report, nullptr, nullptr));
+    ASSERT_OK(this->bm_->Open(&report, BlockManager::MergeReport::NOT_REQUIRED, nullptr, nullptr));
     ASSERT_OK(this->dd_manager_->LoadDataDirGroupFromPB(this->test_tablet_name_,
                                                         this->test_group_pb_));
     ASSERT_OK(report.LogAndCheckForFatalErrors());
@@ -553,15 +562,15 @@ TYPED_TEST(BlockManagerStressTest, StressTest) {
   LOG(INFO) << "Printing test totals";
   LOG(INFO) << "--------------------";
   LOG(INFO) << Substitute("Wrote $0 blocks ($1 bytes) via $2 threads",
-                          this->total_blocks_written_.Load(),
-                          this->total_bytes_written_.Load(),
+                          this->total_blocks_written_.load(),
+                          this->total_bytes_written_.load(),
                           FLAGS_num_writer_threads);
   LOG(INFO) << Substitute("Read $0 blocks ($1 bytes) via $2 threads",
-                          this->total_blocks_read_.Load(),
-                          this->total_bytes_read_.Load(),
+                          this->total_blocks_read_.load(),
+                          this->total_bytes_read_.load(),
                           FLAGS_num_reader_threads);
   LOG(INFO) << Substitute("Deleted $0 blocks via $1 threads",
-                          this->total_blocks_deleted_.Load(),
+                          this->total_blocks_deleted_.load(),
                           FLAGS_num_deleter_threads);
 }
 
