@@ -17,9 +17,11 @@
 
 #pragma once
 
+#include <atomic>
 #include <cstddef>
 #include <cstdint>
 #include <memory>
+#include <set>
 #include <string>
 #include <vector>
 
@@ -53,7 +55,7 @@ struct FsReport;
 // - Blocks opened for writing are not thread-safe.
 class Block {
  public:
-  virtual ~Block() {}
+  virtual ~Block() = default;
 
   // Returns the identifier for this block.
   virtual const BlockId& id() const = 0;
@@ -93,7 +95,7 @@ class WritableBlock : public Block {
 
   // Destroy the WritableBlock. If it was not explicitly closed using Close(),
   // this will Abort() the block.
-  virtual ~WritableBlock() {}
+  ~WritableBlock() override = default;
 
   // Destroys the in-memory representation of the block and synchronizes
   // dirty block data and metadata with the disk. On success, guarantees
@@ -140,7 +142,7 @@ class WritableBlock : public Block {
 // may be shared amongst threads for concurrent reading.
 class ReadableBlock : public Block {
  public:
-  virtual ~ReadableBlock() {}
+  ~ReadableBlock() override = default;
 
   // Destroys the in-memory representation of the block.
   virtual Status Close() = 0;
@@ -177,8 +179,6 @@ struct CreateBlockOptions {
 
 // Block manager creation options.
 struct BlockManagerOptions {
-  BlockManagerOptions();
-
   // The entity under which all metrics should be grouped. If NULL, metrics
   // will not be produced.
   //
@@ -190,23 +190,26 @@ struct BlockManagerOptions {
   std::shared_ptr<MemTracker> parent_mem_tracker;
 
   // Whether the block manager should only allow reading. Defaults to false.
-  bool read_only;
+  bool read_only = false;
 };
 
 // Utilities for Kudu block lifecycle management. All methods are
 // thread-safe.
-class BlockManager {
+class BlockManager : public RefCountedThreadSafe<BlockManager> {
  public:
   // Lists the available block manager types.
-  static std::vector<std::string> block_manager_types() {
-#if defined(__linux__)
-    return { "file", "log" };
-#else
-    return { "file" };
-#endif
-  }
+  static const std::set<std::string>& block_manager_types();
 
-  virtual ~BlockManager() {}
+  virtual ~BlockManager() = default;
+
+  enum class MergeReport {
+    NOT_REQUIRED,    ///< Do no fs report merge operation when open block manager.
+                     ///< The report only records the data of this open operation.
+
+    REQUIRED         ///< Do fs report merge operation when open block manager if report ptr
+                     ///  is not null.
+                     ///< The report updates the data of this operation to the incoming data.
+  };
 
   // Opens an existing on-disk representation of this block manager and
   // checks it for inconsistencies. If found, and if the block manager was not
@@ -223,7 +226,8 @@ class BlockManager {
   // If 'containers_processed' and 'containers_total' are not nullptr, they will
   // be populated with total containers attempted to be opened/processed and
   // total containers present respectively.
-  virtual Status Open(FsReport* report, std::atomic<int>* containers_processed,
+  virtual Status Open(FsReport* report, MergeReport need_merage,
+                      std::atomic<int>* containers_processed,
                       std::atomic<int>* containers_total) = 0;
 
   // Creates a new block using the provided options and opens it for
@@ -248,6 +252,13 @@ class BlockManager {
   virtual Status OpenBlock(const BlockId& block_id,
                            std::unique_ptr<ReadableBlock>* block) = 0;
 
+  // Get the data file where the block located.
+  //
+  // Find the data file according to the block id. The data file is the
+  // unique file where the block data is stored. On success, overwrites
+  // 'path' with the file's path.
+  virtual bool FindBlockPath(const BlockId& block_id, std::string* path) const = 0;
+
   // Constructs a block creation transaction to group a set of block creation
   // operations and closes the registered blocks together.
   virtual std::unique_ptr<BlockCreationTransaction> NewCreationTransaction() = 0;
@@ -273,7 +284,13 @@ class BlockManager {
   virtual void NotifyBlockId(BlockId block_id) = 0;
 
   // Exposes the FsErrorManager used to handle fs errors.
-  virtual FsErrorManager* error_manager() = 0;
+  virtual scoped_refptr<FsErrorManager> error_manager() = 0;
+
+  // Exposes the tenant id.
+  virtual std::string tenant_id() const = 0;
+
+ private:
+  friend class RefCountedThreadSafe<BlockManager>;
 };
 
 // Group a set of block creations together in a transaction. This has two
@@ -314,8 +331,8 @@ class BlockDeletionTransaction {
   // Deletes a group of blocks given the block IDs, the actual deletion will take
   // place after the last open reader or writer is closed for each block that needs
   // be to deleted. The 'deleted' out parameter will be set with the list of block
-  // IDs that were successfully deleted, regardless of the value of returned 'status'
-  // is OK or error.
+  // IDs that were successfully deleted if it's not nullptr, regardless of the value
+  // of returned 'status' is OK or error.
   //
   // Returns the first deletion failure that was seen, if any.
   virtual Status CommitDeletedBlocks(std::vector<BlockId>* deleted) = 0;
