@@ -22,11 +22,12 @@
 #include <cstddef>
 #include <cstdint>
 #include <memory>
-#include <mutex>
 #include <ostream>
 #include <random>
 #include <set>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
@@ -39,6 +40,7 @@
 #include "kudu/fs/block_manager.h"
 #include "kudu/fs/dir_util.h"
 #include "kudu/fs/fs.pb.h"
+#include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/integral_types.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/map-util.h"
@@ -51,7 +53,7 @@
 #include "kudu/util/random.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_util_prod.h"
-#include "kudu/util/threadpool.h"
+#include "kudu/util/threadpool.h" // IWYU pragma: keep
 
 DEFINE_int32(fs_target_data_dirs_per_tablet, 3,
              "Indicates the target number of data dirs to spread each "
@@ -125,14 +127,9 @@ METRIC_DEFINE_gauge_uint64(server, data_dirs_full,
 DECLARE_bool(enable_data_block_fsync);
 DECLARE_string(block_manager);
 
-namespace kudu {
-
-namespace fs {
-
-using internal::DataDirGroup;
+using kudu::fs::internal::DataDirGroup;
 using std::default_random_engine;
-using std::pair;
-using std::set;
+using std::shared_lock;
 using std::shuffle;
 using std::string;
 using std::unique_ptr;
@@ -142,6 +139,8 @@ using std::vector;
 using strings::Substitute;
 using strings::SubstituteAndAppend;
 
+namespace kudu {
+namespace fs {
 
 ////////////////////////////////////////////////////////////
 // DataDirMetrics
@@ -197,36 +196,26 @@ Status DataDirGroup::CopyToPB(const UuidByUuidIndexMap& uuid_by_uuid_idx,
 }
 
 ////////////////////////////////////////////////////////////
-// DataDir
+// DataDirManager
 ////////////////////////////////////////////////////////////
 
-DataDir::DataDir(Env* env, DirMetrics* metrics, FsType fs_type, std::string dir,
-                 std::unique_ptr<DirInstanceMetadataFile> metadata_file,
-                 std::unique_ptr<ThreadPool> pool)
-    : Dir(env, metrics, fs_type, std::move(dir), std::move(metadata_file), std::move(pool)) {}
-
 std::unique_ptr<Dir> DataDirManager::CreateNewDir(
     Env* env, DirMetrics* metrics, FsType fs_type,
     std::string dir, std::unique_ptr<DirInstanceMetadataFile> metadata_file,
     std::unique_ptr<ThreadPool> pool) {
-  return unique_ptr<Dir>(new DataDir(env, metrics, fs_type, std::move(dir),
-                                     std::move(metadata_file), std::move(pool)));
+#if !defined(NO_ROCKSDB)
+  if (FLAGS_block_manager == "logr") {
+    bool newly_created = ContainsKey(created_fs_dir_paths_, dir);
+    return std::make_unique<RdbDir>(env, metrics, fs_type, newly_created, std::move(dir),
+                                    std::move(metadata_file), std::move(pool));
+  }
+#endif
+  return std::make_unique<Dir>(env, metrics, fs_type, std::move(dir),
+                               std::move(metadata_file), std::move(pool));
 }
 
-int DataDir::available_space_cache_secs() const {
-  return FLAGS_fs_data_dirs_available_space_cache_seconds;
-}
-
-int DataDir::reserved_bytes() const {
-  return FLAGS_fs_data_dirs_reserved_bytes;
-}
-
-////////////////////////////////////////////////////////////
-// DataDirManager
-////////////////////////////////////////////////////////////
-
 DataDirManagerOptions::DataDirManagerOptions()
-    : DirManagerOptions(FLAGS_block_manager) {}
+    : DirManagerOptions(FLAGS_block_manager, fs::kDefaultTenantID) {}
 
 DataDirManager::DataDirManager(Env* env,
                                const DataDirManagerOptions& opts,
@@ -237,29 +226,33 @@ DataDirManager::DataDirManager(Env* env,
                  opts, std::move(canonicalized_data_roots)) {}
 
 Status DataDirManager::OpenExistingForTests(Env* env,
-                                            vector<string> data_fs_roots,
+                                            const vector<string>& data_fs_roots,
                                             const DataDirManagerOptions& opts,
-                                            unique_ptr<DataDirManager>* dd_manager) {
+                                            scoped_refptr<kudu::fs::DataDirManager>* dd_manager) {
   CanonicalizedRootsList roots;
   for (const auto& r : data_fs_roots) {
     roots.push_back({ r, Status::OK() });
   }
+
+  // Reset the existing DataDirManager before opening the new one to release resources
+  // (e.g. RocksDB 'LOCK' file when --block_manager=logr) held by the existing one.
+  dd_manager->reset();
   return DataDirManager::OpenExisting(env, std::move(roots), opts, dd_manager);
 }
 
 Status DataDirManager::OpenExisting(Env* env, CanonicalizedRootsList data_fs_roots,
                                     const DataDirManagerOptions& opts,
-                                    unique_ptr<DataDirManager>* dd_manager) {
-  unique_ptr<DataDirManager> dm;
+                                    scoped_refptr<kudu::fs::DataDirManager>* dd_manager) {
+  scoped_refptr<DataDirManager> dm;
   dm.reset(new DataDirManager(env, opts, std::move(data_fs_roots)));
   RETURN_NOT_OK(dm->Open());
   dd_manager->swap(dm);
   return Status::OK();
 }
 
-Status DataDirManager::CreateNewForTests(Env* env, vector<string> data_fs_roots,
+Status DataDirManager::CreateNewForTests(Env* env, const vector<string>& data_fs_roots,
                                          const DataDirManagerOptions& opts,
-                                         unique_ptr<DataDirManager>* dd_manager) {
+                                         scoped_refptr<kudu::fs::DataDirManager>* dd_manager) {
   CanonicalizedRootsList roots;
   for (const auto& r : data_fs_roots) {
     roots.push_back({ r, Status::OK() });
@@ -269,8 +262,8 @@ Status DataDirManager::CreateNewForTests(Env* env, vector<string> data_fs_roots,
 
 Status DataDirManager::CreateNew(Env* env, CanonicalizedRootsList data_fs_roots,
                                  const DataDirManagerOptions& opts,
-                                 unique_ptr<DataDirManager>* dd_manager) {
-  unique_ptr<DataDirManager> dm;
+                                 scoped_refptr<kudu::fs::DataDirManager>* dd_manager) {
+  scoped_refptr<DataDirManager> dm;
   dm.reset(new DataDirManager(env, opts, std::move(data_fs_roots)));
   RETURN_NOT_OK(dm->Create());
   RETURN_NOT_OK(dm->Open());
@@ -291,7 +284,7 @@ int DataDirManager::max_dirs() const {
 }
 
 Status DataDirManager::PopulateDirectoryMaps(const vector<unique_ptr<Dir>>& dirs) {
-  if (opts_.dir_type == "log") {
+  if (FsManager::IsLogType(opts_.dir_type)) {
     return DirManager::PopulateDirectoryMaps(dirs);
   }
   DCHECK_EQ("file", opts_.dir_type);
@@ -344,7 +337,7 @@ Status DataDirManager::PopulateDirectoryMaps(const vector<unique_ptr<Dir>>& dirs
 
 Status DataDirManager::LoadDataDirGroupFromPB(const std::string& tablet_id,
                                               const DataDirGroupPB& pb) {
-  std::lock_guard<percpu_rwlock> lock(dir_group_lock_);
+  std::lock_guard lock(dir_group_lock_);
   DataDirGroup group_from_pb;
   RETURN_NOT_OK_PREPEND(group_from_pb.LoadFromPB(idx_by_uuid_, pb), Substitute(
       "could not load data dir group for tablet $0", tablet_id));
@@ -364,7 +357,7 @@ Status DataDirManager::LoadDataDirGroupFromPB(const std::string& tablet_id,
 
 Status DataDirManager::CreateDataDirGroup(const string& tablet_id,
                                           DirDistributionMode mode) {
-  std::lock_guard<percpu_rwlock> write_lock(dir_group_lock_);
+  std::lock_guard write_lock(dir_group_lock_);
   if (ContainsKey(group_by_tablet_map_, tablet_id)) {
     return Status::AlreadyPresent("Tried to create directory group for tablet but one is already "
                                   "registered", tablet_id);
@@ -490,7 +483,7 @@ Status DataDirManager::GetDirForBlock(const CreateBlockOptions& opts, Dir** dir,
 }
 
 void DataDirManager::DeleteDataDirGroup(const std::string& tablet_id) {
-  std::lock_guard<percpu_rwlock> lock(dir_group_lock_);
+  std::lock_guard lock(dir_group_lock_);
   DataDirGroup* group = FindOrNull(group_by_tablet_map_, tablet_id);
   if (group == nullptr) {
     return;
@@ -535,7 +528,7 @@ Status DataDirManager::GetDirAddIfNecessary(const CreateBlockOptions& opts, Dir*
   // If we couldn't get a directory because the group is out of space, try
   // adding a new directory to the group.
   DCHECK_GT(new_target_group_size, 0);
-  std::lock_guard<percpu_rwlock> l(dir_group_lock_);
+  std::lock_guard l(dir_group_lock_);
   const DataDirGroup& group = FindOrDie(group_by_tablet_map_, tablet_id);
   // If we're already at the new target group size (e.g. because another
   // thread has added a directory), just return the newly added directory.
