@@ -17,20 +17,34 @@
 
 #include "kudu/fs/dir_manager.h"
 
-#include <errno.h>
-
 #include <algorithm>
+#include <cerrno>
 #include <functional>
 #include <iterator>
 #include <memory>
+#include <mutex>
 #include <ostream>
 #include <set>
+#include <shared_mutex>
 #include <string>
 #include <unordered_set>
 #include <utility>
 #include <vector>
 
+#if !defined(NO_ROCKSDB)
+#include <gflags/gflags.h>
+#else
+#include <gflags/gflags_declare.h>
+#endif
 #include <glog/logging.h>
+#if !defined(NO_ROCKSDB)
+#include <rocksdb/cache.h>
+#include <rocksdb/filter_policy.h>
+#include <rocksdb/options.h>
+#include <rocksdb/slice_transform.h>
+#include <rocksdb/status.h>
+#include <rocksdb/table.h>
+#endif
 
 #include "kudu/fs/dir_util.h"
 #include "kudu/fs/fs.pb.h"
@@ -38,17 +52,26 @@
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/substitute.h"
+#include "kudu/gutil/strings/util.h"
 #include "kudu/util/env.h"
 #include "kudu/util/env_util.h"
+#if !defined(NO_ROCKSDB)
+#include "kudu/util/flag_tags.h"
+#endif
 #include "kudu/util/oid_generator.h"
 #include "kudu/util/path_util.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/random_util.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/stopwatch.h"
+#if !defined(NO_ROCKSDB)
+#include "kudu/util/test_util_prod.h"
+#endif
 #include "kudu/util/threadpool.h"
 
 using std::set;
+using std::shared_lock;
+using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
 using std::unordered_map;
@@ -56,10 +79,90 @@ using std::unordered_set;
 using std::vector;
 using strings::Substitute;
 
+DECLARE_int32(fs_data_dirs_available_space_cache_seconds);
+DECLARE_int64(fs_data_dirs_reserved_bytes);
+DECLARE_string(block_manager);
+
+#if !defined(NO_ROCKSDB)
+DEFINE_double(log_container_rdb_bits_per_key, 9.9,
+              "Average number of bits allocated per key in RocksDB bloom filter, for details see "
+              "https://github.com/facebook/rocksdb/wiki/RocksDB-Bloom-Filter. It is only effective "
+              "when --block_manager='logr'");
+TAG_FLAG(log_container_rdb_bits_per_key, advanced);
+TAG_FLAG(log_container_rdb_bits_per_key, experimental);
+
+DEFINE_uint32(log_container_rdb_block_cache_capacity_mb, 10,
+              "The block cache capacity of RocksDB in MiB, it is shared by all RocksDB instances "
+              "in the process. It is only effective when --block_manager='logr'");
+TAG_FLAG(log_container_rdb_block_cache_capacity_mb, advanced);
+TAG_FLAG(log_container_rdb_block_cache_capacity_mb, experimental);
+
+DEFINE_uint32(log_container_rdb_max_background_jobs, 8,
+              "The maximum number of concurrent background jobs (compactions and flushes) shared "
+              "between RocksDB instances. It is only effective when --block_manager='logr'");
+TAG_FLAG(log_container_rdb_max_background_jobs, advanced);
+TAG_FLAG(log_container_rdb_max_background_jobs, experimental);
+
+DEFINE_uint32(log_container_rdb_max_write_buffer_number, 2,
+              "The maximum number of write buffers that are built up in memory of each RocksDB "
+              "instance. It is only effective when --block_manager='logr'");
+TAG_FLAG(log_container_rdb_max_write_buffer_number, advanced);
+TAG_FLAG(log_container_rdb_max_write_buffer_number, experimental);
+
+DEFINE_double(log_container_rdb_memtable_prefix_bloom_size_ratio, 0.1,
+              "Enables a dynamic bloom filter of RocksDB memtable to optimize many queries that "
+              "must go beyond the memtable if it is larger than 0. The size in bytes of the filter "
+              "is --log_container_rdb_write_buffer_size * "
+              "--log_container_rdb_memtable_prefix_bloom_size_ratio. It is only effective when "
+              "--block_manager='logr'");
+TAG_FLAG(log_container_rdb_memtable_prefix_bloom_size_ratio, advanced);
+TAG_FLAG(log_container_rdb_memtable_prefix_bloom_size_ratio, experimental);
+
+DEFINE_uint64(log_container_rdb_write_buffer_size, 64 << 20,
+              "The amount of data in RocksDB to build up in memory (backed by an unsorted log on "
+              "disk) before converting to a sorted on-disk file. It is only effective when "
+              "--block_manager='logr'");
+TAG_FLAG(log_container_rdb_write_buffer_size, advanced);
+TAG_FLAG(log_container_rdb_write_buffer_size, experimental);
+
+DEFINE_string(log_container_rdb_db_log_dir, "",
+              "This specifies the info log dir of RocksDB. If it is empty, the log files are in "
+              "the same dir as data (i.e. each dir in --fs_data_dirs). If it is not empty, the log "
+              "files will be in the specified dir, and the --fs_data_dirs absolute path will "
+              "be used as the log file name's prefixes. It is only effective when "
+              "--block_manager='logr'");
+TAG_FLAG(log_container_rdb_db_log_dir, advanced);
+TAG_FLAG(log_container_rdb_db_log_dir, experimental);
+
+DEFINE_uint64(log_container_rdb_max_log_file_size, 8 << 20,
+              "Maximum byte size of the RocksDB info log file. If the log file is larger "
+              "than specified, a new info log file will be created. If it is 0, all logs will be "
+              "written to one log file. It is only effective when --block_manager='logr'");
+TAG_FLAG(log_container_rdb_max_log_file_size, advanced);
+TAG_FLAG(log_container_rdb_max_log_file_size, experimental);
+
+DEFINE_uint64(log_container_rdb_keep_log_file_num, 10,
+              "Maximum number of RocksDB info log files to keep. It is only effective when "
+              "--block_manager='logr'");
+TAG_FLAG(log_container_rdb_keep_log_file_num, advanced);
+TAG_FLAG(log_container_rdb_keep_log_file_num, experimental);
+
+DEFINE_uint64(log_container_rdb_max_manifest_file_size, 64 << 20,
+              "The RocksDB manifest file is rolled over on reaching this byte limit. It is only "
+              "effective when --block_manager='logr'");
+TAG_FLAG(log_container_rdb_max_manifest_file_size, advanced);
+TAG_FLAG(log_container_rdb_max_manifest_file_size, experimental);
+
+DEFINE_int32(log_container_rdb_level0_file_num_compaction_trigger, 4,
+             "Number of files to trigger level-0 compaction in RocksDB. A value <0 means that "
+             "level-0 compaction will not be triggered by the number of files at all. It is only "
+             "effective when --block_manager='logr'");
+TAG_FLAG(log_container_rdb_level0_file_num_compaction_trigger, advanced);
+TAG_FLAG(log_container_rdb_level0_file_num_compaction_trigger, experimental);
+#endif
+
 namespace kudu {
-
 namespace {
-
 // Wrapper for env_util::DeleteTmpFilesRecursively that is suitable for parallel
 // execution on a data directory's thread pool (which requires the return value
 // be void).
@@ -67,9 +170,33 @@ void DeleteTmpFilesRecursively(Env* env, const string& path) {
   WARN_NOT_OK(env_util::DeleteTmpFilesRecursively(env, path),
               "Error while deleting temp files");
 }
-
 } // anonymous namespace
+
 namespace fs {
+#if !defined(NO_ROCKSDB)
+Status FromRdbStatus(const rocksdb::Status& s) {
+  switch (s.code()) {
+    case rocksdb::Status::kOk:
+      return Status::OK();
+    case rocksdb::Status::kNotFound:
+      return Status::NotFound(s.ToString());
+    case rocksdb::Status::kCorruption:
+      return Status::Corruption(s.ToString());
+    case rocksdb::Status::kNotSupported:
+      return Status::NotSupported(s.ToString());
+    case rocksdb::Status::kInvalidArgument:
+      return Status::InvalidArgument(s.ToString());
+    case rocksdb::Status::kIOError:
+      return Status::IOError(s.ToString());
+    case rocksdb::Status::kIncomplete:
+      return Status::Incomplete(s.ToString());
+    case rocksdb::Status::kAborted:
+      return Status::Aborted(s.ToString());
+    default:
+      return Status::RuntimeError(s.ToString());
+  }
+}
+#endif
 
 Dir::Dir(Env* env,
          DirMetrics* metrics,
@@ -118,7 +245,7 @@ void Dir::WaitOnClosures() {
 Status Dir::RefreshAvailableSpace(RefreshMode mode) {
   switch (mode) {
     case RefreshMode::EXPIRED_ONLY: {
-      std::lock_guard<simple_spinlock> l(lock_);
+      std::lock_guard l(lock_);
       DCHECK(last_space_check_.Initialized());
       MonoTime expiry = last_space_check_ + MonoDelta::FromSeconds(
           available_space_cache_secs());
@@ -143,7 +270,7 @@ Status Dir::RefreshAvailableSpace(RefreshMode mode) {
       }
       RETURN_NOT_OK_PREPEND(s, "Could not refresh fullness"); // Catch other types of IOErrors, etc.
       {
-        std::lock_guard<simple_spinlock> l(lock_);
+        std::lock_guard l(lock_);
         if (metrics_ && is_full_ != is_full_new) {
           metrics_->dirs_full->IncrementBy(is_full_new ? 1 : -1);
         }
@@ -159,8 +286,136 @@ Status Dir::RefreshAvailableSpace(RefreshMode mode) {
   return Status::OK();
 }
 
-DirManagerOptions::DirManagerOptions(const string& dir_type)
-    : dir_type(dir_type), read_only(false),
+int Dir::available_space_cache_secs() {
+  return FLAGS_fs_data_dirs_available_space_cache_seconds;
+}
+
+int Dir::reserved_bytes() {
+  return FLAGS_fs_data_dirs_reserved_bytes;
+}
+
+#if !defined(NO_ROCKSDB)
+shared_ptr<rocksdb::Cache> RdbDir::s_block_cache_;
+RdbDir::RdbDir(Env* env, DirMetrics* metrics,
+               FsType fs_type,
+               bool newly_created,
+               string dir,
+               unique_ptr<DirInstanceMetadataFile> metadata_file,
+               unique_ptr<ThreadPool> pool)
+    : Dir(env, metrics, fs_type, std::move(dir), std::move(metadata_file), std::move(pool)) {
+  if (!metadata_file_->healthy()) {
+    LOG(WARNING) << Substitute("Skip initializing rocksdb instance for the non-healthy "
+                               "directory $0",
+                               dir_);
+    return;
+  }
+
+  // Initialize the directory only if it's healthy.
+  // Note: the unhealthy directories will be kept, but will be skipped when opening block manager.
+  auto s = InitRocksDBInstance(newly_created);
+  if (!s.ok()) {
+    s = s.CloneAndPrepend(Substitute("could not initialize $0", dir_));
+    LOG(WARNING) << s.ToString();
+    // Mark the directory as failed if it could not be initialized.
+    DCHECK(metadata_file_->healthy());
+    metadata_file_->SetInstanceFailed(s);
+  }
+}
+
+Status RdbDir::InitRocksDBInstance(bool newly_created) {
+  DCHECK_STREQ(FLAGS_block_manager.c_str(), "logr");
+  if (db_) {
+    // Some unit tests (e.g. BlockManagerTest.PersistenceTest) reopen the block manager,
+    // 'db_' is possible to be non-nullptr.
+    // In non-test environments, 'db_' is always to be nullptr, log a fatal error if not.
+    DCHECK(IsGTest()) <<
+        Substitute("It's not allowed to reopen the RocksDB $0 except in tests", dir_);
+    return Status::OK();
+  }
+
+  // See the rocksdb::Options details:
+  // https://github.com/facebook/rocksdb/blob/main/include/rocksdb/options.h
+  rocksdb::Options opts;
+  // A RocksDB instance is created if it does not exist when opening the Dir.
+  if (newly_created) {
+      opts.create_if_missing = true;
+      opts.error_if_exists = true;
+  } else {
+      opts.create_if_missing = false;
+      opts.error_if_exists = false;
+  }
+  opts.db_log_dir = FLAGS_log_container_rdb_db_log_dir;
+  opts.max_log_file_size = FLAGS_log_container_rdb_max_log_file_size;
+  opts.keep_log_file_num = FLAGS_log_container_rdb_keep_log_file_num;
+  opts.write_buffer_size = FLAGS_log_container_rdb_write_buffer_size;
+  opts.max_write_buffer_number = FLAGS_log_container_rdb_max_write_buffer_number;
+  opts.max_background_jobs = FLAGS_log_container_rdb_max_background_jobs;
+  opts.max_manifest_file_size = FLAGS_log_container_rdb_max_manifest_file_size;
+  opts.level0_file_num_compaction_trigger =
+      FLAGS_log_container_rdb_level0_file_num_compaction_trigger;
+
+  static std::once_flag flag;
+  std::call_once(flag, [&]() {
+    s_block_cache_ = rocksdb::NewLRUCache(FLAGS_log_container_rdb_block_cache_capacity_mb << 20);
+  });
+  rocksdb::BlockBasedTableOptions tbl_opts;
+  tbl_opts.block_cache = s_block_cache_;
+  tbl_opts.whole_key_filtering = false;
+  tbl_opts.filter_policy.reset(rocksdb::NewBloomFilterPolicy(FLAGS_log_container_rdb_bits_per_key));
+  opts.table_factory.reset(NewBlockBasedTableFactory(tbl_opts));
+  // Take advantage of Prefix-Seek, see https://github.com/facebook/rocksdb/wiki/Prefix-Seek.
+  opts.prefix_extractor.reset(rocksdb::NewFixedPrefixTransform(ObjectIdGenerator::IdLength()));
+  opts.memtable_prefix_bloom_size_ratio = FLAGS_log_container_rdb_memtable_prefix_bloom_size_ratio;
+
+  rdb_dir_ = JoinPathSegments(dir_, kRocksDBDirName);
+  rocksdb::DB* db_temp = nullptr;
+  rocksdb::Status rdb_s = rocksdb::DB::Open(opts, *rdb_dir_, &db_temp);
+  RETURN_NOT_OK_PREPEND(FromRdbStatus(rdb_s),
+                        Substitute("open RocksDB failed, path: $0", *rdb_dir_));
+  db_.reset(db_temp);
+  return Status::OK();
+}
+
+void RdbDir::Shutdown() {
+  if (is_shutdown_) {
+    return;
+  }
+
+  // Shut down the thread pool before closing RocksDB to make sure there aren't any in-flight
+  // write operations.
+  WaitOnClosures();
+  pool_->Shutdown();
+
+  // The 'db_' is nullptr if the Dir open failed.
+  if (db_) {
+    // Flushing memtable before closing RocksDB reduces bootstrapping time upon next start-up.
+    // Call Flush() rather than WaitForCompact(), because it's enough to wait for the flush jobs
+    // to finish, compactions jobs may take more time, which results in longer times to shut down
+    // a server.
+    rocksdb::FlushOptions options;
+    options.wait = true;
+    WARN_NOT_OK(FromRdbStatus(db_->Flush(options)),
+                Substitute("Flush RocksDB failed, path: $0", *rdb_dir_));
+    WARN_NOT_OK(FromRdbStatus(db_->Close()),
+                Substitute("Closed RocksDB with error, path: $0", *rdb_dir_));
+    db_.reset();
+  }
+
+  is_shutdown_ = true;
+}
+
+rocksdb::DB* RdbDir::rdb() {
+  DCHECK_STREQ(FLAGS_block_manager.c_str(), "logr");
+  DCHECK(db_);
+  return db_.get();
+}
+#endif
+
+DirManagerOptions::DirManagerOptions(string dir_type,
+                                     string tid)
+    : dir_type(std::move(dir_type)),
+      tenant_id(std::move(tid)),
+      read_only(false),
       update_instances(UpdateInstanceBehavior::UPDATE_AND_IGNORE_FAILURES) {}
 
 
@@ -322,6 +577,8 @@ Status DirManager::CreateNewDirectoriesAndUpdateInstances(
 
   // Success: don't delete any files.
   deleter.cancel();
+  std::move(created_dirs.begin(), created_dirs.end(),
+            std::inserter(created_fs_dir_paths_, created_fs_dir_paths_.end()));
   return Status::OK();
 }
 
@@ -433,7 +690,7 @@ Status DirManager::LoadInstances(
     string instance_filename = JoinPathSegments(dir, instance_metadata_filename());
 
     // Initialize the instance with a backup UUID. In case the load fails, this
-    // will be the UUID for our instnace.
+    // will be the UUID for our instance.
     string backup_uuid = gen.Next();
     unique_ptr<DirInstanceMetadataFile> instance(
         new DirInstanceMetadataFile(env_, std::move(backup_uuid), opts_.dir_type,
@@ -570,8 +827,8 @@ Status DirManager::Open() {
                   .set_max_threads(num_threads_per_dir_)
                   .set_trace_metric_prefix("dirs")
                   .Build(&pool));
-    unique_ptr<Dir> new_dir = CreateNewDir(env_, metrics_.get(), fs_type, dir, std::move(instance),
-                                           std::move(pool));
+    unique_ptr<Dir> new_dir = CreateNewDir(env_, metrics_.get(), fs_type, dir,
+                                           std::move(instance), std::move(pool));
     dirs.emplace_back(std::move(new_dir));
   }
 
@@ -616,7 +873,16 @@ Dir* DirManager::FindDirByUuidIndex(int uuid_idx) const {
   return FindPtrOrNull(dir_by_uuid_idx_, uuid_idx);
 }
 
-bool DirManager::FindUuidIndexByDir(Dir* dir, int* uuid_idx) const {
+Dir* DirManager::FindDirByFullPathForTests(const std::string& full_path) const {
+  for (const auto& d : dirs_) {
+    if (HasPrefixString(full_path, d->dir())) {
+      return d.get();
+    }
+  }
+  return nullptr;
+}
+
+bool DirManager::FindUuidIndexByDir(const Dir* dir, int* uuid_idx) const {
   return FindCopy(uuid_idx_by_dir_, dir, uuid_idx);
 }
 
@@ -654,7 +920,7 @@ void DirManager::MarkDirFailedByUuid(const std::string& uuid) {
 
 Status DirManager::MarkDirFailed(int uuid_idx, const string& error_message) {
   DCHECK_LT(uuid_idx, dirs_.size());
-  std::lock_guard<percpu_rwlock> lock(dir_group_lock_);
+  std::lock_guard lock(dir_group_lock_);
   Dir* dir = FindDirByUuidIndex(uuid_idx);
   DCHECK(dir);
   if (InsertIfNotPresent(&failed_dirs_, uuid_idx)) {
