@@ -21,12 +21,21 @@
 
 #include <functional>
 #include <memory>
-#include <mutex>
+#if !defined(NO_ROCKSDB)
+#include <optional>
+#endif
 #include <set>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <vector>
 
+#if !defined(NO_ROCKSDB)
+#include <gtest/gtest_prod.h>
+#include <rocksdb/db.h>
+#endif
+
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/util/locks.h"
@@ -35,6 +44,13 @@
 #include "kudu/util/random.h"
 #include "kudu/util/status.h"
 
+#if !defined(NO_ROCKSDB)
+namespace rocksdb {
+class Cache;
+class Status;
+} // namespace rocksdb
+#endif
+
 namespace kudu {
 
 class Env;
@@ -53,6 +69,10 @@ typedef std::vector<CanonicalizedRootAndStatus> CanonicalizedRootsList;
 
 namespace fs {
 
+static const std::string kDefaultTenantName = "default_tenant_kudu";
+static const std::string kDefaultTenantID = "00000000000000000000000000000000";
+static const std::string kRocksDBDirName = "rdb";
+
 typedef std::unordered_map<int, std::string> UuidByUuidIndexMap;
 typedef std::unordered_map<std::string, int> UuidIndexByUuidMap;
 class DirInstanceMetadataFile;
@@ -103,7 +123,7 @@ class Dir {
 
   // Shuts down this dir's thread pool, waiting for any closures submitted via
   // ExecClosure() to finish first.
-  void Shutdown();
+  virtual void Shutdown();
 
   // Run a task on this dir's thread pool.
   //
@@ -139,24 +159,24 @@ class Dir {
   }
 
   bool is_full() const {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     return is_full_;
   }
 
   int64_t available_bytes() {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     return available_bytes_;
   }
 
   // The amount of time to cache the amount of available space in this
   // directory.
-  virtual int available_space_cache_secs() const = 0;
+  static int available_space_cache_secs();
 
   // The number of bytes to reserve in each directory for non-Kudu usage. A
   // value of -1 means 1% of the disk space in a directory will be reserved.
-  virtual int reserved_bytes() const = 0;
+  static int reserved_bytes();
 
- private:
+ protected:
   Env* env_;
   DirMetrics* metrics_;
   const FsType fs_type_;
@@ -177,6 +197,47 @@ class Dir {
   DISALLOW_COPY_AND_ASSIGN(Dir);
 };
 
+#if !defined(NO_ROCKSDB)
+// Convert a rocksdb::Status to a kudu::Status.
+// NOTE: Keep it here rather than util/status.h because the latter is
+// an exported header, but FromRdbStatus() is used only internally.
+Status FromRdbStatus(const rocksdb::Status& s);
+
+// Representation of a directory for LogBlockManagerRdbMeta specially.
+class RdbDir: public Dir {
+ public:
+  RdbDir(Env* env,
+         DirMetrics* metrics,
+         FsType fs_type,
+         bool newly_created,
+         std::string dir,
+         std::unique_ptr<DirInstanceMetadataFile> metadata_file,
+         std::unique_ptr<ThreadPool> pool);
+
+  // Similar to Dir::Shutdown(), but close the RocksDB instance additionally.
+  void Shutdown() override;
+
+  rocksdb::DB* rdb();
+
+ private:
+  // Initialize the RocksDB instance for the directory.
+  //
+  // 'newly_created' indicates whether this is a newly created directory when
+  // opening DirManager.
+  // Returns Status::OK() if prepared successfully, otherwise returns non-OK.
+  Status InitRocksDBInstance(bool newly_created);
+
+  // The shared RocksDB instance for this directory.
+  std::unique_ptr<rocksdb::DB> db_;
+  // The RocksDB full path.
+  std::optional<std::string> rdb_dir_;
+  // The block cache shared by all RocksDB instances in all data directories.
+  static std::shared_ptr<rocksdb::Cache> s_block_cache_;
+
+  DISALLOW_COPY_AND_ASSIGN(RdbDir);
+};
+#endif
+
 struct DirManagerOptions {
  public:
   // The type of directory this directory manager should support.
@@ -184,6 +245,11 @@ struct DirManagerOptions {
   // Must not be empty.
   std::string dir_type;
 
+  // The id used to distinguish different tenants.
+  //
+  // Must not be empty.
+  std::string tenant_id;
+
   // The entity under which all metrics should be grouped. If null, metrics
   // will not be produced.
   //
@@ -202,7 +268,8 @@ struct DirManagerOptions {
   UpdateInstanceBehavior update_instances;
 
  protected:
-  explicit DirManagerOptions(const std::string& dir_type);
+  explicit DirManagerOptions(std::string dir_type,
+                             std::string tid);
 };
 
 class DirManager {
@@ -247,12 +314,12 @@ class DirManager {
   bool IsTabletInFailedDir(const std::string& tablet_id) const;
 
   std::set<int> GetFailedDirs() const {
-    shared_lock<rw_spinlock> group_lock(dir_group_lock_.get_lock());
+    std::shared_lock<rw_spinlock> group_lock(dir_group_lock_.get_lock());
     return failed_dirs_;
   }
 
   bool AreAllDirsFailed() const {
-    shared_lock<rw_spinlock> group_lock(dir_group_lock_.get_lock());
+    std::shared_lock<rw_spinlock> group_lock(dir_group_lock_.get_lock());
     return failed_dirs_.size() == dirs_.size();
   }
 
@@ -269,7 +336,7 @@ class DirManager {
   Dir* FindDirByUuidIndex(int uuid_idx) const;
 
   // Finds a uuid index by directory, returning false if it can't be found.
-  bool FindUuidIndexByDir(Dir* dir, int* uuid_idx) const;
+  bool FindUuidIndexByDir(const Dir* dir, int* uuid_idx) const;
 
   // Finds a uuid index by root path, returning false if it can't be found.
   bool FindUuidIndexByRoot(const std::string& root, int* uuid_idx) const;
@@ -284,6 +351,8 @@ class DirManager {
   // the given UUID index.
   std::set<std::string> FindTabletsByDirUuidIdx(int uuid_idx) const;
 
+  const std::string tenant_id() { return opts_.tenant_id; }
+
   // Create a new directory using the appropriate directory implementation.
   virtual std::unique_ptr<Dir> CreateNewDir(Env* env,
                                             DirMetrics* metrics,
@@ -293,6 +362,10 @@ class DirManager {
                                             std::unique_ptr<ThreadPool> pool) = 0;
 
  protected:
+#if !defined(NO_ROCKSDB)
+  FRIEND_TEST(LogBlockManagerRdbMetaTest, TestHalfPresentContainer);
+#endif
+
   // The name to be used by this directory manager for each sub-directory of
   // each directory root.
   virtual const char* dir_name() const = 0;
@@ -386,6 +459,11 @@ class DirManager {
       const std::vector<std::unique_ptr<DirInstanceMetadataFile>>& instances_to_update,
       const std::set<std::string>& new_all_uuids);
 
+  // Finds a directory by full path name, returning null if it can't be found.
+  //
+  // NOTE: Only for test purpose.
+  Dir* FindDirByFullPathForTests(const std::string& full_path) const;
+
   // The environment to be used for all directory operations.
   Env* env_;
 
@@ -399,6 +477,12 @@ class DirManager {
   // Common roots in the collections have been deduplicated.
   const CanonicalizedRootsList canonicalized_fs_roots_;
 
+  // Absolute paths of the FS directories created while opening the FSManager.
+  // This container is being populated when creating the initial file system
+  // layout with Create(), or by calling Open() on a newly added data directory.
+  // It's safe to use it after DirManager is opened completely.
+  std::set<std::string> created_fs_dir_paths_;
+
   // Directories tracked by this manager.
   std::vector<std::unique_ptr<Dir>> dirs_;
 
@@ -422,7 +506,7 @@ class DirManager {
   typedef std::unordered_map<int, Dir*> UuidIndexMap;
   UuidIndexMap dir_by_uuid_idx_;
 
-  typedef std::unordered_map<Dir*, int> ReverseUuidIndexMap;
+  typedef std::unordered_map<const Dir*, int> ReverseUuidIndexMap;
   ReverseUuidIndexMap uuid_idx_by_dir_;
 
   typedef std::unordered_map<int, std::set<std::string>> TabletsByUuidIndexMap;
