@@ -20,11 +20,11 @@
 #include <cstddef>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <numeric>
 #include <ostream>
 #include <set>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -46,7 +46,6 @@
 #include "kudu/gutil/strings/numbers.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/util/array_view.h"
-#include "kudu/util/atomic.h"
 #include "kudu/util/env.h"
 #include "kudu/util/env_util.h"
 #include "kudu/util/file_cache.h"
@@ -59,6 +58,7 @@
 #include "kudu/util/status.h"
 
 using std::accumulate;
+using std::atomic;
 using std::set;
 using std::shared_ptr;
 using std::string;
@@ -95,7 +95,7 @@ namespace internal {
 class FileBlockLocation {
  public:
   // Empty constructor
-  FileBlockLocation() {
+  FileBlockLocation() : data_dir_(nullptr) {
   }
 
   // Construct a location from its constituent parts.
@@ -232,33 +232,35 @@ void FileBlockLocation::GetAllParentDirs(vector<string>* parent_dirs) const {
 // at Close() time. Embedding a FileBlockLocation (and not a simpler
 // BlockId) consumes more memory, but the number of outstanding
 // FileWritableBlock instances is expected to be low.
-class FileWritableBlock : public WritableBlock {
+class FileWritableBlock final : public WritableBlock {
  public:
   FileWritableBlock(FileBlockManager* block_manager, FileBlockLocation location,
                     shared_ptr<WritableFile> writer);
 
-  virtual ~FileWritableBlock();
+  ~FileWritableBlock() override;
 
-  virtual Status Close() OVERRIDE;
+  Status Close() override;
 
-  virtual Status Abort() OVERRIDE;
+  Status Abort() override;
 
-  virtual BlockManager* block_manager() const OVERRIDE;
+  BlockManager* block_manager() const override;
 
-  virtual const BlockId& id() const OVERRIDE;
+  const BlockId& id() const override;
 
-  virtual Status Append(const Slice& data) OVERRIDE;
+  Status Append(const Slice& data) override;
 
-  virtual Status AppendV(ArrayView<const Slice> data) OVERRIDE;
+  Status AppendV(ArrayView<const Slice> data) override;
 
-  virtual Status Finalize() OVERRIDE;
+  Status Finalize() override;
 
-  virtual size_t BytesAppended() const OVERRIDE;
+  size_t BytesAppended() const override;
 
-  virtual State state() const OVERRIDE;
+  State state() const override;
 
   void HandleError(const Status& s) const;
 
+  string tenant_id() const { return block_manager_->tenant_id(); }
+
   // Starts an asynchronous flush of dirty block data to disk.
   Status FlushDataAsync();
 
@@ -314,7 +316,7 @@ FileWritableBlock::~FileWritableBlock() {
 void FileWritableBlock::HandleError(const Status& s) const {
   HANDLE_DISK_FAILURE(
       s, block_manager_->error_manager()->RunErrorNotificationCb(
-          ErrorHandlerType::DISK_ERROR, location_.data_dir()));
+          ErrorHandlerType::DISK_ERROR, location_.data_dir(), tenant_id()));
 }
 
 Status FileWritableBlock::Close() {
@@ -370,7 +372,7 @@ Status FileWritableBlock::Finalize() {
   VLOG(3) << "Finalizing block " << id();
   if (state_ == DIRTY &&
       FLAGS_block_manager_preflush_control == "finalize") {
-    FlushDataAsync();
+    RETURN_NOT_OK(FlushDataAsync());
   }
   state_ = FINALIZED;
   return Status::OK();
@@ -436,21 +438,21 @@ class FileReadableBlock : public ReadableBlock {
   FileReadableBlock(FileBlockManager* block_manager, BlockId block_id,
                     shared_ptr<RandomAccessFile> reader);
 
-  virtual ~FileReadableBlock();
+  ~FileReadableBlock() override;
 
-  virtual Status Close() OVERRIDE;
+  Status Close() override;
 
-  virtual BlockManager* block_manager() const OVERRIDE;
+  BlockManager* block_manager() const override;
 
-  virtual const BlockId& id() const OVERRIDE;
+  const BlockId& id() const override;
 
-  virtual Status Size(uint64_t* sz) const OVERRIDE;
+  Status Size(uint64_t* sz) const override;
 
-  virtual Status Read(uint64_t offset, Slice result) const OVERRIDE;
+  Status Read(uint64_t offset, Slice result) const override;
 
-  virtual Status ReadV(uint64_t offset, ArrayView<Slice> results) const OVERRIDE;
+  Status ReadV(uint64_t offset, ArrayView<Slice> results) const override;
 
-  virtual size_t memory_footprint() const OVERRIDE;
+  size_t memory_footprint() const override;
 
   void HandleError(const Status& s) const;
 
@@ -466,7 +468,7 @@ class FileReadableBlock : public ReadableBlock {
 
   // Whether or not this block has been closed. Close() is thread-safe, so
   // this must be an atomic primitive.
-  AtomicBool closed_;
+  atomic<bool> closed_;
 
   DISALLOW_COPY_AND_ASSIGN(FileReadableBlock);
 };
@@ -475,7 +477,7 @@ void FileReadableBlock::HandleError(const Status& s) const {
   const Dir* dir = block_manager_->dd_manager_->FindDirByUuidIndex(
       internal::FileBlockLocation::GetDirIdx(block_id_));
   HANDLE_DISK_FAILURE(s, block_manager_->error_manager()->RunErrorNotificationCb(
-      ErrorHandlerType::DISK_ERROR, dir));
+      ErrorHandlerType::DISK_ERROR, dir, block_manager_->dd_manager_->tenant_id()));
 }
 
 FileReadableBlock::FileReadableBlock(FileBlockManager* block_manager,
@@ -497,7 +499,8 @@ FileReadableBlock::~FileReadableBlock() {
 }
 
 Status FileReadableBlock::Close() {
-  if (closed_.CompareAndSet(false, true)) {
+  bool is_closed = false;
+  if (closed_.compare_exchange_strong(is_closed, true)) {
     reader_.reset();
     if (block_manager_->metrics_) {
       block_manager_->metrics_->blocks_open_reading->Decrement();
@@ -516,7 +519,7 @@ const BlockId& FileReadableBlock::id() const {
 }
 
 Status FileReadableBlock::Size(uint64_t* sz) const {
-  DCHECK(!closed_.Load());
+  DCHECK(!closed_);
 
   RETURN_NOT_OK_HANDLE_ERROR(reader_->Size(sz));
   *sz -= reader_->GetEncryptionHeaderSize();
@@ -528,7 +531,7 @@ Status FileReadableBlock::Read(uint64_t offset, Slice result) const {
 }
 
 Status FileReadableBlock::ReadV(uint64_t offset, ArrayView<Slice> results) const {
-  DCHECK(!closed_.Load());
+  DCHECK(!closed_);
 
   RETURN_NOT_OK_HANDLE_ERROR(reader_->ReadV(offset + reader_->GetEncryptionHeaderSize(), results));
 
@@ -557,18 +560,18 @@ class FileBlockCreationTransaction : public BlockCreationTransaction {
  public:
   FileBlockCreationTransaction() = default;
 
-  virtual ~FileBlockCreationTransaction() = default;
+  ~FileBlockCreationTransaction() override = default;
 
-  virtual void AddCreatedBlock(std::unique_ptr<WritableBlock> block) override;
+  void AddCreatedBlock(unique_ptr<WritableBlock> block) override;
 
-  virtual Status CommitCreatedBlocks() override;
+  Status CommitCreatedBlocks() override;
 
  private:
-  std::vector<std::unique_ptr<FileWritableBlock>> created_blocks_;
+  vector<unique_ptr<FileWritableBlock>> created_blocks_;
 };
 
 void FileBlockCreationTransaction::AddCreatedBlock(
-    std::unique_ptr<WritableBlock> block) {
+    unique_ptr<WritableBlock> block) {
   FileWritableBlock* fwb =
       down_cast<FileWritableBlock*>(block.release());
   created_blocks_.emplace_back(unique_ptr<FileWritableBlock>(fwb));
@@ -607,16 +610,16 @@ class FileBlockDeletionTransaction : public BlockDeletionTransaction {
       : fbm_(fbm) {
   }
 
-  virtual ~FileBlockDeletionTransaction() = default;
+  ~FileBlockDeletionTransaction() override = default;
 
-  virtual void AddDeletedBlock(BlockId block) override;
+  void AddDeletedBlock(BlockId block) override;
 
-  virtual Status CommitDeletedBlocks(std::vector<BlockId>* deleted) override;
+  Status CommitDeletedBlocks(vector<BlockId>* deleted) override;
 
  private:
   // The owning FileBlockManager. Must outlive the FileBlockDeletionTransaction.
   FileBlockManager* fbm_;
-  std::vector<BlockId> deleted_blocks_;
+  vector<BlockId> deleted_blocks_;
   DISALLOW_COPY_AND_ASSIGN(FileBlockDeletionTransaction);
 };
 
@@ -624,16 +627,22 @@ void FileBlockDeletionTransaction::AddDeletedBlock(BlockId block) {
   deleted_blocks_.emplace_back(block);
 }
 
-Status FileBlockDeletionTransaction::CommitDeletedBlocks(std::vector<BlockId>* deleted) {
-  deleted->clear();
+Status FileBlockDeletionTransaction::CommitDeletedBlocks(vector<BlockId>* deleted) {
+  if (deleted) {
+    deleted->clear();
+  }
   Status first_failure;
+  uint64_t deleted_blocks_count = 0;
   for (BlockId block : deleted_blocks_) {
     Status s = fbm_->DeleteBlock(block);
     // If we get NotFound, then the block was already deleted.
     if (!s.ok() && !s.IsNotFound()) {
       if (first_failure.ok()) first_failure = s;
     } else {
-      deleted->emplace_back(block);
+      ++deleted_blocks_count;
+      if (deleted) {
+        deleted->emplace_back(block);
+      }
       if (s.ok() && fbm_->metrics_) {
         fbm_->metrics_->total_blocks_deleted->Increment();
       }
@@ -641,9 +650,10 @@ Status FileBlockDeletionTransaction::CommitDeletedBlocks(std::vector<BlockId>* d
   }
 
   if (!first_failure.ok()) {
-    first_failure = first_failure.CloneAndPrepend(strings::Substitute("only deleted $0 blocks, "
-                                                                      "first failure",
-                                                                      deleted->size()));
+    first_failure = first_failure.CloneAndPrepend(
+        Substitute("only $0/$1 blocks deleted, first failure",
+                   deleted_blocks_count,
+                   deleted_blocks_.size()));
   }
   deleted_blocks_.clear();
   return first_failure;
@@ -662,7 +672,7 @@ Status FileBlockManager::SyncMetadata(const internal::FileBlockLocation& locatio
   // Figure out what directories to sync.
   vector<string> to_sync;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     for (const string& parent_dir : parent_dirs) {
       if (dirty_dirs_.erase(parent_dir)) {
         to_sync.push_back(parent_dir);
@@ -676,7 +686,8 @@ Status FileBlockManager::SyncMetadata(const internal::FileBlockLocation& locatio
       if (metrics_) metrics_->total_disk_sync->Increment();
       RETURN_NOT_OK_HANDLE_DISK_FAILURE(env_->SyncDir(s),
           error_manager_->RunErrorNotificationCb(ErrorHandlerType::DISK_ERROR,
-                                                 location.data_dir()));
+                                                 location.data_dir(),
+                                                 tenant_id()));
     }
   }
   return Status::OK();
@@ -694,20 +705,22 @@ bool FileBlockManager::FindBlockPath(const BlockId& block_id,
 }
 
 FileBlockManager::FileBlockManager(Env* env,
-                                   DataDirManager* dd_manager,
-                                   FsErrorManager* error_manager,
+                                   scoped_refptr<DataDirManager> dd_manager,
+                                   scoped_refptr<FsErrorManager> error_manager,
                                    FileCache* file_cache,
-                                   BlockManagerOptions opts)
+                                   BlockManagerOptions opts,
+                                   string tenant_id)
   : env_(DCHECK_NOTNULL(env)),
-    dd_manager_(dd_manager),
-    error_manager_(DCHECK_NOTNULL(error_manager)),
+    dd_manager_(std::move(dd_manager)),
+    error_manager_(DCHECK_NOTNULL(std::move(error_manager))),
     opts_(std::move(opts)),
     file_cache_(file_cache),
     rand_(GetRandomSeed32()),
     next_block_id_(rand_.Next64()),
     mem_tracker_(MemTracker::CreateTracker(-1,
                                            "file_block_manager",
-                                           opts_.parent_mem_tracker)) {
+                                           opts_.parent_mem_tracker)),
+    tenant_id_(std::move(tenant_id)) {
   if (opts_.metric_entity) {
     metrics_.reset(new internal::BlockManagerMetrics(opts_.metric_entity));
   }
@@ -716,8 +729,9 @@ FileBlockManager::FileBlockManager(Env* env,
 FileBlockManager::~FileBlockManager() {
 }
 
-Status FileBlockManager::Open(FsReport* report, std::atomic<int>* containers_processed,
-                              std::atomic<int>* containers_total) {
+Status FileBlockManager::Open(FsReport* report, MergeReport need_merage,
+                              atomic<int>* /* containers_processed */,
+                              atomic<int>* /* containers_total */) {
   // Prepare the filesystem report and either return or log it.
   FsReport local_report;
   set<int> failed_dirs = dd_manager_->GetFailedDirs();
@@ -737,7 +751,11 @@ Status FileBlockManager::Open(FsReport* report, std::atomic<int>* containers_pro
     local_report.data_dirs.push_back(dd->dir());
   }
   if (report) {
-    *report = std::move(local_report);
+    if (need_merage == MergeReport::REQUIRED) {
+      report->MergeFrom(local_report);
+    } else {
+      *report = std::move(local_report);
+    }
   } else {
     RETURN_NOT_OK(local_report.LogAndCheckForFatalErrors());
   }
@@ -746,11 +764,13 @@ Status FileBlockManager::Open(FsReport* report, std::atomic<int>* containers_pro
 
 Status FileBlockManager::CreateBlock(const CreateBlockOptions& opts,
                                      unique_ptr<WritableBlock>* block) {
-  CHECK(!opts_.read_only);
+  DCHECK(!opts_.read_only);
 
   Dir* dir;
   RETURN_NOT_OK_EVAL(dd_manager_->GetDirAddIfNecessary(opts, &dir),
-      error_manager_->RunErrorNotificationCb(ErrorHandlerType::NO_AVAILABLE_DISKS, opts.tablet_id));
+      error_manager_->RunErrorNotificationCb(ErrorHandlerType::NO_AVAILABLE_DISKS,
+                                             opts.tablet_id,
+                                             tenant_id()));
   int uuid_idx;
   CHECK(dd_manager_->FindUuidIndexByDir(dir, &uuid_idx));
 
@@ -768,14 +788,14 @@ Status FileBlockManager::CreateBlock(const CreateBlockOptions& opts,
     // If we failed to generate a unique ID, start trying again from a random
     // part of the key space.
     if (attempt_num++ > 0) {
-      next_block_id_.Store(rand_.Next64());
+      next_block_id_ = rand_.Next64();
     }
 
     // Make sure we don't accidentally create a location using the magic
     // invalid ID value.
     BlockId id;
     do {
-      id.SetId(next_block_id_.Increment());
+      id.SetId(next_block_id_++);
     } while (id.IsNull());
 
     location = internal::FileBlockLocation::FromParts(dir, uuid_idx, id);
@@ -786,7 +806,8 @@ Status FileBlockManager::CreateBlock(const CreateBlockOptions& opts,
     // no point in doing so. On disk failure, the tablet specified by 'opts'
     // will be shut down, so the returned block would not be used.
     RETURN_NOT_OK_HANDLE_DISK_FAILURE(s,
-        error_manager_->RunErrorNotificationCb(ErrorHandlerType::DISK_ERROR, dir));
+        error_manager_->RunErrorNotificationCb(ErrorHandlerType::DISK_ERROR,
+                                               dir, tenant_id()));
     WritableFileOptions wr_opts;
     wr_opts.mode = Env::MUST_CREATE;
     wr_opts.is_sensitive = true;
@@ -798,7 +819,7 @@ Status FileBlockManager::CreateBlock(const CreateBlockOptions& opts,
       // Update dirty_dirs_ with those provided as well as the block's
       // directory, which may not have been created but is definitely dirty
       // (because we added a file to it).
-      std::lock_guard<simple_spinlock> l(lock_);
+      std::lock_guard l(lock_);
       for (const string& created : created_dirs) {
         dirty_dirs_.insert(created);
       }
@@ -807,7 +828,8 @@ Status FileBlockManager::CreateBlock(const CreateBlockOptions& opts,
     block->reset(new internal::FileWritableBlock(this, location, writer));
   } else {
     HANDLE_DISK_FAILURE(s,
-        error_manager_->RunErrorNotificationCb(ErrorHandlerType::DISK_ERROR, dir));
+        error_manager_->RunErrorNotificationCb(ErrorHandlerType::DISK_ERROR,
+                                               dir, tenant_id()));
     return s;
   }
   return Status::OK();
@@ -816,8 +838,9 @@ Status FileBlockManager::CreateBlock(const CreateBlockOptions& opts,
 #define RETURN_NOT_OK_FBM_DISK_FAILURE(status_expr) do { \
   RETURN_NOT_OK_HANDLE_DISK_FAILURE((status_expr), \
       error_manager_->RunErrorNotificationCb(ErrorHandlerType::DISK_ERROR, \
-      dd_manager_->FindDirByUuidIndex( \
-      internal::FileBlockLocation::GetDirIdx(block_id)))); \
+          dd_manager_->FindDirByUuidIndex( \
+              internal::FileBlockLocation::GetDirIdx(block_id)), \
+          tenant_id())); \
 } while (0)
 
 Status FileBlockManager::OpenBlock(const BlockId& block_id,
@@ -846,7 +869,7 @@ Status FileBlockManager::OpenBlock(const BlockId& block_id,
 }
 
 Status FileBlockManager::DeleteBlock(const BlockId& block_id) {
-  CHECK(!opts_.read_only);
+  DCHECK(!opts_.read_only);
 
   // Return early if deleting a block in a failed directory.
   set<int> failed_dirs = dd_manager_->GetFailedDirs();
@@ -883,13 +906,13 @@ Status FileBlockManager::DeleteBlock(const BlockId& block_id) {
 }
 
 unique_ptr<BlockCreationTransaction> FileBlockManager::NewCreationTransaction() {
-  CHECK(!opts_.read_only);
+  DCHECK(!opts_.read_only);
   return unique_ptr<internal::FileBlockCreationTransaction>(
       new internal::FileBlockCreationTransaction());
 }
 
 shared_ptr<BlockDeletionTransaction> FileBlockManager::NewDeletionTransaction() {
-  CHECK(!opts_.read_only);
+  DCHECK(!opts_.read_only);
   return std::make_shared<internal::FileBlockDeletionTransaction>(this);
 }
 
