@@ -173,6 +173,28 @@ struct LBMPartialRecordCheck {
   std::vector<Entry> entries;
 };
 
+#if !defined(NO_ROCKSDB)
+// Checks for corrupted LBM metadata records in RocksDB.
+//
+// Error type: non-fatal and repairable (by removing the records from RocksDB).
+struct LBMCorruptedRdbRecordCheck {
+
+  // Merges the contents of another check into this one.
+  void MergeFrom(const LBMCorruptedRdbRecordCheck& other);
+
+  // Returns a multi-line string representation of this check.
+  std::string ToString() const;
+
+  struct Entry {
+    Entry(std::string c, std::string k);
+    std::string container;
+    std::string rocksdb_key;
+    bool repaired;
+  };
+  std::vector<Entry> entries;
+};
+#endif
+
 // Results of a Kudu filesystem-wide check. The report contains general
 // statistics about the filesystem as well as a series of "checks" that
 // describe possible on-disk inconsistencies.
@@ -268,6 +290,9 @@ struct FsReport {
   std::optional<LBMMalformedRecordCheck> malformed_record_check;
   std::optional<LBMMisalignedBlockCheck> misaligned_block_check;
   std::optional<LBMPartialRecordCheck> partial_record_check;
+#if !defined(NO_ROCKSDB)
+  std::optional<LBMCorruptedRdbRecordCheck> corrupted_rdb_record_check;
+#endif
 };
 
 } // namespace fs
