@@ -22,6 +22,7 @@
 #include <ostream>
 #include <random>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -601,7 +602,7 @@ class AllTypesItest : public KuduTest {
       int total_rows_in_tablet = 0;
       while (scanner.HasMoreRows()) {
         KuduScanBatch batch;
-        scanner.NextBatch(&batch);
+        RETURN_NOT_OK(scanner.NextBatch(&batch));
         verifier(batch, i, &total_rows_in_tablet);
       }
       CHECK_EQ(total_rows_in_tablet, setup_.GetRowsPerTablet());
@@ -623,7 +624,7 @@ class AllTypesItest : public KuduTest {
     ASSERT_OK(VerifyRows(scanner_setup, verifier));
   }
 
-  virtual void TearDown() OVERRIDE {
+  void TearDown() override {
     cluster_->AssertNoCrashes();
     cluster_->Shutdown();
   }
