@@ -18,6 +18,7 @@
 // Integration test for flexible partitioning (eg buckets, range partitioning
 // of PK subsets, etc).
 
+#include <functional>
 #include <memory>
 #include <ostream>
 #include <string>
@@ -38,6 +39,7 @@
 #include "kudu/common/wire_protocol.h"
 #include "kudu/consensus/metadata.pb.h"
 #include "kudu/gutil/strings/substitute.h"
+#include "kudu/integration-tests/cluster_verifier.h"
 #include "kudu/mini-cluster/external_mini_cluster.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/tablet/tablet.pb.h"
@@ -252,7 +254,7 @@ TEST_F(AutoIncrementingItest, BasicInserts) {
     rpc::RpcController rpc;
     tserver::ListTabletsRequestPB req;
     tserver::ListTabletsResponsePB resp;
-    server->ListTablets(req, &resp, &rpc);
+    ASSERT_OK(server->ListTablets(req, &resp, &rpc));
     ASSERT_EQ(1, resp.status_and_schema_size());
     vector<string> results;
     ASSERT_OK(ScanTablet(j, resp.status_and_schema(0).tablet_status().tablet_id(), &results));
@@ -271,7 +273,7 @@ TEST_F(AutoIncrementingItest, BasicInserts) {
     rpc::RpcController rpc;
     tserver::ListTabletsRequestPB req;
     tserver::ListTabletsResponsePB resp;
-    server->ListTablets(req, &resp, &rpc);
+    ASSERT_OK(server->ListTablets(req, &resp, &rpc));
     ASSERT_EQ(1, resp.status_and_schema_size());
     vector<string> results;
     ASSERT_OK(ScanTablet(j, resp.status_and_schema(0).tablet_status().tablet_id(), &results));
@@ -299,7 +301,7 @@ TEST_F(AutoIncrementingItest, BasicUpserts) {
     rpc::RpcController rpc;
     tserver::ListTabletsRequestPB req;
     tserver::ListTabletsResponsePB resp;
-    server->ListTablets(req, &resp, &rpc);
+    ASSERT_OK(server->ListTablets(req, &resp, &rpc));
     ASSERT_EQ(1, resp.status_and_schema_size());
     vector<string> results;
     ASSERT_OK(ScanTablet(j, resp.status_and_schema(0).tablet_status().tablet_id(), &results));
@@ -317,7 +319,7 @@ TEST_F(AutoIncrementingItest, BasicUpserts) {
     rpc::RpcController rpc;
     tserver::ListTabletsRequestPB req;
     tserver::ListTabletsResponsePB resp;
-    server->ListTablets(req, &resp, &rpc);
+    ASSERT_OK(server->ListTablets(req, &resp, &rpc));
     ASSERT_EQ(1, resp.status_and_schema_size());
     vector<string> results;
     ASSERT_OK(ScanTablet(j, resp.status_and_schema(0).tablet_status().tablet_id(), &results));
@@ -351,7 +353,7 @@ TEST_F(AutoIncrementingItest, TestNegatives) {
 
 TEST_F(AutoIncrementingItest, BootstrapWithNoWals) {
   string tablet_uuid;
-  TestSetup(&tablet_uuid);
+  ASSERT_OK(TestSetup(&tablet_uuid));
 
   // Delete the first 100 rows to make sure at least latest 2 wals
   // are not insert operations.
@@ -379,9 +381,13 @@ TEST_F(AutoIncrementingItest, BootstrapWithNoWals) {
   // Restart the cluster.
   cluster_->Shutdown();
   ASSERT_OK(cluster_->Restart());
+  // Ensure that the tablet is running and leader elected.
+  ASSERT_EVENTUALLY([&] { ASSERT_OK(ClusterVerifier(cluster_.get()).RunKsck()); });
 
   // Insert new data and validate the auto incrementing column values.
   ASSERT_OK(InsertData(kNumRows, kNumRows * 2));
+  // Wait for all the replicas to converge.
+  NO_FATALS(ClusterVerifier(cluster_.get()).CheckCluster());
   for (int j = 0; j < kNumTabletServers; j++) {
     vector<string> results;
     ASSERT_OK(ScanTablet(j, tablet_uuid, &results));
@@ -389,15 +395,13 @@ TEST_F(AutoIncrementingItest, BootstrapWithNoWals) {
     for (int i = 0; i < results.size(); i++) {
       ASSERT_EQ(Substitute("(int32 c0=$0, int64 $1=$2, string c1=\"string_val\")", i + 100,
                            Schema::GetAutoIncrementingColumnName(), i + 100 + 1), results[i]);
-
     }
   }
 }
 
-
 TEST_F(AutoIncrementingItest, BootstrapNoWalsNoData) {
   string tablet_uuid;
-  TestSetup(&tablet_uuid);
+  ASSERT_OK(TestSetup(&tablet_uuid));
 
   // Delete all the rows.
   for (int i = 0; i < kNumRows; i++) {
@@ -437,13 +441,17 @@ TEST_F(AutoIncrementingItest, BootstrapNoWalsNoData) {
     cluster_->tablet_server(i)->Shutdown();
     ASSERT_OK(cluster_->tablet_server(i)->Restart());
   }
+  // Ensure that the tablet is running and leader elected.
+  ASSERT_EVENTUALLY([&] { ASSERT_OK(ClusterVerifier(cluster_.get()).RunKsck()); });
 
   // Insert new data and verify auto_incrementing_id starts from 1.
   ASSERT_OK(InsertData(kNumRows, kNumRows * 2));
+  // Wait for all the replicas to converge.
+  NO_FATALS(ClusterVerifier(cluster_.get()).CheckCluster());
   for (int j = 0; j < kNumTabletServers; j++) {
     vector<string> results;
     ASSERT_OK(ScanTablet(j, tablet_uuid, &results));
-    ASSERT_EQ(200, results.size());
+    ASSERT_EQ(kNumRows, results.size());
     for (int i = 0; i < results.size(); i++) {
       ASSERT_EQ(Substitute("(int32 c0=$0, int64 $1=$2, string c1=\"string_val\")", i + kNumRows,
                            Schema::GetAutoIncrementingColumnName(), i + 1), results[i]);
@@ -468,7 +476,7 @@ TEST_F(AutoIncrementingItest, BootstrapWalsDiverge) {
     rpc::RpcController rpc;
     tserver::ListTabletsRequestPB req;
     tserver::ListTabletsResponsePB resp;
-    cluster_->tserver_proxy(i)->ListTablets(req, &resp, &rpc);
+    ASSERT_OK(cluster_->tserver_proxy(i)->ListTablets(req, &resp, &rpc));
     ASSERT_EQ(1, resp.status_and_schema_size());
     if (tablet_uuid.empty()) {
       tablet_uuid = resp.status_and_schema(0).tablet_status().tablet_id();
@@ -503,7 +511,7 @@ TEST_F(AutoIncrementingItest, BootstrapWalsDiverge) {
   // Write 200 rows at the rate of 1 row every 5ms which are sent to the leader replica. After
   // 100ms of starting to insert data, we shutdown the followers and at this point the write
   // request is expected to 900ms more. Since the leader would mark the followers as
-  // unavailable after 3 lost hearbeats (1500ms), there will for sure be a situation where the
+  // unavailable after 3 lost heartbeats (1500ms), there will for sure be a situation where the
   // leader has sent a write op and hasn't gotten the response from majority-1 number of
   // followers. In this case the write op is not marked committed in the leader replica. All
   // the writes including this are considered failed.
@@ -518,16 +526,14 @@ TEST_F(AutoIncrementingItest, BootstrapWalsDiverge) {
   ASSERT_OK(InsertData(kNumRows, kNumRows * 2));
 
   // Start back the tablet server and ensure all the replicas contain the same data.
-  // We scan the replica on leader_ts_index last to give it reasonable time to start
-  // serving scans.
   ASSERT_OK(cluster_->tablet_server(leader_ts_index)->Restart());
-  int j = (leader_ts_index + 1) % kNumTabletServers;
+  // Wait for all the replicas to converge.
+  NO_FATALS(ClusterVerifier(cluster_.get()).CheckCluster());
   vector<vector<string>> results;
   for (int i = 0; i < kNumTabletServers; i++) {
     vector<string> result;
-    ASSERT_OK(ScanTablet(j, tablet_uuid, &result));
+    ASSERT_OK(ScanTablet(i, tablet_uuid, &result));
     results.emplace_back(result);
-    j = (j + 1) % kNumTabletServers;
   }
   ASSERT_EQ(kNumTabletServers, results.size());
   for (int i = 0; i < kNumTabletServers - 1; i++) {
