@@ -21,7 +21,9 @@
 #include <cstdint>
 #include <memory>
 #include <ostream>
+#include <set>
 #include <string>
+#include <tuple>
 #include <utility>
 #include <vector>
 
@@ -30,6 +32,8 @@
 #include <gtest/gtest.h>
 
 #include "kudu/client/schema.h"
+#include "kudu/fs/block_manager.h"
+#include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/integral_types.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/integration-tests/cluster_itest_util.h"
@@ -39,6 +43,7 @@
 #include "kudu/util/env.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/os-util.h"
 #include "kudu/util/status.h"
 #include "kudu/util/stopwatch.h"
 #include "kudu/util/test_macros.h"
@@ -55,6 +60,7 @@ METRIC_DECLARE_gauge_uint64(threads_running);
 
 DECLARE_bool(enable_rowset_compaction);
 DECLARE_string(block_manager);
+DECLARE_string(block_cache_eviction_policy);
 
 DEFINE_bool(measure_startup_drop_caches, false,
             "Whether to drop kernel caches before measuring startup time. Must be root");
@@ -75,6 +81,7 @@ using client::KuduColumnSchema;
 using client::KuduSchema;
 using client::KuduSchemaBuilder;
 using cluster::ExternalMiniClusterOptions;
+using kudu::fs::BlockManager;
 using std::pair;
 using std::string;
 using std::unique_ptr;
@@ -83,10 +90,14 @@ using strings::Substitute;
 
 class DenseNodeTest :
   public ExternalMiniClusterITestBase,
-  public testing::WithParamInterface<bool> {
+  public testing::WithParamInterface<std::tuple<bool, string, string>> {
 };
 
-INSTANTIATE_TEST_SUITE_P(, DenseNodeTest, testing::Values(false, true));
+INSTANTIATE_TEST_SUITE_P(, DenseNodeTest,
+                         ::testing::Combine(
+                             ::testing::Bool(),
+                             ::testing::ValuesIn(BlockManager::block_manager_types()),
+                             ::testing::Values("LRU", "SLRU")));
 
 // Integration test that simulates "dense" Kudu nodes.
 //
@@ -97,6 +108,10 @@ INSTANTIATE_TEST_SUITE_P(, DenseNodeTest, testing::Values(false, true));
 // proxy for data in areas we care about (such as start up time, thread count,
 // memory usage, etc.).
 TEST_P(DenseNodeTest, RunTest) {
+  #ifdef THREAD_SANITIZER
+    SKIP_IF_SLOW_NOT_ALLOWED();
+  #endif
+
   ExternalMiniClusterOptions opts;
 
   opts.extra_tserver_flags = {
@@ -152,11 +167,23 @@ TEST_P(DenseNodeTest, RunTest) {
     opts.extra_master_flags.emplace_back("--never_fsync=false");
   }
 
-  if (GetParam()) {
+  const auto& setup = GetParam();
+
+  if (std::get<0>(setup)) {
     opts.extra_master_flags.emplace_back("--encrypt_data_at_rest=true");
     opts.extra_tserver_flags.emplace_back("--encrypt_data_at_rest=true");
   }
 
+  FLAGS_block_manager = std::get<1>(setup);
+  opts.extra_master_flags.emplace_back(Substitute("--block_manager=$0", FLAGS_block_manager));
+  opts.extra_tserver_flags.emplace_back(Substitute("--block_manager=$0", FLAGS_block_manager));
+
+  FLAGS_block_cache_eviction_policy = std::get<2>(setup);
+  opts.extra_master_flags.emplace_back(Substitute("--block_cache_eviction_policy=$0",
+                                                  FLAGS_block_cache_eviction_policy));
+  opts.extra_tserver_flags.emplace_back(Substitute("--block_cache_eviction_policy=$0",
+                                                   FLAGS_block_cache_eviction_policy));
+
   // With the amount of data we're going to write, we need to make sure the
   // tserver has enough time to start back up (startup is only considered to be
   // "complete" when the tserver has loaded all fs metadata from disk).
@@ -199,7 +226,7 @@ TEST_P(DenseNodeTest, RunTest) {
   // metrics are logged so that they're easier to find in the log output.
   vector<pair<string, int64_t>> metrics;
   vector<GaugePrototype<uint64>*> metric_prototypes;
-  if (FLAGS_block_manager == "log") {
+  if (FsManager::IsLogType(FLAGS_block_manager)) {
     metric_prototypes = { &METRIC_log_block_manager_blocks_under_management,
                           &METRIC_log_block_manager_bytes_under_management,
                           &METRIC_log_block_manager_containers,
@@ -236,9 +263,7 @@ TEST_P(DenseNodeTest, RunTest) {
       unique_ptr<WritableFile> f;
       WritableFileOptions opts;
       opts.mode = Env::MUST_EXIST;
-      ASSERT_OK(env_->NewWritableFile(opts, "/proc/sys/vm/drop_caches", &f));
-      ASSERT_OK(f->Append("3\n"));
-      ASSERT_OK(f->Close());
+      ASSERT_OK(FreeSlabObjectsAndPagecache());
     }
   }
 
@@ -251,9 +276,9 @@ TEST_P(DenseNodeTest, RunTest) {
   if (FLAGS_measure_startup_wait_for_bootstrap) {
     LOG_TIMING(INFO, "bootstrapping tablets") {
       LOG(INFO) << "waiting for tablets running";
-      cluster_->WaitForTabletsRunning(cluster_->tablet_server(0),
-                                      FLAGS_num_tablets,
-                                      MonoDelta::FromSeconds(3600));
+      ASSERT_OK(cluster_->WaitForTabletsRunning(cluster_->tablet_server(0),
+                                                FLAGS_num_tablets,
+                                                MonoDelta::FromSeconds(3600)));
     }
   } else {
     LOG(INFO) << "not waiting for bootstrapping tablets (flag disabled)";
