@@ -29,6 +29,7 @@
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/mini-cluster/external_mini_cluster.h"
 #include "kudu/util/env.h"
+#include "kudu/util/monotime.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
@@ -53,6 +54,13 @@ static int64_t CountInfoLogs(const string& log_dir) {
 
 // Tests that logs roll on startup, and get cleaned up appropriately.
 TEST_F(LogRollingITest, TestLogCleanupOnStartup) {
+#if defined(ADDRESS_SANITIZER)
+  LOG(WARNING) << "skip asan, because thirdparty lib glog has a global variable: (static "
+                  "vector<string>* logging_directories_list;) which allocated by new "
+                  "operator, but ShutdownGoogleLoggingSafe() isn't called as needed, "
+                  "which may cause asan error.";
+  GTEST_SKIP();
+#endif
   ExternalMiniClusterOptions opts;
   opts.num_masters = 1;
   opts.num_tablet_servers = 0;
@@ -70,9 +78,10 @@ TEST_F(LogRollingITest, TestLogCleanupOnStartup) {
   ASSERT_OK(cluster.master()->WaitForCatalogManager());
 
   for (int i = 1; i <= 10; i++) {
-    ASSERT_EVENTUALLY([&] () {
+    AssertEventually([&] () {
         ASSERT_EQ(std::min(3, i), CountInfoLogs(cluster.master()->log_dir()));
-    });
+    }, MonoDelta::FromSeconds(60));
+    NO_PENDING_FATALS();
     cluster.master()->Shutdown();
     ASSERT_OK(cluster.master()->Restart());
   }
