@@ -18,11 +18,11 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <ostream>
 #include <string>
 #include <thread>
 #include <tuple>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -49,7 +49,6 @@
 #include "kudu/master/master.proxy.h"
 #include "kudu/mini-cluster/external_mini_cluster.h"
 #include "kudu/rpc/messenger.h"
-#include "kudu/rpc/response_callback.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/thrift/client.h"
@@ -436,8 +435,8 @@ class MasterStressTest : public ExternalMiniClusterITestBase,
     MessengerBuilder bld("RestartMasterMessenger");
     CHECK_OK(bld.Build(&messenger));
 
-    MonoTime deadline(MonoTime::Now());
-    deadline.AddDelta(MonoDelta::FromSeconds(FLAGS_num_seconds_to_run));
+    const auto deadline = MonoTime::Now() +
+        MonoDelta::FromSeconds(FLAGS_num_seconds_to_run);
 
     MonoTime now(MonoTime::Now());
     while (now < deadline) {
@@ -495,7 +494,7 @@ class MasterStressTest : public ExternalMiniClusterITestBase,
   }
 
   bool BlockingGetTableName(string* chosen_table) {
-    std::lock_guard<Mutex> l(table_names_lock_);
+    std::lock_guard l(table_names_lock_);
     while (table_names_.empty() && done_.count()) {
       table_names_condvar_.Wait();
     }
@@ -516,7 +515,7 @@ class MasterStressTest : public ExternalMiniClusterITestBase,
   }
 
   void PutTableName(const string& table_name) {
-    std::lock_guard<Mutex> l(table_names_lock_);
+    std::lock_guard l(table_names_lock_);
     table_names_.push_back(table_name);
     table_names_condvar_.Signal();
   }
