@@ -18,6 +18,7 @@
 #include "kudu/master/auto_leader_rebalancer.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <map>
@@ -47,6 +48,7 @@
 #include "kudu/master/ts_manager.h"
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/rpc_controller.h"
+#include "kudu/security/init.h"
 #include "kudu/util/cow_object.h"
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/monotime.h"
@@ -85,6 +87,11 @@ DEFINE_uint32(leader_rebalancing_max_moves_per_round, 10,
 TAG_FLAG(leader_rebalancing_max_moves_per_round, advanced);
 TAG_FLAG(leader_rebalancing_max_moves_per_round, runtime);
 
+DEFINE_bool(leader_rebalancing_ignore_soft_deleted_tables, false,
+            "Whether to ignore rebalancing the soft deleted tables");
+TAG_FLAG(leader_rebalancing_ignore_soft_deleted_tables, advanced);
+TAG_FLAG(leader_rebalancing_ignore_soft_deleted_tables, runtime);
+
 DECLARE_bool(auto_leader_rebalancing_enabled);
 
 namespace kudu {
@@ -99,11 +106,19 @@ AutoLeaderRebalancerTask::AutoLeaderRebalancerTask(CatalogManager* catalog_manag
       number_of_loop_iterations_for_test_(0),
       moves_scheduled_this_round_for_test_(0) {}
 
-AutoLeaderRebalancerTask::~AutoLeaderRebalancerTask() { Shutdown(); }
+AutoLeaderRebalancerTask::~AutoLeaderRebalancerTask() {
+  if (thread_) {
+    Shutdown();
+  }
+}
 
 Status AutoLeaderRebalancerTask::Init() {
   DCHECK(!thread_) << "AutoleaderRebalancerTask is already initialized";
-  RETURN_NOT_OK(MessengerBuilder("auto-leader-rebalancer").Build(&messenger_));
+  MessengerBuilder builder("auto-leader-rebalancer");
+  if (auto username = kudu::security::GetLoggedInUsernameFromKeytab()) {
+    builder.set_sasl_proto_name(username.value());
+  }
+  RETURN_NOT_OK(std::move(builder).Build(&messenger_));
   return Thread::Create("catalog manager", "auto-leader-rebalancer",
                         [this]() { this->RunLoop(); }, &thread_);
 }
@@ -201,29 +216,43 @@ Status AutoLeaderRebalancerTask::RunLeaderRebalanceForTable(
   map<string, std::pair<int32_t, int32_t>> replica_and_leader_count_by_ts_uuid;
   // uuid->leader should transfer count
   map<string, int32_t> leader_transfer_source;
+  size_t remaining_tablets = tablet_infos.size();
+  size_t remaining_tservers = tserver_uuids.size();
   for (const auto& uuid : tserver_uuids) {
     auto* tablet_ids_ptr = FindOrNull(tablet_ids_by_ts_uuid, uuid);
-    uint32_t replica_count = tablet_ids_ptr ? tablet_ids_ptr->size() : 0;
+    int32_t replica_count = tablet_ids_ptr ? tablet_ids_ptr->size() : 0;
     if (replica_count == 0) {
       // means no replicas (and no leaders), maybe a tserver joined kudu cluster just now, skip it
+      remaining_tservers--;
       continue;
     }
     auto* leader_tablet_ids_ptr = FindOrNull(leader_tablet_ids_by_ts_uuid, uuid);
-    uint32_t leader_count = leader_tablet_ids_ptr ? leader_tablet_ids_ptr->size() : 0;
+    int32_t leader_count = leader_tablet_ids_ptr ? leader_tablet_ids_ptr->size() : 0;
     replica_and_leader_count_by_ts_uuid.insert(
         {uuid, std::pair<int32_t, int32_t>(replica_count, leader_count)});
     VLOG(1) << Substitute(
         "uuid: $0, replica_count: $1, leader_count: $2", uuid, replica_count, leader_count);
 
-    // Our target is every tserver' replicas, number of leader : number of follower is
-    // 1 : (replica_refactor -1). The constant 1 is a coarse-grained correction factor to help
-    // leader rebalancer to converge stable.
-    int32_t should_transfer_count = static_cast<int32_t>(leader_count) -
-                                    (static_cast<int32_t>(replica_count) / replication_factor + 1);
+    // If the number of remaining tablets is divisible by the number of remaining tablet
+    // servers, the leader num of all the remaining tablet servers should be the division
+    // result.
+    // Else, the maximum number of leader replicas per tablet server should be the ceil value
+    // of the average leaders num. Transfer the excess leaders if a tablet server's
+    // leader num is more than that.
+    const uint32_t target_leader_count =
+        (remaining_tablets + remaining_tservers - 1) / remaining_tservers;
+    int32_t should_transfer_count = leader_count - static_cast<int32_t>(target_leader_count);
     if (should_transfer_count > 0) {
       leader_transfer_source.insert({uuid, should_transfer_count});
       VLOG(1) << Substitute("$0 should transfer leader count: $1", uuid, should_transfer_count);
     }
+    if (remaining_tablets % remaining_tservers == 0) {
+      remaining_tablets -= target_leader_count;
+    } else {
+      remaining_tablets -= (should_transfer_count >= 0 ? target_leader_count :
+                            (target_leader_count - 1));
+    }
+    remaining_tservers--;
   }
 
   // Step 3.
@@ -363,7 +392,7 @@ Status AutoLeaderRebalancerTask::RunLeaderRebalanceForTable(
 }
 
 Status AutoLeaderRebalancerTask::RunLeaderRebalancer() {
-  MutexLock auto_lock(running_mutex_);
+  std::lock_guard guard(running_mutex_);
 
   // If catalog manager isn't initialized or isn't the leader, don't do leader
   // rebalancing. Putting the auto-rebalancer to sleep shouldn't affect the
@@ -383,7 +412,10 @@ Status AutoLeaderRebalancerTask::RunLeaderRebalancer() {
   // we get all tserver uuids
   TSDescriptorVector descriptors;
   ts_manager_->GetAllDescriptors(&descriptors);
-
+  if (PREDICT_FALSE(descriptors.empty())) {
+    VLOG(1) << "No tserver registered for now, skipping this leader rebalancing round.";
+    return Status::OK();
+  }
   vector<string> tserver_uuids;
   for (const auto& e : descriptors) {
     if (e->PresumedDead()) {
@@ -404,10 +436,15 @@ Status AutoLeaderRebalancerTask::RunLeaderRebalancer() {
   {
     CatalogManager::ScopedLeaderSharedLock leader_lock(catalog_manager_);
     RETURN_NOT_OK(leader_lock.first_failed_status());
-    catalog_manager_->GetAllTables(&table_infos);
+    if (FLAGS_leader_rebalancing_ignore_soft_deleted_tables) {
+      catalog_manager_->GetNormalizedTables(&table_infos);
+    } else {
+      catalog_manager_->GetAllTables(&table_infos);
+    }
   }
   for (const auto& table_info : table_infos) {
-    RunLeaderRebalanceForTable(table_info, tserver_uuids, exclude_dest_uuids);
+    RETURN_NOT_OK(RunLeaderRebalanceForTable(
+        table_info, tserver_uuids, exclude_dest_uuids));
   }
   // @TODO(duyuqi)
   // Enrich the log and add metrics for leader rebalancer.
