@@ -18,8 +18,9 @@
 #include "kudu/master/location_cache.h"
 
 #include <cstdio>
-#include <mutex>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -48,6 +49,7 @@ METRIC_DEFINE_counter(server, location_mapping_cache_queries,
                       "Number of queries to the location mapping cache",
                       kudu::MetricLevel::kDebug);
 
+using std::shared_lock;
 using std::string;
 using std::vector;
 using strings::Substitute;
@@ -111,7 +113,7 @@ Status LocationCache::GetLocation(const string& key, string* location) {
   TRACE(Substitute("key $0: assigned location '$1'", key, value));
   if (s.ok()) {
     CHECK(!value.empty());
-    std::lock_guard<rw_spinlock> l(location_map_lock_);
+    std::lock_guard l(location_map_lock_);
     // This simple implementation doesn't protect against multiple runs of the
     // location-mapping command for the same key.
     // TODO(KUDU-2771): queue concurrent requests for the same key
