@@ -36,6 +36,9 @@ class PeriodicWebUIChecker {
                        const std::string& tablet_id = "",
                        const std::vector<std::string>& master_pages = {
                            "/dump-entities",
+                           "/ipki-ca-cert",
+                           "/ipki-ca-cert-der",
+                           "/ipki-ca-cert-pem",
                            "/masters",
                            "/mem-trackers",
                            "/metrics",
