@@ -143,6 +143,10 @@ const rpc::RequestIdPB* RpcContext::request_id() const {
   return call_->header().has_request_id() ? &call_->header().request_id() : nullptr;
 }
 
+int32_t RpcContext::call_id() const {
+  return call_->call_id();
+}
+
 size_t RpcContext::GetTransferSize() const {
   return call_->GetTransferSize();
 }
@@ -171,6 +175,10 @@ const Sockaddr& RpcContext::remote_address() const {
   return call_->remote_address();
 }
 
+const Sockaddr& RpcContext::local_address() const {
+  return call_->local_address();
+}
+
 std::string RpcContext::requestor_string() const {
   return call_->remote_user().ToString() + " at " +
     call_->remote_address().ToString();
