@@ -16,8 +16,10 @@
 // under the License.
 #pragma once
 
-#include <memory>
 #include <stddef.h>
+#include <stdint.h>
+
+#include <memory>
 #include <string>
 
 #include <glog/logging.h>
@@ -190,6 +192,10 @@ class RpcContext {
   // Return the remote IP address and port which sent the current RPC call.
   const Sockaddr& remote_address() const;
 
+  // Return the local IP address and port of the connection which is used
+  // to receive the current RPC call.
+  const Sockaddr& local_address() const;
+
   // A string identifying the requestor -- both the user info and the IP address.
   // Suitable for use in log messages.
   std::string requestor_string() const;
@@ -226,6 +232,9 @@ class RpcContext {
   // Returns this call's request id, if it is set.
   const rpc::RequestIdPB* request_id() const;
 
+  // Returns this call's call_id.
+  int32_t call_id() const;
+
   // Returns the size of the transfer buffer that backs 'call_'. If the
   // transfer buffer no longer exists (e.g. GetTransferSize() is called after
   // DiscardTransfer()), returns 0.
