@@ -18,6 +18,7 @@
 #include <algorithm>
 #include <atomic>
 #include <csignal>
+#include <cstdint>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
@@ -27,6 +28,7 @@
 #include <ostream>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <vector>
 
 #include <gflags/gflags.h>
@@ -34,9 +36,7 @@
 #include <glog/stl_logging.h>
 #include <gtest/gtest.h>
 
-#include "kudu/gutil/atomicops.h"
 #include "kudu/gutil/ref_counted.h"
-#include "kudu/rpc/messenger.h"
 #include "kudu/rpc/proxy.h"
 #include "kudu/rpc/rpc-test-base.h"
 #include "kudu/rpc/rpc_controller.h"
@@ -61,6 +61,12 @@
 #include "kudu/util/thread_restrictions.h"
 #include "kudu/util/user.h"
 
+namespace kudu {
+namespace rpc {
+class Messenger;
+} // namespace rpc
+} // namespace kudu
+
 DEFINE_bool(is_panic_test_child, false, "Used by TestRpcPanic");
 DECLARE_bool(socket_inject_short_recvs);
 
@@ -69,7 +75,6 @@ using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
 using std::vector;
-using base::subtle::NoBarrier_Load;
 
 namespace kudu {
 namespace rpc {
@@ -123,10 +128,9 @@ TEST_F(RpcStubTest, TestShortRecvs) {
 // IO threads can deal with read/write calls that don't succeed
 // in sending the entire data in one go.
 TEST_F(RpcStubTest, TestBigCallData) {
-  const int kNumSentAtOnce = 20;
-  const size_t kMessageSize = 5 * 1024 * 1024;
-  string data;
-  data.resize(kMessageSize);
+  constexpr int kNumSentAtOnce = 20;
+  constexpr size_t kMessageSize = 5 * 1024 * 1024;
+  string data(kMessageSize, string::value_type());
 
   CalculatorServiceProxy p(client_messenger_, server_addr_, server_addr_.host());
 
@@ -297,12 +301,6 @@ TEST_F(RpcStubTest, TestCallWithInvalidParam) {
                       "missing fields: y");
 }
 
-// Wrapper around AtomicIncrement, since AtomicIncrement returns the 'old'
-// value, and our callback needs to be a void function.
-static void DoIncrement(Atomic32* count) {
-  base::subtle::Barrier_AtomicIncrement(count, 1);
-}
-
 // Test sending a PB parameter with a missing field on the client side.
 // This also ensures that the async callback is only called once
 // (regression test for a previously-encountered bug).
@@ -314,13 +312,13 @@ TEST_F(RpcStubTest, TestCallWithMissingPBFieldClientSide) {
   req.set_x(10);
   // Request is missing the 'y' field.
   AddResponsePB resp;
-  Atomic32 callback_count = 0;
-  p.AddAsync(req, &resp, &controller, [&callback_count]() { DoIncrement(&callback_count); });
-  while (NoBarrier_Load(&callback_count) == 0) {
+  std::atomic<uint32_t> callback_count(0);
+  p.AddAsync(req, &resp, &controller, [&callback_count]() { ++callback_count; });
+  while (callback_count == 0) {
     SleepFor(MonoDelta::FromMicroseconds(10));
   }
   SleepFor(MonoDelta::FromMicroseconds(100));
-  ASSERT_EQ(1, NoBarrier_Load(&callback_count));
+  ASSERT_EQ(1, callback_count);
   ASSERT_STR_CONTAINS(controller.status().ToString(),
                       "Invalid argument: invalid parameter for call "
                       "kudu.rpc_test.CalculatorService.Add: missing fields: y");
@@ -431,7 +429,7 @@ TEST_F(RpcStubTest, TestRpcPanic) {
     RpcController controller;
     PanicRequestPB req;
     PanicResponsePB resp;
-    p.Panic(req, &resp, &controller);
+    CHECK_OK(p.Panic(req, &resp, &controller));
   }
 }
 
