@@ -28,9 +28,11 @@
 #include <openssl/x509v3.h>
 
 #include <algorithm>
-#include <mutex>
-#include <ostream>
+#include <functional>
+#include <memory>
 #include <optional>
+#include <ostream>
+#include <shared_mutex>
 #include <string>
 #include <type_traits>
 #include <vector>
@@ -88,8 +90,8 @@
 using kudu::security::ca::CertRequestGenerator;
 using std::nullopt;
 using std::optional;
+using std::shared_lock;
 using std::string;
-using std::unique_lock;
 using std::vector;
 using strings::Substitute;
 
@@ -365,7 +367,7 @@ Status TlsContext::UseCertificateAndKey(const Cert& cert, const PrivateKey& key)
   // Verify that the cert and key match.
   RETURN_NOT_OK(cert.CheckKeyMatch(key));
 
-  std::unique_lock<RWMutex> lock(lock_);
+  std::lock_guard lock(lock_);
 
   // Verify that the appropriate CA certs have been loaded into the context
   // before we adopt a cert. Otherwise, client connections without the CA cert
@@ -402,7 +404,7 @@ Status TlsContext::AddTrustedCertificate(const Cert& cert) {
     CHECK_OK(cert.GetPublicKey(&k));
   }
 
-  unique_lock<RWMutex> lock(lock_);
+  std::lock_guard lock(lock_);
   auto* cert_store = SSL_CTX_get_cert_store(ctx_.get());
 
   // Iterate through the certificate chain and add each individual certificate to the store.
@@ -526,7 +528,7 @@ Status TlsContext::GenerateSelfSignedCertAndKey() {
   ERR_clear_error(); // in case it left anything on the queue.
 
   // Step 4: Adopt the new key and cert.
-  unique_lock<RWMutex> lock(lock_);
+  std::lock_guard lock(lock_);
   CHECK(!has_cert_);
   OPENSSL_RET_NOT_OK(SSL_CTX_use_PrivateKey(ctx_.get(), key.GetRawData()),
                      "failed to use private key");
@@ -548,7 +550,7 @@ optional<CertSignRequest> TlsContext::GetCsrIfNecessary() const {
 
 Status TlsContext::AdoptSignedCert(const Cert& cert) {
   SCOPED_OPENSSL_NO_PENDING_ERRORS;
-  unique_lock<RWMutex> lock(lock_);
+  std::lock_guard lock(lock_);
 
   if (!csr_) {
     // A signed cert has already been adopted.
@@ -645,5 +647,14 @@ Status TlsContext::InitiateHandshake(TlsHandshake* handshake) const {
   return handshake->Init(std::move(ssl));
 }
 
+const char* TlsContext::GetEngineVersionInfo() const {
+  CHECK(ctx_);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+  return CHECK_NOTNULL(SSLeay_version(SSLEAY_VERSION));
+#else
+  return CHECK_NOTNULL(OpenSSL_version(OPENSSL_VERSION));
+#endif
+}
+
 } // namespace security
 } // namespace kudu
