@@ -18,6 +18,7 @@
 #include "kudu/server/default_path_handlers.h"
 
 #include <sys/stat.h>
+// IWYU pragma: no_include <bits/struct_stat.h>
 
 #include <cstddef>
 #include <cstdint>
@@ -31,6 +32,7 @@
 
 #include <gflags/gflags.h>
 #include <glog/logging.h>
+
 #include "kudu/util/prometheus_writer.h"
 #include "kudu/util/version_info.h"
 #include "kudu/util/version_info.pb.h"
@@ -85,11 +87,12 @@ TAG_FLAG(web_log_bytes, advanced);
 TAG_FLAG(web_log_bytes, runtime);
 
 DEFINE_string(metrics_default_level, "debug",
-              "The default severity level to use when filtering the metrics. "
-              "Valid choices are 'debug', 'info', and 'warn'. "
-              "The levels are ordered and lower levels include the levels above them. "
-              "This value can be overridden by passing the level query parameter to the "
-              "'/metrics' endpoint.");
+              "The default severity level to use when filtering the metrics, "
+              "both in JSON and Prometheus formats. Valid choices are 'debug', "
+              "'info', and 'warn'. The levels are ordered and lower levels "
+              "include the levels above them. This value can be overridden "
+              "by passing the level query parameter to the '/metrics' endpoint "
+              "when fetching metrics in JSON format.");
 TAG_FLAG(metrics_default_level, advanced);
 TAG_FLAG(metrics_default_level, runtime);
 TAG_FLAG(metrics_default_level, evolving);
@@ -441,27 +444,27 @@ static void ConfigurationHandler(const Webserver::WebRequest& /* req */,
   FillTimeSourceConfigs(output);
 }
 void AddPreInitializedDefaultPathHandlers(Webserver* webserver) {
-  bool styled = true;
   bool on_nav_bar = true;
-  webserver->RegisterPathHandler("/logs", "Logs", LogsHandler, styled, on_nav_bar);
-  webserver->RegisterPrerenderedPathHandler("/varz", "Flags", FlagsHandler, styled, on_nav_bar);
+  webserver->RegisterPathHandler("/logs", "Logs", LogsHandler, StyleMode::STYLED, on_nav_bar);
+  webserver->RegisterPrerenderedPathHandler("/varz", "Flags", FlagsHandler, StyleMode::STYLED,
+                                            on_nav_bar);
   webserver->RegisterPathHandler("/config", "Configuration", ConfigurationHandler,
-                                 styled, on_nav_bar);
+                                 StyleMode::STYLED, on_nav_bar);
   webserver->RegisterPrerenderedPathHandler("/memz", "Memory (total)", MemUsageHandler,
-                                            styled, on_nav_bar);
+                                            StyleMode::STYLED, on_nav_bar);
   webserver->RegisterPrerenderedPathHandler("/mem-trackers", "Memory (detail)", MemTrackersHandler,
-                                            styled, on_nav_bar);
+                                            StyleMode::STYLED, on_nav_bar);
 }
 
 void AddPostInitializedDefaultPathHandlers(Webserver* webserver) {
   webserver->RegisterPrerenderedPathHandler("/stacks", "Stacks", StacksHandler,
-                                            /*is_styled=*/false,
+                                            StyleMode::UNSTYLED,
                                             /*is_on_nav_bar=*/true);
   webserver->RegisterPrerenderedPathHandler("/version", "VersionInfo", VersionInfoHandler,
-                                            /*is_styled=*/false,
+                                            StyleMode::UNSTYLED,
                                             /*is_on_nav_bar*/false);
   webserver->RegisterPrerenderedPathHandler("/healthz", "Health", HealthHandler,
-                                            /*is_styled=*/false,
+                                            StyleMode::UNSTYLED,
                                             /*is_on_nav_bar=*/true);
   AddPprofPathHandlers(webserver);
 }
@@ -522,8 +525,12 @@ static void WriteMetricsAsJson(const MetricRegistry* const metrics,
 static void WriteMetricsAsPrometheus(const MetricRegistry* const metrics,
                                      const Webserver::WebRequest& /*req*/,
                                      Webserver::PrerenderedWebResponse* resp) {
+  MetricPrometheusOptions opts;
+  opts.filters.entity_level = FLAGS_metrics_default_level;
+
   PrometheusWriter writer(&resp->output);
-  WARN_NOT_OK(metrics->WriteAsPrometheus(&writer), "Couldn't write Prometheus metrics over HTTP");
+  WARN_NOT_OK(metrics->WriteAsPrometheus(&writer, opts),
+              "couldn't write Prometheus metrics over HTTP");
 }
 
 void RegisterMetricsJsonHandler(Webserver* webserver, const MetricRegistry* const metrics) {
@@ -531,16 +538,13 @@ void RegisterMetricsJsonHandler(Webserver* webserver, const MetricRegistry* cons
                             Webserver::PrerenderedWebResponse* resp) {
     WriteMetricsAsJson(metrics, req, resp);
   };
-  bool not_styled = false;
   bool not_on_nav_bar = false;
   bool is_on_nav_bar = true;
-  webserver->RegisterPrerenderedPathHandler("/metrics", "JSON Metrics", callback,
-                                            not_styled, is_on_nav_bar);
+  webserver->RegisterJsonPathHandler("/metrics", "JSON Metrics", callback, is_on_nav_bar);
 
   // The old name -- this is preserved for compatibility with older releases of
   // monitoring software which expects the old name.
-  webserver->RegisterPrerenderedPathHandler("/jsonmetricz", "Metrics", callback,
-                                            not_styled, not_on_nav_bar);
+  webserver->RegisterJsonPathHandler("/jsonmetricz", "Metrics", callback, not_on_nav_bar);
 }
 
 void RegisterMetricsPrometheusHandler(Webserver* webserver, const MetricRegistry* const metrics) {
@@ -548,10 +552,9 @@ void RegisterMetricsPrometheusHandler(Webserver* webserver, const MetricRegistry
                             Webserver::PrerenderedWebResponse* resp) {
     WriteMetricsAsPrometheus(metrics, req, resp);
   };
-  constexpr bool not_styled = false;
   constexpr bool is_on_nav_bar = true;
   webserver->RegisterPrerenderedPathHandler("/metrics_prometheus", "Prometheus Metrics", callback,
-                                            not_styled, is_on_nav_bar);
+                                            StyleMode::UNSTYLED, is_on_nav_bar);
 }
 
 } // namespace kudu
