@@ -17,9 +17,15 @@
 
 #include "kudu/server/server_base.h"
 
+#if defined(__APPLE__)
+#include <sys/sysctl.h>
+#endif
+
 #include <algorithm>
+#include <cerrno> // IWYU pragma: keep
 #include <cstdint>
 #include <functional>
+#include <limits>
 #include <mutex>
 #include <optional>
 #include <set>
@@ -29,6 +35,9 @@
 
 #include <gflags/gflags.h>
 #include <glog/logging.h>
+#ifdef TCMALLOC_ENABLED
+#include <gperftools/malloc_extension.h>
+#endif
 
 #include "kudu/clock/clock.h"
 #include "kudu/clock/hybrid_clock.h"
@@ -72,6 +81,7 @@
 #include "kudu/util/cloud/instance_detector.h"
 #include "kudu/util/cloud/instance_metadata.h"
 #include "kudu/util/env.h"
+#include "kudu/util/errno.h"  // IWYU pragma: keep
 #include "kudu/util/faststring.h"
 #include "kudu/util/file_cache.h"
 #include "kudu/util/flag_tags.h"
@@ -99,10 +109,6 @@
 #include "kudu/util/user.h"
 #include "kudu/util/version_info.h"
 
-namespace kudu {
-class JwtVerifier;
-}  // namespace kudu
-
 DEFINE_int32(num_reactor_threads, 4, "Number of libev reactor threads to start.");
 TAG_FLAG(num_reactor_threads, advanced);
 
@@ -301,12 +307,26 @@ DEFINE_uint32(wall_clock_jump_threshold_sec, 15 * 60,
               "is enabled");
 TAG_FLAG(wall_clock_jump_threshold_sec, experimental);
 
+DEFINE_uint64(tcmalloc_max_total_thread_cache_bytes, 128 * 1024 * 1024,
+              "Upper limit on total number of bytes stored across all thread cache "
+              "in tcmalloc. Increasing this value helps in reducing lock contention "
+              "in tcmalloc for memory-intensive workloads. WARNING: This flag will "
+              "cover the TCMALLOC_MAX_TOTAL_THREAD_CACHE_BYTES environment variable. "
+              "Please change the max size of all thread caches via this flag. Note "
+              "that This bound is not strict, so it is possible for the cache to go "
+              "over this bound in certain circumstances. And the maximum value of this "
+              "flag is capped to 1 GB");
+TAG_FLAG(tcmalloc_max_total_thread_cache_bytes, advanced);
+TAG_FLAG(tcmalloc_max_total_thread_cache_bytes, experimental);
+
+DECLARE_bool(rpc_reuseport);
 DECLARE_bool(use_hybrid_clock);
 DECLARE_int32(dns_resolver_max_threads_num);
 DECLARE_uint32(dns_resolver_cache_capacity_mb);
 DECLARE_uint32(dns_resolver_cache_ttl_sec);
 DECLARE_int32(fs_data_dirs_available_space_cache_seconds);
 DECLARE_int32(fs_wal_dir_available_space_cache_seconds);
+DECLARE_int32(rpc_acceptor_listen_backlog);
 DECLARE_int64(fs_wal_dir_reserved_bytes);
 DECLARE_int64(fs_data_dirs_reserved_bytes);
 DECLARE_string(log_filename);
@@ -374,6 +394,16 @@ enum class TriState {
   DISABLED,
 };
 
+bool ValidateThreadCacheMaxTotalSize(const char* flagname, uint64_t value) {
+  if (value >= 0 && value <= 1 * 1024 * 1024 * 1024) {
+    return true;
+  }
+  LOG(ERROR) << Substitute("$0 must be a value in range [0, 1GB], value $1 is invalid.",
+                           flagname, value);
+  return false;
+}
+DEFINE_validator(tcmalloc_max_total_thread_cache_bytes, &ValidateThreadCacheMaxTotalSize);
+
 // This is a helper function to parse a flag that has three possible values:
 // "auto", "enabled", "disabled".  That directly maps into the TriState enum.
 Status ParseTriStateFlag(const string& name,
@@ -483,6 +513,19 @@ bool ValidateEitherJWKSFilePathOrUrlSet() {
 
 GROUP_FLAG_VALIDATOR(jwks_file_or_url_set_validator, &ValidateEitherJWKSFilePathOrUrlSet);
 
+bool ValidateSocketOptionCompatibility() {
+  if (FLAGS_rpc_listen_on_unix_domain_socket && FLAGS_rpc_reuseport) {
+    LOG(ERROR) << Substitute(
+        "Either --rpc_listen_on_unix_domain_socket or --rpc_reuseport should "
+        "be set, reuse of port is not applicable to Unix domain sockets.");
+    return false;
+  }
+
+  return true;
+}
+
+GROUP_FLAG_VALIDATOR(socket_options_compatibility, ValidateSocketOptionCompatibility);
+
 } // namespace
 
 static bool ValidateRpcAuthentication(const char* flag_name, const string& flag_value) {
@@ -577,7 +620,7 @@ shared_ptr<MemTracker> CreateMemTrackerForServer() {
   if (id != 0) {
     StrAppend(&id_str, " ", id);
   }
-  return shared_ptr<MemTracker>(MemTracker::CreateTracker(-1, id_str));
+  return MemTracker::CreateTracker(-1, id_str);
 }
 
 // Calculates the free space on the given WAL/data directory's disk. Returns -1 in case of disk
@@ -630,6 +673,68 @@ int64_t GetFileCacheCapacity(Env* env) {
   return FLAGS_server_max_open_files;
 }
 
+#if defined(__linux__)
+// See https://www.kernel.org/doc/Documentation/networking/ip-sysctl.txt
+// and https://man7.org/linux/man-pages/man2/listen.2.html for details.
+constexpr const char* const kListenBacklogMax = "/proc/sys/net/core/somaxconn";
+#endif
+
+#if defined(__APPLE__)
+// See https://man.freebsd.org/cgi/man.cgi?query=listen for details.
+// NOTE: it might be not exactly relevant to Darwin/macOS, but so far nothing
+//       indicates it's not applicable at least for Darwin 20.6.0/macOS 11.7.
+constexpr const char* const kListenBacklogMax = "kern.ipc.somaxconn";
+#endif
+
+int32_t GetEffectiveListenSocketBacklog(Env* env, int backlog) {
+#if defined(__APPLE__)
+  uint32_t buf_val;
+  size_t len = sizeof(buf_val);
+  if (sysctlbyname(kListenBacklogMax, &buf_val, &len, nullptr, 0) == -1) {
+    int err = errno;
+    LOG(WARNING) << Substitute(
+        "could not retrieve $0 to get listened socket queue size limit: $1",
+        kListenBacklogMax, ErrnoToString(err));
+    return backlog;
+  }
+  DCHECK_EQ(sizeof(buf_val), len);
+#elif defined(__linux__)
+  faststring buf;
+  if (auto s = ReadFileToString(env, kListenBacklogMax, &buf); !s.ok()) {
+    LOG(WARNING) << Substitute(
+        "could not read $0 to get listened socket queue size limit: $1",
+        kListenBacklogMax, s.ToString());
+    return backlog;
+  }
+  uint32_t buf_val;
+  if (!safe_strtou32(buf.ToString(), &buf_val)) {
+    LOG(WARNING) << Substitute(
+        "could not parse contents of $0 ('$1') to get listened socket queue size limit",
+        kListenBacklogMax, buf.ToString());
+    return backlog;
+  }
+#else
+  return backlog;
+#endif
+
+  // 'rpc_acceptor_listen_backlog == -1' means the highest possible value
+  // for the backlog size.
+  uint32_t effective_backlog =
+      (backlog >= 0) ? backlog : std::numeric_limits<uint32_t>::max();
+
+  // The system-wide limit caps the 'backlog' parameter of the listen()
+  // system call.
+  effective_backlog = std::min<uint32_t>(effective_backlog, buf_val);
+
+  // A bit of paranoia w.r.t. the system-level limit:
+  //   * for a debug build, use DCHECK to make sure the value is sane
+  //   * for a release build, cap it to INT32_MAX
+  DCHECK_LE(effective_backlog, std::numeric_limits<int32_t>::max());
+  effective_backlog = std::min<uint32_t>(effective_backlog,
+                                         std::numeric_limits<int32_t>::max());
+  return effective_backlog;
+}
+
 } // anonymous namespace
 
 ServerBase::ServerBase(string name, const ServerBaseOptions& options,
@@ -645,8 +750,8 @@ ServerBase::ServerBase(string name, const ServerBaseOptions& options,
                                 GetFileCacheCapacity(options.env), metric_entity_)),
       rpc_server_(new RpcServer(options.rpc_opts)),
       startup_path_handler_(new StartupPathHandler(metric_entity_)),
-      result_tracker_(new rpc::ResultTracker(shared_ptr<MemTracker>(
-          MemTracker::CreateTracker(-1, "result-tracker", mem_tracker_)))),
+      result_tracker_(new rpc::ResultTracker(
+          MemTracker::CreateTracker(-1, "result-tracker", mem_tracker_))),
       is_first_run_(false),
       stop_background_threads_latch_(1),
       dns_resolver_(new DnsResolver(
@@ -677,6 +782,7 @@ ServerBase::ServerBase(string name, const ServerBaseOptions& options,
 
 ServerBase::~ServerBase() {
   ShutdownImpl();
+  security::DestroyKerberosForServer();
 }
 
 Sockaddr ServerBase::first_rpc_address() const {
@@ -733,10 +839,31 @@ Status ServerBase::Init() {
   init->Start();
   glog_metrics_.reset(new ScopedGLogMetrics(metric_entity_));
   tcmalloc::RegisterMetrics(metric_entity_);
+
+#ifdef TCMALLOC_ENABLED
+  MallocExtension::instance()->SetNumericProperty(
+    "tcmalloc.max_total_thread_cache_bytes", FLAGS_tcmalloc_max_total_thread_cache_bytes);
   RegisterSpinLockContentionMetrics(metric_entity_);
+#else
+  LOG(INFO) << "Flag tcmalloc_max_total_thread_cache_bytes is not working since tcmalloc "
+               "is not enabled.";
+#endif
 
   InitSpinLockContentionProfiling();
 
+  // Get the FQDN of the node where the server is running. If fetching of the
+  // FQDN fails, it attempts to set the 'hostname_' field to the local hostname.
+  string hostname;
+  if (auto s = GetFQDN(&hostname); !s.ok()) {
+    const auto& msg = Substitute("could not determine host FQDN: $0", s.ToString());
+    if (hostname.empty()) {
+      LOG(ERROR) << msg;
+      return s;
+    }
+    LOG(WARNING) << msg;
+  }
+  DCHECK(!hostname.empty());
+
   RETURN_NOT_OK(security::InitKerberosForServer(FLAGS_principal, FLAGS_keytab_file));
   RETURN_NOT_OK(file_cache_->Init());
 
@@ -768,14 +895,27 @@ Status ServerBase::Init() {
   if (s.IsNotFound()) {
     LOG(INFO) << "This appears to be a new deployment of Kudu; creating new FS layout";
     is_first_run_ = true;
-    if (options_.server_key.empty()) {
+
+    if (options_.server_key_info.server_key.empty() &&
+        options_.tenant_key_info.tenant_key.empty()) {
       s = fs_manager_->CreateInitialFileSystemLayout();
+    } else if (!options_.tenant_key_info.tenant_key.empty()) {
+      // The priority of tenant key is higher than that of server key.
+      s = fs_manager_->CreateInitialFileSystemLayout(std::nullopt,
+                                                     options_.tenant_key_info.tenant_name,
+                                                     options_.tenant_key_info.tenant_id,
+                                                     options_.tenant_key_info.tenant_key,
+                                                     options_.tenant_key_info.tenant_key_iv,
+                                                     options_.tenant_key_info.tenant_key_version);
     } else {
       s = fs_manager_->CreateInitialFileSystemLayout(std::nullopt,
-                                                     options_.server_key,
-                                                     options_.server_key_iv,
-                                                     options_.server_key_version);
+                                                     std::nullopt,
+                                                     std::nullopt,
+                                                     options_.server_key_info.server_key,
+                                                     options_.server_key_info.server_key_iv,
+                                                     options_.server_key_info.server_key_version);
     }
+
     if (s.IsAlreadyPresent()) {
       return s.CloneAndPrepend("FS layout already exists; not overwriting existing layout");
     }
@@ -791,23 +931,17 @@ Status ServerBase::Init() {
   vector<string> rpc_tls_excluded_protocols = strings::Split(
       FLAGS_rpc_tls_excluded_protocols, ",", strings::SkipEmpty());
 
+  const auto listen_backlog = FLAGS_rpc_acceptor_listen_backlog;
+  const auto effective_listen_backlog = GetEffectiveListenSocketBacklog(
+      Env::Default(), listen_backlog);
+  if (effective_listen_backlog != listen_backlog) {
+    LOG(WARNING) << Substitute(
+        "--rpc_acceptor_listen_backlog setting $0 is capped at $1 by $2",
+        listen_backlog, effective_listen_backlog, kListenBacklogMax);
+  }
+
   // Create the Messenger.
   rpc::MessengerBuilder builder(name_);
-  shared_ptr<JwtVerifier> jwt_verifier = nullptr;
-  if (FLAGS_enable_jwt_token_auth) {
-    if (!FLAGS_jwks_url.empty()) {
-      jwt_verifier =
-          std::make_shared<KeyBasedJwtVerifier>(FLAGS_jwks_url,
-                                                FLAGS_jwks_verify_server_certificate,
-                                                FLAGS_trusted_certificate_file);
-    } else if (!FLAGS_jwks_file_path.empty()) {
-      jwt_verifier =
-          std::make_shared<KeyBasedJwtVerifier>(FLAGS_jwks_file_path);
-    } else {
-      LOG(WARNING) << Substitute("JWT authentication enabled, but neither 'jwks_url' or "
-          "'jwks_file_path' are set!");
-    }
-  }
   builder.set_num_reactors(FLAGS_num_reactor_threads)
          .set_min_negotiation_threads(FLAGS_min_negotiation_threads)
          .set_max_negotiation_threads(FLAGS_max_negotiation_threads)
@@ -823,10 +957,16 @@ Status ServerBase::Init() {
          .set_epki_cert_key_files(FLAGS_rpc_certificate_file, FLAGS_rpc_private_key_file)
          .set_epki_certificate_authority_file(FLAGS_rpc_ca_certificate_file)
          .set_epki_private_password_key_cmd(FLAGS_rpc_private_key_password_cmd)
-         .set_jwt_verifier(std::move(jwt_verifier))
          .set_keytab_file(FLAGS_keytab_file)
+         .set_hostname(hostname)
+         .set_acceptor_listen_backlog(listen_backlog)
          .enable_inbound_tls();
 
+  auto username = kudu::security::GetLoggedInUsernameFromKeytab();
+  if (username.has_value()) {
+    builder.set_sasl_proto_name(username.value());
+  }
+
   if (options_.rpc_opts.rpc_reuseport) {
     builder.set_reuseport();
   }
@@ -837,6 +977,21 @@ Status ServerBase::Init() {
     builder.set_sasl_proto_name(service_name);
   }
 
+  if (FLAGS_enable_jwt_token_auth) {
+    if (!FLAGS_jwks_url.empty()) {
+      builder.set_jwt_verifier(std::make_shared<KeyBasedJwtVerifier>(
+          FLAGS_jwks_url,
+          FLAGS_jwks_verify_server_certificate,
+          FLAGS_trusted_certificate_file));
+    } else if (!FLAGS_jwks_file_path.empty()) {
+      builder.set_jwt_verifier(std::make_shared<KeyBasedJwtVerifier>(
+          FLAGS_jwks_file_path));
+    } else {
+      LOG(WARNING) << Substitute("JWT authentication enabled, but neither "
+                                 "'jwks_url' nor 'jwks_file_path' is set");
+    }
+  }
+
   RETURN_NOT_OK(builder.Build(&messenger_));
   rpc_server_->set_too_busy_hook([this](rpc::ServicePool* pool) {
     this->ServiceQueueOverflowed(pool);
@@ -1081,6 +1236,7 @@ void ServerBase::ShutdownImpl() {
     web_server_->Stop();
   }
   rpc_server_->Shutdown();
+  dns_resolver_->Shutdown();
   if (messenger_) {
     messenger_->Shutdown();
   }
@@ -1098,8 +1254,6 @@ void ServerBase::ShutdownImpl() {
     tcmalloc_memory_gc_thread_->Join();
   }
 #endif
-
-  security::DestroyKerberosForServer();
 }
 
 #ifdef TCMALLOC_ENABLED
@@ -1136,12 +1290,10 @@ std::string ServerBase::FooterHtml() const {
 Status ServerBase::Start() {
   GenerateInstanceID();
 
+  DCHECK(!fs_manager_->uuid().empty());
   metric_entity_->SetAttribute("uuid", fs_manager_->uuid());
-  // Get the FQDN. If that fails server_hostname will have either the local hostname
-  // (if GetHostname() succeeds) or still be empty (if GetHostname() fails)
-  string server_hostname = "";
-  WARN_NOT_OK(GetFQDN(&server_hostname), "could not determine host FQDN");
-  metric_entity_->SetAttribute("hostname", server_hostname);
+  DCHECK(!messenger_->hostname().empty());
+  metric_entity_->SetAttribute("hostname", messenger_->hostname());
 
   RETURN_NOT_OK(RegisterService(
       unique_ptr<rpc::ServiceIf>(new GenericServiceImpl(this))));
@@ -1216,7 +1368,7 @@ Status ServerBase::Start() {
 int64_t ServerBase::RefreshWalDirAvailableSpaceIfExpired(const ServerBaseOptions& options,
                                                          const FsManager& fs_manager) {
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     if (MonoTime::Now() < wal_dir_space_last_check_ + MonoDelta::FromSeconds(
             FLAGS_fs_wal_dir_available_space_cache_seconds))
       return wal_dir_available_space_;
@@ -1225,7 +1377,7 @@ int64_t ServerBase::RefreshWalDirAvailableSpaceIfExpired(const ServerBaseOptions
   int64_t wal_dir_space = CalculateAvailableSpace(options, fs_manager.GetWalsRootDir(),
                                                   FLAGS_fs_wal_dir_reserved_bytes,
                                                   &space_info_waldir);
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   wal_dir_space_last_check_ = MonoTime::Now();
   wal_dir_available_space_ = wal_dir_space;
   return wal_dir_available_space_;
@@ -1234,7 +1386,7 @@ int64_t ServerBase::RefreshWalDirAvailableSpaceIfExpired(const ServerBaseOptions
 int64_t ServerBase::RefreshDataDirAvailableSpaceIfExpired(const ServerBaseOptions& options,
                                                           const FsManager& fs_manager) {
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     if (MonoTime::Now() < data_dirs_space_last_check_ + MonoDelta::FromSeconds(
             FLAGS_fs_data_dirs_available_space_cache_seconds))
       return data_dirs_available_space_;
@@ -1254,7 +1406,7 @@ int64_t ServerBase::RefreshDataDirAvailableSpaceIfExpired(const ServerBaseOption
       data_dirs_available_space += available_space;
     }
   }
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   data_dirs_space_last_check_ = MonoTime::Now();
   data_dirs_available_space_ = data_dirs_available_space;
   return data_dirs_available_space_;
@@ -1265,15 +1417,16 @@ void ServerBase::UnregisterAllServices() {
 }
 
 void ServerBase::ServiceQueueOverflowed(rpc::ServicePool* service) {
-  if (!diag_log_) return;
+  if (!diag_log_) {
+    return;
+  }
 
   // Logging all of the stacks is relatively heavy-weight, so if we are in a persistent
   // state of overload, it's probably not a good idea to start compounding the issue with
-  // a lot of stack-logging activity. So, we limit the frequency of stack-dumping.
-  static logging::LogThrottler throttler;
-  const int kStackDumpFrequencySecs = 5;
-  int suppressed = 0;
-  if (PREDICT_TRUE(!throttler.ShouldLog(kStackDumpFrequencySecs, "", &suppressed))) {
+  // a lot of stack-logging activity. So, we limit the frequency of stack-dumping
+  // to happen no more than once in five (5) seconds.
+  static logging::LogThrottler throttler(5, "Service Queue Overflow");
+  if (int64_t suppressed = 0; !throttler.ShouldLog(&suppressed)) {
     return;
   }
 
