@@ -18,6 +18,7 @@
 #include "kudu/tablet/compaction.h"
 
 #include <algorithm>
+#include <cinttypes>
 #include <cstdint>
 #include <cstdio>
 #include <cstdlib>
@@ -105,9 +106,7 @@ using strings::Substitute;
 namespace kudu {
 namespace tablet {
 
-class RowSetMetadata;
-
-constexpr const char* const kRowKeyFormat = "hello %010ld";
+constexpr const char* const kRowKeyFormat = "hello %010" PRId64;
 constexpr const size_t kLargeRollThreshold = 8UL * 1024 * 1024 * 1024;  // 8GB
 constexpr const size_t kSmallRollThreshold = 1024; // 1KB
 
@@ -301,15 +300,15 @@ class TestCompaction : public KuduRowSetTest {
 
   // Iterate over the given compaction input, stringifying and dumping each
   // yielded row to *out
-  static void IterateInput(CompactionInput* input, vector<string>* out) {
+  static void IterateInput(CompactionOrFlushInput* input, vector<string>* out) {
     ASSERT_OK(DebugDumpCompactionInput(input, nullptr, out));
   }
 
-  // Flush the given CompactionInput 'input' to disk with the given snapshot.
+  // Flush the given CompactionOrFlushInput 'input' to disk with the given snapshot.
   // If 'result_rowsets' is not NULL, reopens the resulting rowset(s) and appends
   // them to the vector.
   Status DoFlushAndReopen(
-      CompactionInput* input,
+      CompactionOrFlushInput* input,
       const Schema& projection,
       const MvccSnapshot& snap,
       size_t roll_threshold,
@@ -351,14 +350,14 @@ class TestCompaction : public KuduRowSetTest {
   static Status BuildCompactionInput(const MvccSnapshot& merge_snap,
                                      const vector<shared_ptr<DiskRowSet>>& rowsets,
                                      const Schema& projection,
-                                     unique_ptr<CompactionInput>* out) {
-    vector<shared_ptr<CompactionInput>> merge_inputs;
+                                     shared_ptr<CompactionOrFlushInput>* out) {
+    vector<shared_ptr<CompactionOrFlushInput>> merge_inputs;
     for (const auto& rs : rowsets) {
-      unique_ptr<CompactionInput> input;
-      RETURN_NOT_OK(CompactionInput::Create(*rs, &projection, merge_snap, nullptr, &input));
-      merge_inputs.emplace_back(input.release());
+      shared_ptr<CompactionOrFlushInput> input;
+      RETURN_NOT_OK(CompactionOrFlushInput::Create(*rs, &projection, merge_snap, nullptr, &input));
+      merge_inputs.emplace_back(std::move(input));
     }
-    out->reset(CompactionInput::Merge(merge_inputs, &projection));
+    *out = CompactionOrFlushInput::Merge(merge_inputs, &projection);
     return Status::OK();
   }
 
@@ -370,7 +369,7 @@ class TestCompaction : public KuduRowSetTest {
                           size_t roll_threshold,
                           vector<shared_ptr<DiskRowSet>>* result_rowsets) {
     MvccSnapshot merge_snap(mvcc_);
-    unique_ptr<CompactionInput> compact_input;
+    shared_ptr<CompactionOrFlushInput> compact_input;
     RETURN_NOT_OK(BuildCompactionInput(merge_snap, rowsets, projection, &compact_input));
     return DoFlushAndReopen(
         compact_input.get(), projection, merge_snap, roll_threshold, result_rowsets);
@@ -381,7 +380,7 @@ class TestCompaction : public KuduRowSetTest {
                               const Schema& projection,
                               shared_ptr<DiskRowSet>* result_rs) {
     vector<shared_ptr<DiskRowSet>> result_rowsets;
-    CompactAndReopen(input_rowsets, projection, kLargeRollThreshold, &result_rowsets);
+    ASSERT_OK(CompactAndReopen(input_rowsets, projection, kLargeRollThreshold, &result_rowsets));
     ASSERT_EQ(1, result_rowsets.size());
     *result_rs = result_rowsets[0];
   }
@@ -394,8 +393,7 @@ class TestCompaction : public KuduRowSetTest {
                            size_t roll_threshold,
                            vector<shared_ptr<DiskRowSet>>* result_rowsets) {
     MvccSnapshot snap(mvcc_);
-    vector<shared_ptr<RowSetMetadata>> rowset_metas;
-    unique_ptr<CompactionInput> input(CompactionInput::Create(mrs, &projection, snap));
+    auto input(CompactionOrFlushInput::Create(mrs, &projection, snap));
     return DoFlushAndReopen(input.get(), projection, snap, roll_threshold, result_rowsets);
   }
 
@@ -530,7 +528,7 @@ class TestCompaction : public KuduRowSetTest {
     LOG_TIMING(INFO, "compacting " +
                std::string((OVERLAP_INPUTS ? "with overlap" : "without overlap"))) {
       MvccSnapshot merge_snap(mvcc_);
-      unique_ptr<CompactionInput> compact_input;
+      shared_ptr<CompactionOrFlushInput> compact_input;
       ASSERT_OK(BuildCompactionInput(merge_snap, rowsets, schema_, &compact_input));
       // Use a low target row size to increase the number of resulting rowsets.
       RollingDiskRowSetWriter rdrsw(tablet()->metadata(), schema_,
@@ -578,7 +576,7 @@ TEST_F(TestCompaction, TestMemRowSetInput) {
   // and mutations.
   vector<string> out;
   MvccSnapshot snap(mvcc_);
-  unique_ptr<CompactionInput> input(CompactionInput::Create(*mrs, &schema_, snap));
+  auto input(CompactionOrFlushInput::Create(*mrs, &schema_, snap));
   IterateInput(input.get(), &out);
   ASSERT_EQ(10, out.size());
   EXPECT_EQ("RowIdxInBlock: 0; Base: (string key=\"hello 0000000000\", int64 val=0, "
@@ -605,13 +603,13 @@ TEST_F(TestCompaction, TestFlushMRSWithRolling) {
 
   vector<string> rows;
   rows.reserve(30000 / 2);
-  rowsets[0]->DebugDump(&rows);
+  ASSERT_OK(rowsets[0]->DebugDump(&rows));
   EXPECT_EQ("RowIdxInBlock: 0; Base: (string key=\"hello 0000000000\", int64 val=0, "
             "int32 nullable_val=0); Undo Mutations: [@1(DELETE)]; Redo Mutations: [];",
             rows[0]);
 
   rows.clear();
-  rowsets[1]->DebugDump(&rows);
+  ASSERT_OK(rowsets[1]->DebugDump(&rows));
   EXPECT_EQ("RowIdxInBlock: 0; Base: (string key=\"hello 0000017150\", int64 val=1715, "
             "int32 nullable_val=NULL); Undo Mutations: [@1716(DELETE)]; Redo Mutations: [];",
             rows[0]);
@@ -641,8 +639,8 @@ TEST_F(TestCompaction, TestRowSetInput) {
 
   // Check compaction input
   vector<string> out;
-  unique_ptr<CompactionInput> input;
-  ASSERT_OK(CompactionInput::Create(*rs, &schema_, MvccSnapshot(mvcc_), nullptr, &input));
+  shared_ptr<CompactionOrFlushInput> input;
+  ASSERT_OK(CompactionOrFlushInput::Create(*rs, &schema_, MvccSnapshot(mvcc_), nullptr, &input));
   IterateInput(input.get(), &out);
   ASSERT_EQ(10, out.size());
   EXPECT_EQ("RowIdxInBlock: 0; Base: (string key=\"hello 0000000000\", int64 val=0, "
@@ -704,12 +702,12 @@ TEST_F(TestCompaction, TestDuplicatedGhostRowsMerging) {
   shared_ptr<DiskRowSet> result;
   CompactAndReopenNoRoll(all_rss, schema_, &result);
 
-  unique_ptr<CompactionInput> input;
-  ASSERT_OK(CompactionInput::Create(*result,
-                                    &schema_,
-                                    MvccSnapshot::CreateSnapshotIncludingAllOps(),
-                                    nullptr,
-                                    &input));
+  shared_ptr<CompactionOrFlushInput> input;
+  ASSERT_OK(CompactionOrFlushInput::Create(*result,
+                                           &schema_,
+                                           MvccSnapshot::CreateSnapshotIncludingAllOps(),
+                                           nullptr,
+                                           &input));
   vector<string> out;
   IterateInput(input.get(), &out);
   ASSERT_EQ(out.size(), 10);
@@ -866,21 +864,21 @@ TEST_F(TestCompaction, TestDuplicatedRowsRandomCompaction) {
     row->undo_head = reinsert->next();
     row->row = block.row(i);
     BuildRow(i, i);
-    CopyRow(row_builder_.row(), &row->row, &mem.arena);
+    ASSERT_OK(CopyRow(row_builder_.row(), &row->row, &mem.arena));
     RowChangeListDecoder redo_decoder(reinsert->changelist());
     ASSERT_OK(redo_decoder.Init());
     faststring buf;
     RowChangeListEncoder dummy(&buf);
     dummy.SetToUpdate();
-    redo_decoder.MutateRowAndCaptureChanges(&row->row, &mem.arena, &dummy);
+    ASSERT_OK(redo_decoder.MutateRowAndCaptureChanges(&row->row, &mem.arena, &dummy));
     AddExpectedDelete(&row->redo_head, reinsert->timestamp());
   }
 
-  vector<shared_ptr<CompactionInput>> inputs;
+  vector<shared_ptr<CompactionOrFlushInput>> inputs;
   for (const auto& row_set : row_sets) {
-    unique_ptr<CompactionInput> ci;
+    shared_ptr<CompactionOrFlushInput> ci;
     ASSERT_OK(row_set->NewCompactionInput(&schema_, all_snap, nullptr, &ci));
-    inputs.emplace_back(ci.release());
+    inputs.emplace_back(std::move(ci));
   }
 
   // Compact the row sets by picking a few at random until we're left with just one.
@@ -901,7 +899,7 @@ TEST_F(TestCompaction, TestDuplicatedRowsRandomCompaction) {
   }
 
   vector<string> out;
-  unique_ptr<CompactionInput> ci;
+  shared_ptr<CompactionOrFlushInput> ci;
   ASSERT_OK(row_sets[0]->NewCompactionInput(&schema_, all_snap, nullptr, &ci));
   IterateInput(ci.get(), &out);
 
@@ -948,19 +946,19 @@ TEST_F(TestCompaction, TestMRSCompactionDoesntOutputUnobservableRows) {
 
   MvccSnapshot all_snap = MvccSnapshot::CreateSnapshotIncludingAllOps();
 
-  vector<shared_ptr<CompactionInput>> to_merge;
+  vector<shared_ptr<CompactionOrFlushInput>> to_merge;
   {
-    unique_ptr<CompactionInput> rs1_input;
-    ASSERT_OK(CompactionInput::Create(*rs1, &schema_, all_snap, nullptr, &rs1_input));
+    shared_ptr<CompactionOrFlushInput> rs1_input;
+    ASSERT_OK(CompactionOrFlushInput::Create(*rs1, &schema_, all_snap, nullptr, &rs1_input));
 
-    unique_ptr<CompactionInput> rs2_input;
-    ASSERT_OK(CompactionInput::Create(*rs2, &schema_, all_snap, nullptr, &rs2_input));
+    shared_ptr<CompactionOrFlushInput> rs2_input;
+    ASSERT_OK(CompactionOrFlushInput::Create(*rs2, &schema_, all_snap, nullptr, &rs2_input));
 
-    to_merge.emplace_back(rs1_input.release());
-    to_merge.emplace_back(rs2_input.release());
+    to_merge.emplace_back(std::move(rs1_input));
+    to_merge.emplace_back(std::move(rs2_input));
   }
 
-  unique_ptr<CompactionInput> merged(CompactionInput::Merge(to_merge, &schema_));
+  auto merged(CompactionOrFlushInput::Merge(to_merge, &schema_));
 
   // Make sure the unobservable version of the row that was inserted and deleted in the MRS
   // in the same op doesn't show up in the compaction input.
@@ -995,19 +993,17 @@ TEST_F(TestCompaction, TestOneToOne) {
 
   // Catch the updates that came in after the snapshot flush was made.
   MvccSnapshot snap2(mvcc_);
-  unique_ptr<CompactionInput> input(CompactionInput::Create(*mrs, &schema_, snap2));
+  auto input(CompactionOrFlushInput::Create(*mrs, &schema_, snap2));
 
   // Add some more updates which come into the new rowset while the "reupdate" is happening.
   UpdateRows(rs.get(), 1000, 0, 3);
 
-  string dummy_name = "";
-
   ASSERT_OK(ReupdateMissedDeltas(nullptr, input.get(), HistoryGcOpts::Disabled(), snap, snap2,
                                  { rs }));
 
   // If we look at the contents of the DiskRowSet now, we should see the "re-updated" data.
   vector<string> out;
-  ASSERT_OK(CompactionInput::Create(*rs, &schema_, MvccSnapshot(mvcc_), nullptr, &input));
+  ASSERT_OK(CompactionOrFlushInput::Create(*rs, &schema_, MvccSnapshot(mvcc_), nullptr, &input));
   IterateInput(input.get(), &out);
   ASSERT_EQ(1000, out.size());
   EXPECT_EQ("RowIdxInBlock: 0; Base: (string key=\"hello 0000000000\", int64 val=1, "
@@ -1017,8 +1013,8 @@ TEST_F(TestCompaction, TestOneToOne) {
 
   // And compact (1 input to 1 output)
   MvccSnapshot snap3(mvcc_);
-  unique_ptr<CompactionInput> compact_input;
-  ASSERT_OK(CompactionInput::Create(*rs, &schema_, snap3, nullptr, &compact_input));
+  shared_ptr<CompactionOrFlushInput> compact_input;
+  ASSERT_OK(CompactionOrFlushInput::Create(*rs, &schema_, snap3, nullptr, &compact_input));
   ASSERT_OK(DoFlushAndReopen(compact_input.get(), schema_, snap3, kLargeRollThreshold, nullptr));
 }
 
@@ -1048,12 +1044,10 @@ TEST_F(TestCompaction, TestKUDU102) {
   // Catch the updates that came in after the snapshot flush was made.
   // Note that we are merging two MRS, it's a hack
   MvccSnapshot snap2(mvcc_);
-  vector<shared_ptr<CompactionInput>> merge_inputs;
-  merge_inputs.emplace_back(CompactionInput::Create(*mrs, &schema_, snap2));
-  merge_inputs.emplace_back(CompactionInput::Create(*mrs_b, &schema_, snap2));
-  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
-
-  string dummy_name = "";
+  vector<shared_ptr<CompactionOrFlushInput>> merge_inputs;
+  merge_inputs.emplace_back(CompactionOrFlushInput::Create(*mrs, &schema_, snap2));
+  merge_inputs.emplace_back(CompactionOrFlushInput::Create(*mrs_b, &schema_, snap2));
+  auto input(CompactionOrFlushInput::Merge(merge_inputs, &schema_));
 
   // This would fail without KUDU-102
   ASSERT_OK(ReupdateMissedDeltas(nullptr, input.get(), HistoryGcOpts::Disabled(), snap, snap2,
@@ -1106,11 +1100,11 @@ TEST_F(TestCompaction, TestMergeMRS) {
   InsertRows(mrs_a.get(), 5, 1);
 
   MvccSnapshot snap(mvcc_);
-  vector<shared_ptr<CompactionInput>> merge_inputs {
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_a, &schema_, snap)),
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_b, &schema_, snap))
+  vector<shared_ptr<CompactionOrFlushInput>> merge_inputs {
+    CompactionOrFlushInput::Create(*mrs_a, &schema_, snap),
+    CompactionOrFlushInput::Create(*mrs_b, &schema_, snap)
   };
-  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
+  auto input(CompactionOrFlushInput::Merge(merge_inputs, &schema_));
   vector<shared_ptr<DiskRowSet>> result_rs;
   ASSERT_OK(DoFlushAndReopen(input.get(), schema_, snap, kSmallRollThreshold, &result_rs));
   ASSERT_EQ(20, CountRows(result_rs));
@@ -1125,11 +1119,11 @@ TEST_F(TestCompaction, TestMergeMRSWithInvisibleRows) {
                               mem_trackers_.tablet_tracker, &mrs_b));
   InsertRows(mrs_b.get(), 10, 0);
   MvccSnapshot snap(mvcc_);
-  vector<shared_ptr<CompactionInput>> merge_inputs {
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_a, &schema_, snap)),
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_b, &schema_, snap))
+  vector<shared_ptr<CompactionOrFlushInput>> merge_inputs {
+    CompactionOrFlushInput::Create(*mrs_a, &schema_, snap),
+    CompactionOrFlushInput::Create(*mrs_b, &schema_, snap)
   };
-  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
+  auto input(CompactionOrFlushInput::Merge(merge_inputs, &schema_));
   vector<shared_ptr<DiskRowSet>> result_rs;
   ASSERT_OK(DoFlushAndReopen(input.get(), schema_, snap, kSmallRollThreshold, &result_rs));
   ASSERT_EQ(1, result_rs.size());
@@ -1204,12 +1198,12 @@ TEST_F(TestCompaction, TestRandomizeDuplicatedRowsAcrossTransactions) {
     }
   }
   MvccSnapshot snap(mvcc_);
-  vector<shared_ptr<CompactionInput>> merge_inputs;
-  merge_inputs.emplace_back(CompactionInput::Create(*main_mrs, &schema_, snap));
-  for (auto& mrs : txn_mrss) {
-    merge_inputs.emplace_back(CompactionInput::Create(*mrs, &schema_, snap));
+  vector<shared_ptr<CompactionOrFlushInput>> merge_inputs;
+  merge_inputs.emplace_back(CompactionOrFlushInput::Create(*main_mrs, &schema_, snap));
+  for (const auto& mrs : txn_mrss) {
+    merge_inputs.emplace_back(CompactionOrFlushInput::Create(*mrs, &schema_, snap));
   }
-  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
+  auto input(CompactionOrFlushInput::Merge(merge_inputs, &schema_));
   vector<shared_ptr<DiskRowSet>> result_rs;
   ASSERT_OK(DoFlushAndReopen(input.get(), schema_, snap, kSmallRollThreshold, &result_rs));
   ASSERT_EQ(1, result_rs.size());
@@ -1250,12 +1244,12 @@ TEST_F(TestCompaction, TestRowHistoryJumpsBetweenRowsets) {
   // Despite the overlapping time ranges across these inputs, the compaction
   // should go off without a hitch.
   MvccSnapshot snap(mvcc_);
-  vector<shared_ptr<CompactionInput>> merge_inputs {
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_a, &schema_, snap)),
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_b, &schema_, snap)),
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_c, &schema_, snap)),
+  vector<shared_ptr<CompactionOrFlushInput>> merge_inputs {
+    CompactionOrFlushInput::Create(*mrs_a, &schema_, snap),
+    CompactionOrFlushInput::Create(*mrs_b, &schema_, snap),
+    CompactionOrFlushInput::Create(*mrs_c, &schema_, snap),
   };
-  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
+  auto input(CompactionOrFlushInput::Merge(merge_inputs, &schema_));
   vector<shared_ptr<DiskRowSet>> result_rs;
   ASSERT_OK(DoFlushAndReopen(input.get(), schema_, snap, kSmallRollThreshold, &result_rs));
   ASSERT_EQ(1, result_rs.size());
@@ -1267,11 +1261,11 @@ TEST_F(TestCompaction, TestMergeMRSWithAllInvisibleRows) {
   shared_ptr<MemRowSet> mrs_a = CreateInvisibleMRS();
   shared_ptr<MemRowSet> mrs_b = CreateInvisibleMRS();
   MvccSnapshot snap(mvcc_);
-  vector<shared_ptr<CompactionInput>> merge_inputs {
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_a, &schema_, snap)),
-    shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_b, &schema_, snap))
+  vector<shared_ptr<CompactionOrFlushInput>> merge_inputs {
+    CompactionOrFlushInput::Create(*mrs_a, &schema_, snap),
+    CompactionOrFlushInput::Create(*mrs_b, &schema_, snap)
   };
-  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
+  auto input(CompactionOrFlushInput::Merge(merge_inputs, &schema_));
   vector<shared_ptr<DiskRowSet>> result_rs;
   ASSERT_OK(DoFlushAndReopen(input.get(), schema_, snap, kSmallRollThreshold, &result_rs));
   ASSERT_TRUE(result_rs.empty());
@@ -1385,15 +1379,14 @@ TEST_F(TestCompaction, TestCompactionFreesDiskSpace) {
 // Regression test for KUDU-1237, a bug in which empty flushes or compactions
 // would result in orphaning near-empty cfile blocks on the disk.
 TEST_F(TestCompaction, TestEmptyFlushDoesntLeakBlocks) {
-  if (FLAGS_block_manager != "log") {
-    LOG(WARNING) << "Test requires the log block manager";
-    GTEST_SKIP();
+  if (!FsManager::IsLogType(FLAGS_block_manager)) {
+    GTEST_SKIP() << "Test requires the log block manager";
   }
 
   // Fetch the metric for the number of on-disk blocks, so we can later verify
   // that we actually remove data.
   fs::LogBlockManager* lbm = down_cast<fs::LogBlockManager*>(
-      harness_->fs_manager()->block_manager());
+      harness_->fs_manager()->block_manager().get());
 
   vector<BlockId> before_block_ids;
   ASSERT_OK(lbm->GetAllBlockIds(&before_block_ids));
