@@ -81,7 +81,7 @@ class DeltaIteratorMerger : public DeltaIterator {
                                          std::vector<DeltaKeyAndUpdate>* out,
                                          Arena* arena) override;
 
-  bool HasNext() override;
+  bool HasNext() const override;
 
   bool MayHaveDeltas() const override;
 
@@ -95,6 +95,8 @@ class DeltaIteratorMerger : public DeltaIterator {
     LOG(DFATAL) << "Not implemented";
   }
 
+  size_t memory_footprint() override;
+
  private:
   explicit DeltaIteratorMerger(std::vector<std::unique_ptr<DeltaIterator> > iters);
 
