@@ -19,10 +19,10 @@
 
 #include <algorithm>
 #include <limits>
-#include <mutex>
 #include <optional>
 #include <ostream>
 #include <set>
+#include <shared_mutex>
 #include <string>
 #include <type_traits>
 #include <utility>
@@ -71,25 +71,26 @@ DEFINE_uint64(all_delete_op_delta_file_cnt_for_compaction,
               "setting, this behavior is effectively disabled; see KUDU-3619 "
               "for details.");
 
-namespace kudu {
-
-class RowChangeList;
-
-namespace tablet {
-
-using cfile::ReaderOptions;
-using fs::CreateBlockOptions;
-using fs::IOContext;
-using fs::ReadableBlock;
-using fs::WritableBlock;
-using log::LogAnchorRegistry;
+using kudu::cfile::ReaderOptions;
+using kudu::fs::CreateBlockOptions;
+using kudu::fs::IOContext;
+using kudu::fs::ReadableBlock;
+using kudu::fs::WritableBlock;
+using kudu::log::LogAnchorRegistry;
 using std::set;
+using std::shared_lock;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
 using std::vector;
 using strings::Substitute;
 
+namespace kudu {
+
+class RowChangeList;
+
+namespace tablet {
+
 Status DeltaTracker::Open(const shared_ptr<RowSetMetadata>& rowset_metadata,
                           LogAnchorRegistry* log_anchor_registry,
                           const TabletMemTrackers& mem_trackers,
@@ -316,7 +317,7 @@ void DeltaTracker::AtomicUpdateStores(const SharedDeltaStoreVector& stores_to_re
                                       const SharedDeltaStoreVector& new_stores,
                                       const IOContext* io_context,
                                       DeltaType type) {
-  std::lock_guard<rw_spinlock> lock(component_lock_);
+  std::lock_guard lock(component_lock_);
   SharedDeltaStoreVector* stores_to_update =
       type == REDO ? &redo_delta_stores_ : &undo_delta_stores_;
   SharedDeltaStoreVector::iterator start_it;
@@ -434,7 +435,7 @@ Status DeltaTracker::CompactStores(const IOContext* io_context, int start_idx, i
   // Prevent concurrent compactions or a compaction concurrent with a flush
   //
   // TODO(perf): this could be more fine grained
-  std::lock_guard<Mutex> l(compact_flush_lock_);
+  std::lock_guard l(compact_flush_lock_);
   RETURN_NOT_OK(CheckWritableUnlocked());
 
   // At the time of writing, minor delta compaction only compacts REDO delta
@@ -488,7 +489,7 @@ Status DeltaTracker::CompactStores(const IOContext* io_context, int start_idx, i
 
 bool DeltaTracker::EstimateAllRedosAreAncient(Timestamp ancient_history_mark) {
   shared_ptr<DeltaStore> newest_redo;
-  std::lock_guard<rw_spinlock> lock(component_lock_);
+  std::lock_guard lock(component_lock_);
   const std::optional<Timestamp> dms_highest_timestamp =
       dms_ ? dms_->highest_timestamp() : std::nullopt;
   if (dms_highest_timestamp) {
@@ -577,7 +578,7 @@ Status DeltaTracker::DeleteAncientUndoDeltas(Timestamp ancient_history_mark,
   // updating both the rowset metadata and the delta stores in this method, so
   // we need to be the only thread doing a flush or a compaction on this RowSet
   // while we do our work.
-  std::lock_guard<Mutex> l(compact_flush_lock_);
+  std::lock_guard l(compact_flush_lock_);
   RETURN_NOT_OK(CheckWritableUnlocked());
 
   // Get the list of undo deltas.
@@ -655,7 +656,7 @@ Status DeltaTracker::DoCompactStores(const IOContext* io_context,
 
 void DeltaTracker::CollectStores(vector<shared_ptr<DeltaStore>>* deltas,
                                  WhichStores which) const {
-  std::lock_guard<rw_spinlock> lock(component_lock_);
+  std::lock_guard lock(component_lock_);
   if (which != REDOS_ONLY) {
     deltas->assign(undo_delta_stores_.begin(), undo_delta_stores_.end());
   }
@@ -681,7 +682,7 @@ Status DeltaTracker::NewDeltaFileIterator(
     vector<shared_ptr<DeltaStore>>* included_stores,
     unique_ptr<DeltaIterator>* out) const {
   {
-    std::lock_guard<rw_spinlock> lock(component_lock_);
+    std::lock_guard lock(component_lock_);
     // TODO perf: is this really needed? Will check
     // DeltaIteratorMerger::Create()
     if (type == UNDO) {
@@ -721,7 +722,7 @@ Status DeltaTracker::Update(Timestamp timestamp,
                             OperationResultPB* result) {
   while (true) {
     if (!dms_exists_) {
-      std::lock_guard<rw_spinlock> lock(component_lock_);
+      std::lock_guard lock(component_lock_);
       // Should check dms_exists_ here in case multiple threads are blocked.
       if (!dms_exists_) {
         RETURN_NOT_OK(CreateAndInitDMSUnlocked(nullptr));
@@ -821,7 +822,7 @@ Status DeltaTracker::FlushDMS(DeltaMemStore* dms,
     // Merge the deleted row count of the old DMS to the RowSetMetadata
     // and reset deleted_row_count_ should be atomic, so we lock the
     // component_lock_ in exclusive mode.
-    std::lock_guard<rw_spinlock> lock(component_lock_);
+    std::lock_guard lock(component_lock_);
     rowset_metadata_->CommitRedoDeltaDataBlock(
         dms->id(), deleted_row_count_, block_id);
     deleted_row_count_ = 0;
@@ -835,7 +836,7 @@ Status DeltaTracker::FlushDMS(DeltaMemStore* dms,
 }
 
 Status DeltaTracker::Flush(const IOContext* io_context, MetadataFlushType flush_type) {
-  std::lock_guard<Mutex> l(compact_flush_lock_);
+  std::lock_guard l(compact_flush_lock_);
   RETURN_NOT_OK(CheckWritableUnlocked());
 
   // First, swap out the old DeltaMemStore a new one,
@@ -846,7 +847,7 @@ Status DeltaTracker::Flush(const IOContext* io_context, MetadataFlushType flush_
   {
     // Lock the component_lock_ in exclusive mode.
     // This shuts out any concurrent readers or writers.
-    std::lock_guard<rw_spinlock> lock(component_lock_);
+    std::lock_guard lock(component_lock_);
 
     count = dms_exists_ ? dms_->Count() : 0;
 
@@ -878,7 +879,7 @@ Status DeltaTracker::Flush(const IOContext* io_context, MetadataFlushType flush_
     // A failure here leaves a DeltaMemStore permanently in the store list.
     // This isn't allowed, and rolling back the store is difficult, so we leave
     // the delta tracker in an safe, read-only state.
-    CHECK(s.IsDiskFailure()) << LogPrefix() << s.ToString();
+    DCHECK(s.IsDiskFailure()) << LogPrefix() << s.ToString();
     read_only_ = true;
     return s;
   }
@@ -886,7 +887,7 @@ Status DeltaTracker::Flush(const IOContext* io_context, MetadataFlushType flush_
   // Now, re-take the lock and swap in the DeltaFileReader in place of
   // of the DeltaMemStore
   {
-    std::lock_guard<rw_spinlock> lock(component_lock_);
+    std::lock_guard lock(component_lock_);
     size_t idx = redo_delta_stores_.size() - 1;
 
     CHECK_EQ(redo_delta_stores_[idx], old_dms)
@@ -937,7 +938,7 @@ size_t DeltaTracker::CountRedoDeltaStores() const {
 uint64_t DeltaTracker::UndoDeltaOnDiskSize() const {
   shared_lock<rw_spinlock> lock(component_lock_);
   uint64_t size = 0;
-  for (const shared_ptr<DeltaStore>& ds : undo_delta_stores_) {
+  for (const auto& ds : undo_delta_stores_) {
     size += ds->EstimateSize();
   }
   return size;
@@ -946,7 +947,7 @@ uint64_t DeltaTracker::UndoDeltaOnDiskSize() const {
 uint64_t DeltaTracker::RedoDeltaOnDiskSize() const {
   shared_lock<rw_spinlock> lock(component_lock_);
   uint64_t size = 0;
-  for (const shared_ptr<DeltaStore>& ds : redo_delta_stores_) {
+  for (const auto& ds : redo_delta_stores_) {
     size += ds->EstimateSize();
   }
   return size;
@@ -956,20 +957,19 @@ void DeltaTracker::GetColumnIdsToCompact(std::vector<ColumnId>* col_ids) const {
   shared_lock<rw_spinlock> lock(component_lock_);
 
   set<ColumnId> column_ids_to_compact;
-  uint32_t all_delete_op_delta_store_cnt = 0;
-  for (const shared_ptr<DeltaStore>& ds : redo_delta_stores_) {
+  uint64_t all_delete_op_delta_store_cnt = 0;
+  for (const auto& ds : redo_delta_stores_) {
     // We won't force open files just to read their stats.
     if (!ds->has_delta_stats()) {
       continue;
     }
-
-    ds->delta_stats().AddColumnIdsWithUpdates(&column_ids_to_compact);
+    const auto& delta_stats = ds->delta_stats();
+    delta_stats.AddColumnIdsWithUpdates(&column_ids_to_compact);
 
     // Count the number of REDO delta stores that contain only DELETE
     // operations where more than a single DELETE operation is present.
-    if ((ds->delta_stats().reinsert_count() == 0) &&
-        (ds->delta_stats().UpdateCount() == 0) &&
-        (ds->delta_stats().delete_count() > 1)) {
+    if ((delta_stats.reinsert_count() == 0) && (delta_stats.UpdateCount() == 0) &&
+        (delta_stats.delete_count() > 1)) {
       all_delete_op_delta_store_cnt++;
     }
   }
@@ -989,6 +989,33 @@ void DeltaTracker::GetColumnIdsToCompact(std::vector<ColumnId>* col_ids) const {
   col_ids->assign(column_ids_to_compact.begin(), column_ids_to_compact.end());
 }
 
+bool DeltaTracker::DeltaStoreNeedToBeCompacted() const {
+  uint64_t all_delete_op_delta_store_cnt = 0;
+  {
+    shared_lock<rw_spinlock> lock(component_lock_);
+
+    for (const auto& ds: redo_delta_stores_) {
+      if (!ds->has_delta_stats()) {
+        continue;
+      }
+      const auto& delta_stats = ds->delta_stats();
+      if (delta_stats.ColumnUpdated()) {
+        return true;
+      }
+      if ((delta_stats.reinsert_count() == 0) && (delta_stats.UpdateCount() == 0) &&
+          (delta_stats.delete_count() > 1)) {
+        all_delete_op_delta_store_cnt++;
+      }
+    }
+  }
+  // If there is no updated column, try to find a cold delta file
+  // with delete only ops to be compacted.
+  const auto max_delta_age = MonoDelta::FromSeconds(FLAGS_tablet_history_max_age_sec);
+  MonoTime last_update_time = last_update_time_.load();
+  return all_delete_op_delta_store_cnt > FLAGS_all_delete_op_delta_file_cnt_for_compaction
+      && MonoTime::Now() - last_update_time > max_delta_age;
+}
+
 Status DeltaTracker::InitAllDeltaStoresForTests(WhichStores stores) {
   shared_lock<rw_spinlock> lock(component_lock_);
   if (stores == UNDOS_AND_REDOS || stores == UNDOS_ONLY) {
