@@ -23,6 +23,7 @@
 #include <memory>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -44,7 +45,6 @@
 #include "kudu/fs/block_manager.h"
 #include "kudu/fs/fs-test-util.h"
 #include "kudu/fs/fs_manager.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/delta_key.h"
 #include "kudu/tablet/delta_stats.h"
@@ -53,6 +53,7 @@
 #include "kudu/tablet/mvcc.h"
 #include "kudu/tablet/rowset.h"
 #include "kudu/tablet/tablet-test-util.h"
+#include "kudu/util/bitmap.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/memory/arena.h"
 #include "kudu/util/random.h"
@@ -88,8 +89,7 @@ class TestDeltaFile : public KuduTest {
       schema_(CreateSchema()) {
   }
 
- public:
-  void SetUp() OVERRIDE {
+  void SetUp() override {
     fs_manager_.reset(new FsManager(env_, FsManagerOpts(GetTestPath("fs"))));
     ASSERT_OK(fs_manager_->CreateInitialFileSystemLayout());
     ASSERT_OK(fs_manager_->Open());
@@ -309,9 +309,7 @@ TEST_F(TestDeltaFile, TestCollectMutations) {
     ASSERT_OK(it->Init(nullptr));
     ASSERT_OK(it->SeekToOrdinal(0));
 
-    vector<Mutation *> mutations;
-    mutations.resize(100);
-
+    vector<Mutation*> mutations(100);
     Arena arena(1024);
 
     int start_row = 0;
