@@ -18,7 +18,6 @@
 #include "kudu/tablet/mvcc.h"
 
 #include <memory>
-#include <mutex>
 #include <ostream>
 #include <string>
 #include <thread>
@@ -59,12 +58,12 @@ class MvccTest : public KuduTest {
     MvccSnapshot s;
     CHECK_OK(mgr->WaitForSnapshotWithAllApplied(ts, &s, MonoTime::Max()));
     CHECK(s.is_clean()) << "verifying postcondition";
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     result_snapshot_.reset(new MvccSnapshot(s));
   }
 
   bool HasResultSnapshot() {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     return result_snapshot_ != nullptr;
   }
 
@@ -587,7 +586,7 @@ TEST_F(MvccTest, TestDontWaitAfterClose) {
   Status s;
   simple_spinlock status_lock;
   thread waiting_thread = thread([&] {
-    std::lock_guard<simple_spinlock> l(status_lock);
+    std::lock_guard l(status_lock);
     s = mgr.WaitForApplyingOpsToApply();
   });
 
@@ -646,7 +645,7 @@ TEST_F(MvccTest, TestOpAbort) {
 // coalesce to the latest timestamp.
 TEST_F(MvccTest, TestAutomaticCleanTimeMoveToSafeTimeOnApply) {
   MvccManager mgr;
-  clock_.Update(Timestamp(20));
+  ASSERT_OK(clock_.Update(Timestamp(20)));
 
   ScopedOp op1(&mgr, Timestamp(10));
   ScopedOp op2(&mgr, Timestamp(15));
@@ -684,7 +683,7 @@ TEST_F(MvccTest, TestIllegalStateTransitionsCrash) {
     "Trying to apply an op with a future timestamp|"
     "Trying to remove timestamp which isn't in the in-flight set: 1");
 
-  clock_.Update(Timestamp(20));
+  ASSERT_OK(clock_.Update(Timestamp(20)));
 
   EXPECT_DEATH({
       mgr.FinishApplyingOp(Timestamp(1));
