@@ -20,9 +20,10 @@
 #include <algorithm>
 #include <cstddef>
 #include <limits>
-#include <mutex>
 #include <ostream>
 #include <string>
+#include <type_traits>
+#include <utility>
 #include <vector>
 
 #include <gflags/gflags.h>
@@ -135,7 +136,7 @@ OpTracker::OpTracker() {
 
 OpTracker::~OpTracker() {
 #ifndef NDEBUG
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   DCHECK(pending_ops_.empty());
 #endif
 }
@@ -170,7 +171,7 @@ Status OpTracker::Add(shared_ptr<OpDriver> driver) {
 
   // Cache the op memory footprint so we needn't refer to the request
   // again, as it may disappear between now and then.
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   const auto* driver_ptr = driver.get();
   EmplaceOrDie(&pending_ops_,
                driver_ptr,
@@ -223,20 +224,25 @@ void OpTracker::Release(OpDriver* driver) {
   DecrementCounters(*driver);
 
   // Remove the op from the map updating memory consumption if needed.
-  std::lock_guard<simple_spinlock> l(lock_);
-  if (mem_tracker_) {
-    const State& st = FindOrDie(pending_ops_, driver);
-    mem_tracker_->Release(st.memory_footprint);
+  int64_t memory_footprint = 0;
+  {
+    std::lock_guard l(lock_);
+    const auto it = pending_ops_.find(driver);
+    if (PREDICT_FALSE(it == pending_ops_.end())) {
+      LOG(FATAL) << Substitute("$0: pending op not found",
+                               driver->ToStringUnlocked());
+    }
+    memory_footprint = it->second.memory_footprint;
+    pending_ops_.erase(it);
   }
-  if (PREDICT_FALSE(pending_ops_.erase(driver) != 1)) {
-    LOG(FATAL) << Substitute("Could not remove pending op from map: $0",
-                             driver->ToStringUnlocked());
+  if (mem_tracker_) {
+    mem_tracker_->Release(memory_footprint);
   }
 }
 
 vector<shared_ptr<OpDriver>> OpTracker::GetPendingOps() const {
   vector<shared_ptr<OpDriver>> ret;
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   ret.reserve(pending_ops_.size());
   for (const auto& [_, s]: pending_ops_) {
     ret.emplace_back(s.driver);
@@ -245,7 +251,7 @@ vector<shared_ptr<OpDriver>> OpTracker::GetPendingOps() const {
 }
 
 int OpTracker::GetNumPendingForTests() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return pending_ops_.size();
 }
 
