@@ -24,7 +24,9 @@
 #include <new>
 #include <optional>
 #include <ostream>
+#include <shared_mutex>
 #include <type_traits>
+#include <utility>
 #include <vector>
 
 #include <boost/container/small_vector.hpp>
@@ -61,8 +63,8 @@
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/memory/arena.h"
 #include "kudu/util/metrics.h"
+#include "kudu/util/monotime.h"
 #include "kudu/util/pb_util.h"
-#include "kudu/util/rw_semaphore.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/trace.h"
 
@@ -81,6 +83,7 @@ TAG_FLAG(tablet_inject_latency_on_prepare_write_op_ms, runtime);
 DECLARE_bool(enable_txn_partition_lock);
 
 using std::optional;
+using std::shared_lock;
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -155,7 +158,6 @@ Status WriteAuthorizationContext::CheckPrivileges() const {
 WriteOp::WriteOp(unique_ptr<WriteOpState> state, DriverType type)
   : Op(type, Op::WRITE_OP),
   state_(std::move(state)) {
-  start_time_ = MonoTime::Now();
 }
 
 void WriteOp::NewReplicateMsg(unique_ptr<ReplicateMsg>* replicate_msg) {
@@ -216,7 +218,7 @@ Status WriteOp::Prepare() {
           replicate_msg()->write_request().auto_incrementing_column().auto_incrementing_counter());
     }
   }
-  s = tablet->DecodeWriteOperations(&client_schema, state(), is_leader);
+  s = tablet->DecodeWriteOperations(&client_schema, state());
   if (!s.ok()) {
     // TODO(unknown): is MISMATCHED_SCHEMA always right here? probably not.
     state()->completion_callback()->set_error(s, TabletServerErrorPB::MISMATCHED_SCHEMA);
@@ -268,6 +270,7 @@ Status WriteOp::Start() {
   TRACE("Start()");
   DCHECK(!state_->has_timestamp());
   DCHECK(state_->consensus_round()->replicate_msg()->has_timestamp());
+  state_->set_start_time(MonoTime::Now());
   state_->set_timestamp(Timestamp(state_->consensus_round()->replicate_msg()->timestamp()));
   state_->tablet_replica()->tablet()->StartOp(state_.get());
   TRACE("Timestamp: $0", state_->tablet_replica()->clock()->Stringify(state_->timestamp()));
@@ -356,7 +359,7 @@ void WriteOp::Finish(OpResult result) {
         metrics->commit_wait_duration->Increment(op_m.commit_wait_duration_usec);
       }
       uint64_t op_duration_usec =
-          (MonoTime::Now() - start_time_).ToMicroseconds();
+          (MonoTime::Now() - state_->start_time()).ToMicroseconds();
       switch (state()->external_consistency_mode()) {
         case CLIENT_PROPAGATED:
           metrics->write_op_duration_client_propagated_consistency->Increment(op_duration_usec);
@@ -372,13 +375,20 @@ void WriteOp::Finish(OpResult result) {
 }
 
 string WriteOp::ToString() const {
-  MonoTime now(MonoTime::Now());
-  MonoDelta d = now - start_time_;
-  WallTime abs_time = WallTime_Now() - d.ToSeconds();
-  string abs_time_formatted;
-  StringAppendStrftime(&abs_time_formatted, "%Y-%m-%d %H:%M:%S", (time_t)abs_time, true);
+  const auto& start_time = state_->start_time();
+  string start_time_str = "n/a";
+  if (start_time.Initialized()) {
+    const auto d = MonoTime::Now() - start_time;
+    const WallTime abs_time = WallTime_Now() - d.ToSeconds();
+    string abs_time_formatted;
+    StringAppendStrftime(&abs_time_formatted,
+                         "%Y-%m-%d %H:%M:%S",
+                         static_cast<time_t>(abs_time),
+                         true);
+    start_time_str = std::move(abs_time_formatted);
+  }
   return Substitute("WriteOp [type=$0, start_time=$1, state=$2]",
-                    DriverType_Name(type()), abs_time_formatted, state_->ToString());
+                    DriverType_Name(type()), start_time_str, state_->ToString());
 }
 
 WriteOpState::WriteOpState(TabletReplica* tablet_replica,
@@ -445,7 +455,7 @@ void WriteOpState::ReleaseSchemaLock() {
 }
 
 void WriteOpState::SetRowOps(vector<DecodedRowOperation> decoded_ops) {
-  std::lock_guard<simple_spinlock> l(op_state_lock_);
+  std::lock_guard l(op_state_lock_);
   row_ops_.clear();
   row_ops_.reserve(decoded_ops.size());
 
@@ -665,7 +675,7 @@ void WriteOpState::Reset() {
 }
 
 void WriteOpState::ResetRpcFields() {
-  std::lock_guard<simple_spinlock> l(op_state_lock_);
+  std::lock_guard l(op_state_lock_);
   request_ = nullptr;
   response_ = nullptr;
   // these are allocated from the arena, so just run the dtors.
@@ -686,7 +696,7 @@ string WriteOpState::ToString() const {
   // Stringify the actual row operations (eg INSERT/UPDATE/etc)
   string row_ops_str = "[";
   {
-    std::lock_guard<simple_spinlock> l(op_state_lock_);
+    std::lock_guard l(op_state_lock_);
     const size_t kMaxToStringify = 3;
     for (int i = 0; i < std::min(row_ops_.size(), kMaxToStringify); i++) {
       if (i > 0) {
