@@ -478,7 +478,7 @@ class TabletTestBase : public KuduTabletTest {
     // select the col to update (the third if there is only one key
     // or the fourth if there are two col keys).
     int col_idx = schema_.num_key_columns() == 1 ? 2 : 3;
-    CHECK_OK(row.SetNull(col_idx));
+    RETURN_NOT_OK(row.SetNull(col_idx));
     return writer->Update(row);
   }
 
@@ -533,8 +533,7 @@ class TabletTestBase : public KuduTabletTest {
 
     // Keep a bitmap of which rows have been seen from the requested
     // range.
-    std::vector<bool> seen_rows;
-    seen_rows.resize(expected_row_count);
+    std::vector<bool> seen_rows(expected_row_count);
 
     uint64_t actual_row_count = 0;
     while (iter->HasNext()) {
@@ -573,7 +572,7 @@ class TabletTestBase : public KuduTabletTest {
     }
     ASSERT_EQ(expected_row_count, actual_row_count)
         << "Expected row count didn't match actual row count";
-    LOG(INFO) << "Successfully verified " << expected_row_count << "rows";
+    LOG(INFO) << Substitute("Successfully verified $0 rows", expected_row_count);
   }
 
   // Iterate through the full table, stringifying the resulting rows
