@@ -24,7 +24,9 @@
 #include <memory>
 #include <mutex>
 #include <ostream>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
@@ -63,17 +65,29 @@ class ConstContiguousRow;
 class EncodedKey;
 class KeyRange;
 class MemTracker;
+class RandomizedTabletHistoryGcITest;
 class RowBlock;
 class ScanSpec;
+class TabletHistoryGcITest;
 class Throttler;
 class Timestamp;
 struct IterWithBounds;
 struct IteratorStats;
 
+class AlterTableTest_TestMajorCompactDeltasAfterAddUpdateRemoveColumn_Test;
+class AlterTableTest_TestMajorCompactDeltasAfterUpdatingRemovedColumn_Test;
+class AlterTableTest_TestMajorCompactDeltasIntoMissingBaseData_Test;
+class RandomizedTabletHistoryGcITest_TestRandomHistoryGCWorkload_Test;
+class TabletHistoryGcITest_TestUndoDeltaBlockGc_Test;
+
 namespace consensus {
 class OpId;
 }  // namespace consensus
 
+namespace client {
+class ClientTest_TestInsertEmptyPK_Test;
+} // namespace client
+
 namespace clock {
 class Clock;
 } // namespace clock
@@ -82,6 +96,16 @@ namespace log {
 class LogAnchorRegistry;
 } // namespace log
 
+namespace tserver {
+class TabletServerTest_TestRecoveryWithMutationsWhileFlushingAndCompacting_Test;
+class TabletServerTest_SetEncodedKeysWhenStartingUp_Test;
+class TabletServerTest_TestEIODuringDelete_Test;
+class TabletServerTest_TestKUDU_176_RecoveryAfterMajorDeltaCompaction_Test;
+class TabletServerTest_TestKUDU_177_RecoveryOfDMSEditsAfterMajorDeltaCompaction_Test;
+class TabletServerTest_TestRecoveryWithMutationsWhileFlushing_Test;
+class TabletServerTest_TestRecoveryWithMutationsWhileFlushingAndCompacting_Test;
+} // namespace tserver
+
 namespace tablet {
 
 class AlterSchemaOpState;
@@ -90,7 +114,7 @@ class HistoryGcOpts;
 class MemRowSet;
 class ParticipantOpState;
 class RowSetTree;
-class RowSetsInCompaction;
+class RowSetsInCompactionOrFlush;
 class TxnMetadata;
 class WriteOpState;
 struct RowOp;
@@ -160,8 +184,7 @@ class Tablet {
   // Decode the Write (insert/mutate) operations from within a user's
   // request.
   Status DecodeWriteOperations(const Schema* client_schema,
-                               WriteOpState* op_state,
-                               bool is_leader = false);
+                               WriteOpState* op_state);
 
   // Acquire locks for each of the operations in the given write op.
   // This also sets the row op's RowSetKeyProbe.
@@ -185,10 +208,6 @@ class Tablet {
   void StartOp(WriteOpState* op_state);
   void StartOp(ParticipantOpState* op_state);
 
-  // Like the above but actually assigns the timestamp. Only used for tests that
-  // don't boot a tablet server.
-  void AssignTimestampAndStartOpForTests(WriteOpState* op_state);
-
   // Signal that the given op is about to Apply.
   void StartApplying(WriteOpState* op_state);
   void StartApplying(ParticipantOpState* op_state);
@@ -206,18 +225,18 @@ class Tablet {
   // Begins the transaction, recording its presence in the tablet metadata.
   // Upon calling this, 'op_id' will be anchored until the metadata is flushed,
   // using 'txn' as the anchor owner.
-  void BeginTransaction(Txn* txn, const consensus::OpId& op_id);
+  Status BeginTransaction(Txn* txn, const consensus::OpId& op_id);
 
   // Indicates that the transaction has started to commit, recording the
   // timestamp used by the MVCC op to demarcate the end of the transaction in
   // the tablet metadata. Upon calling this, 'op_id' will be anchored until
   // the metadata is flushed, using 'txn' as the anchor owner.
-  void BeginCommit(Txn* txn, Timestamp mvcc_op_ts, const consensus::OpId& op_id);
+  Status BeginCommit(Txn* txn, Timestamp mvcc_op_ts, const consensus::OpId& op_id);
 
   // Commits the transaction, recording its commit timestamp in the tablet metadata.
   // Upon calling this, 'op_id' will be anchored until the metadata is flushed,
   // using 'txn' as the anchor owner.
-  void CommitTransaction(Txn* txn, Timestamp commit_ts, const consensus::OpId& op_id);
+  Status CommitTransaction(Txn* txn, Timestamp commit_ts, const consensus::OpId& op_id);
 
   // Merges the uncommitted transaction rowsets associated with the given
   // 'txn_id' with the committed rowsets.
@@ -226,7 +245,7 @@ class Tablet {
   // Aborts the transaction, recording the abort in the tablet metadata.
   // Upon calling this, 'op_id' will be anchored until the metadata is flushed,
   // using 'txn' as the anchor owner.
-  void AbortTransaction(Txn* txn, const consensus::OpId& op_id);
+  Status AbortTransaction(Txn* txn, const consensus::OpId& op_id);
 
   // Creates new rowsets for the given transaction.
   void CreateTxnRowSets(int64_t txn_id, scoped_refptr<TxnMetadata> txn_meta);
@@ -332,16 +351,6 @@ class Tablet {
   // Flushes the DMS with the highest retention.
   Status FlushBestDMS(const ReplaySizeMap &replay_size_map) const;
 
-  // Flush only the biggest DMS. Only used for tests.
-  Status FlushBiggestDMSForTests();
-
-  // Flush all delta memstores. Only used for tests.
-  Status FlushAllDMSForTests();
-
-  // Run a major compaction on all delta stores. Initializes any un-initialized
-  // redo delta stores. Only used for tests.
-  Status MajorCompactAllDeltaStoresForTests();
-
   // Finds the RowSet which has the most separate delta files and
   // issues a delta compaction.
   Status CompactWorstDeltas(RowSet::DeltaCompactionType type);
@@ -395,10 +404,6 @@ class Tablet {
   // state is change.
   Status DeleteAncientDeletedRowsets();
 
-  // Counts the number of deltas in the tablet. Only used for tests.
-  int64_t CountUndoDeltasForTests() const;
-  int64_t CountRedoDeltasForTests() const;
-
   // Return the current number of rowsets in the tablet.
   size_t num_rowsets() const;
 
@@ -410,11 +415,6 @@ class Tablet {
   // Count the number of live rows in this tablet.
   Status CountLiveRows(uint64_t* count) const;
 
-  // Verbosely dump this entire tablet to the logs. This is only
-  // really useful when debugging unit tests failures where the tablet
-  // has a very small number of rows.
-  Status DebugDump(std::vector<std::string> *lines = NULL);
-
   const SchemaPtr schema() const {
     return metadata_->schema();
   }
@@ -436,24 +436,6 @@ class Tablet {
   TabletMetadata *metadata() { return metadata_.get(); }
   scoped_refptr<TabletMetadata> shared_metadata() const { return metadata_; }
 
-  void SetCompactionHooksForTests(const std::shared_ptr<CompactionFaultHooks> &hooks);
-  void SetFlushHooksForTests(const std::shared_ptr<FlushFaultHooks> &hooks);
-  void SetFlushCompactCommonHooksForTests(
-      const std::shared_ptr<FlushCompactCommonHooks> &hooks);
-
-  // Returns the current MemRowSet id, for tests.
-  // This method takes a read lock on component_lock_ and is thread-safe.
-  int32_t CurrentMrsIdForTests() const;
-
-  // Runs a major delta major compaction on columns with specified IDs.
-  // NOTE: RowSet must presently be a DiskRowSet. (Perhaps the API should be
-  // a shared_ptr API for now?)
-  //
-  // Only used in tests.
-  Status DoMajorDeltaCompaction(const std::vector<ColumnId>& col_ids,
-                                const std::shared_ptr<RowSet>& input_rs,
-                                const fs::IOContext* io_context = nullptr);
-
   // Calculates the ancient history mark and returns true iff tablet history GC
   // is enabled, which requires the use of a HybridClock.
   // Otherwise, returns false.
@@ -462,11 +444,6 @@ class Tablet {
   // Calculates history GC options based on properties of the Clock implementation.
   HistoryGcOpts GetHistoryGcOpts() const;
 
-  // Method used by tests to retrieve all rowsets of this table. This
-  // will be removed once code for selecting the appropriate RowSet is
-  // finished and delta files is finished is part of Tablet class.
-  void GetRowSetsForTests(std::vector<std::shared_ptr<RowSet> >* out);
-
   // Register the maintenance ops associated with this tablet
   void RegisterMaintenanceOps(MaintenanceManager* maint_mgr);
 
@@ -507,9 +484,9 @@ class Tablet {
 
   std::string LogPrefix() const;
 
-  // Return false if the tablets need to compact,
-  // otherwise return true.
-  bool disable_compaction() const;
+  // Return 'true' if the tablet's data may be compacted,
+  // otherwise return 'false'.
+  bool compaction_enabled() const;
 
   // Return the default bloom filter sizing parameters, configured by server flags.
   static BloomFilterSizing DefaultBloomSizing();
@@ -552,9 +529,36 @@ class Tablet {
   }
  private:
   friend class kudu::AlterTableTest;
+  friend class kudu::TabletHistoryGcITest;
+  friend class FuzzTest;
   friend class Iterator;
+  friend class LocalTabletWriter;
+  friend class TabletHistoryGcTest;
   friend class TabletReplicaTest;
   friend class TabletReplicaTestBase;
+  friend class TestHighMemCompaction;
+  template<typename T> friend class MultiThreadedTabletTest;
+  template<typename T> friend class TestTablet;
+  FRIEND_TEST(DiffScanTest, TestDiffScan);
+  FRIEND_TEST(OrderedDiffScanWithDeletesTest, TestDiffScanAfterDeltaFlushRacesWithBatchUpdate);
+  FRIEND_TEST(TabletDeletedRowsetGcTest, TestGCDeletedRowsetsAfterMajorCompaction);
+  FRIEND_TEST(TabletDeletedRowsetGcTest, TestGCDeletedRowsetsAfterMinorCompaction);
+  FRIEND_TEST(TabletDeletedRowsetGcTest, TestGCDeletedRowsetsWithRedoFiles);
+  FRIEND_TEST(TabletHistoryGcNoMaintMgrTest, TestUndoDeltaBlockGc);
+  FRIEND_TEST(TabletHistoryGcTest, TestGcWithConcurrentCompaction);
+  FRIEND_TEST(TabletHistoryGcTest, TestNoGenerateUndoOnMajorDeltaCompaction);
+  FRIEND_TEST(TabletHistoryGcTest, TestNoGenerateUndoOnRowSetCompaction);
+  FRIEND_TEST(TabletHistoryGcTest, TestMajorDeltaCompactionOnSubsetOfColumns);
+  FRIEND_TEST(TabletReplicaTest, TDMSAnchorPreventsLogGC);
+  FRIEND_TEST(TabletReplicaTest, TestActiveOpPreventsLogGC);
+  FRIEND_TEST(TabletReplicaTest, TestDMSAnchorPreventsLogGC);
+  FRIEND_TEST(TabletReplicaTest, TestRestartAfterGCDeletedRowsets);
+  FRIEND_TEST(TestMajorDeltaCompaction, TestCarryDeletesOver);
+  FRIEND_TEST(TestMajorDeltaCompaction, TestCompact);
+  FRIEND_TEST(TestMajorDeltaCompaction, TestJustDeletes);
+  FRIEND_TEST(TestMajorDeltaCompaction, TestKudu2656);
+  FRIEND_TEST(TestMajorDeltaCompaction, TestReinserts);
+  FRIEND_TEST(TestMajorDeltaCompaction, TestUndos);
   FRIEND_TEST(TestTablet, TestGetReplaySizeForIndex);
   FRIEND_TEST(TestTabletStringKey, TestSplitKeyRange);
   FRIEND_TEST(TestTabletStringKey, TestSplitKeyRangeWithZeroRowSets);
@@ -562,6 +566,23 @@ class Tablet {
   FRIEND_TEST(TestTabletStringKey, TestSplitKeyRangeWithNonOverlappingRowSets);
   FRIEND_TEST(TestTabletStringKey, TestSplitKeyRangeWithMinimumValueRowSet);
   FRIEND_TEST(TxnParticipantTest, TestFlushMultipleMRSs);
+  FRIEND_TEST(client::ClientTest, TestInsertEmptyPK);
+  FRIEND_TEST(kudu::AlterTableTest, TestMajorCompactDeltasAfterAddUpdateRemoveColumn);
+  FRIEND_TEST(kudu::AlterTableTest, TestMajorCompactDeltasAfterUpdatingRemovedColumn);
+  FRIEND_TEST(kudu::AlterTableTest, TestMajorCompactDeltasIntoMissingBaseData);
+  FRIEND_TEST(kudu::RandomizedTabletHistoryGcITest, TestRandomHistoryGCWorkload);
+  FRIEND_TEST(kudu::TabletHistoryGcITest, TestUndoDeltaBlockGc);
+  FRIEND_TEST(tserver::TabletServerTest, SetEncodedKeysWhenStartingUp);
+  FRIEND_TEST(tserver::TabletServerTest, TestEIODuringDelete);
+  FRIEND_TEST(tserver::TabletServerTest, TestKUDU_176_RecoveryAfterMajorDeltaCompaction);
+  FRIEND_TEST(tserver::TabletServerTest, TestKUDU_177_RecoveryOfDMSEditsAfterMajorDeltaCompaction);
+  FRIEND_TEST(tserver::TabletServerTest, TestRecoveryWithMutationsWhileFlushing);
+  FRIEND_TEST(tserver::TabletServerTest, TestRecoveryWithMutationsWhileFlushingAndCompacting);
+  template<typename T> FRIEND_TEST(TestTablet, TestCompaction);
+  template<typename T> FRIEND_TEST(TestTablet, TestCompactionWithConcurrentMutation);
+  template<typename T> FRIEND_TEST(TestTablet, TestFlushWithConcurrentMutation);
+  template<typename T> FRIEND_TEST(TestTablet, TestReinsertDuringFlush);
+  template<typename T> FRIEND_TEST(TestTablet, TestRowIteratorOrdered);
 
   // Lifecycle states that a Tablet can be in. Legal state transitions for a
   // Tablet object:
@@ -626,6 +647,52 @@ class Tablet {
   // Validate the given update/delete operation.
   static Status ValidateMutateUnlocked(const RowOp& op);
 
+  // Returns the current MemRowSet id, for tests.
+  // This method takes a read lock on component_lock_ and is thread-safe.
+  int32_t CurrentMrsIdForTests() const;
+
+  // Runs a major delta major compaction on columns with specified IDs.
+  // NOTE: RowSet must presently be a DiskRowSet. (Perhaps the API should be
+  // a shared_ptr API for now?)
+  //
+  // Only used in tests.
+  Status DoMajorDeltaCompaction(const std::vector<ColumnId>& col_ids,
+                                const std::shared_ptr<RowSet>& input_rs,
+                                const fs::IOContext* io_context = nullptr);
+
+  // Like the above but actually assigns the timestamp. Only used for tests that
+  // don't boot a tablet server.
+  void AssignTimestampAndStartOpForTests(WriteOpState* op_state);
+
+  // Flush only the biggest DMS. Only used for tests.
+  Status FlushBiggestDMSForTests();
+
+  // Flush all delta memstores. Only used for tests.
+  Status FlushAllDMSForTests();
+
+  // Run a major compaction on all delta stores. Initializes any un-initialized
+  // redo delta stores. Only used for tests.
+  Status MajorCompactAllDeltaStoresForTests();
+
+  // Counts the number of deltas in the tablet. Only used for tests.
+  int64_t CountUndoDeltasForTests() const;
+  int64_t CountRedoDeltasForTests() const;
+
+  void SetCompactionHooksForTests(const std::shared_ptr<CompactionFaultHooks> &hooks);
+  void SetFlushHooksForTests(const std::shared_ptr<FlushFaultHooks> &hooks);
+  void SetFlushCompactCommonHooksForTests(
+      const std::shared_ptr<FlushCompactCommonHooks> &hooks);
+
+  // Method used by tests to retrieve all rowsets of this table. This
+  // will be removed once code for selecting the appropriate RowSet is
+  // finished and delta files is finished is part of Tablet class.
+  void GetRowSetsForTests(std::vector<std::shared_ptr<RowSet> >* out);
+
+  // Verbosely dump this entire tablet to the logs. This is only
+  // really useful when debugging unit tests failures where the tablet
+  // has a very small number of rows.
+  Status DebugDump(std::vector<std::string> *lines = NULL);
+
   // Perform an INSERT, INSERT_IGNORE, UPSERT, or UPSERT_IGNORE operation, assuming that the op is
   // already in a prepared state. This state ensures that:
   // - the row lock is acquired
@@ -674,11 +741,11 @@ class Tablet {
                                     const ScanSpec* spec,
                                     std::vector<IterWithBounds>* iters) const;
 
-  Status PickRowSetsToCompact(RowSetsInCompaction *picked,
+  Status PickRowSetsToCompact(RowSetsInCompactionOrFlush *picked,
                               CompactFlags flags) const;
 
   // Performs a merge compaction or a flush.
-  Status DoMergeCompactionOrFlush(const RowSetsInCompaction &input,
+  Status DoMergeCompactionOrFlush(const RowSetsInCompactionOrFlush &input,
                                   int64_t mrs_being_flushed,
                                   const std::vector<TxnInfoBeingFlushed>& txns_being_flushed);
 
@@ -698,6 +765,9 @@ class Tablet {
                        int64_t mrs_being_flushed,
                        const std::vector<TxnInfoBeingFlushed>& txns_being_flushed);
 
+  // Computes on-disk size of all the deltas in provided rowsets.
+  size_t GetAllDeltasSizeOnDisk(const RowSetsInCompactionOrFlush& input);
+
   static void ModifyRowSetTree(const RowSetTree& old_tree,
                                const RowSetVector& rowsets_to_remove,
                                const RowSetVector& rowsets_to_add,
@@ -714,12 +784,12 @@ class Tablet {
                                  const RowSetVector &to_add);
 
   void GetComponents(scoped_refptr<TabletComponents>* comps) const {
-    shared_lock<rw_spinlock> l(component_lock_);
+    std::shared_lock<rw_spinlock> l(component_lock_);
     *comps = CHECK_NOTNULL(components_.get());
   }
 
   void GetComponentsOrNull(scoped_refptr<TabletComponents>* comps) const {
-    shared_lock<rw_spinlock> l(component_lock_);
+    std::shared_lock<rw_spinlock> l(component_lock_);
     *comps = components_;
   }
 
@@ -728,7 +798,7 @@ class Tablet {
   // before the replacement. If any MemRowSet is not empty it will be added to
   // the 'compaction' input and the MemRowSets' compaction locks will be taken
   // to prevent the inclusion in any concurrent compactions.
-  Status ReplaceMemRowSetsUnlocked(RowSetsInCompaction* compaction,
+  Status ReplaceMemRowSetsUnlocked(RowSetsInCompactionOrFlush* new_mrss,
                                    std::vector<std::shared_ptr<MemRowSet>>* old_mrss);
 
   // Convert the specified read client schema (without IDs) to a server schema (with IDs)
@@ -882,43 +952,43 @@ class Tablet {
 // parts of the compaction code.
 class Tablet::CompactionFaultHooks {
  public:
+  virtual ~CompactionFaultHooks() = default;
   virtual Status PostSelectIterators() { return Status::OK(); }
-  virtual ~CompactionFaultHooks() {}
 };
 
 class Tablet::FlushCompactCommonHooks {
  public:
+  virtual ~FlushCompactCommonHooks() = default;
   virtual Status PostTakeMvccSnapshot() { return Status::OK(); }
   virtual Status PostWriteSnapshot() { return Status::OK(); }
   virtual Status PostSwapInDuplicatingRowSet() { return Status::OK(); }
   virtual Status PostReupdateMissedDeltas() { return Status::OK(); }
   virtual Status PostSwapNewRowSet() { return Status::OK(); }
-  virtual ~FlushCompactCommonHooks() {}
 };
 
 // Hooks used in test code to inject faults or other code into interesting
 // parts of the Flush() code.
 class Tablet::FlushFaultHooks {
  public:
+  virtual ~FlushFaultHooks() = default;
   virtual Status PostSwapNewMemRowSet() { return Status::OK(); }
-  virtual ~FlushFaultHooks() {}
 };
 
 class Tablet::Iterator : public RowwiseIterator {
  public:
-  virtual ~Iterator();
+  ~Iterator() override;
 
-  virtual Status Init(ScanSpec *spec) OVERRIDE;
+  Status Init(ScanSpec *spec) override;
 
-  virtual bool HasNext() const OVERRIDE;
+  bool HasNext() const override;
 
-  virtual Status NextBlock(RowBlock *dst) OVERRIDE;
+  Status NextBlock(RowBlock *dst) override;
 
-  std::string ToString() const OVERRIDE;
+  std::string ToString() const override;
 
-  const Schema &schema() const OVERRIDE;
+  const Schema &schema() const override;
 
-  virtual void GetIteratorStats(std::vector<IteratorStats>* stats) const OVERRIDE;
+  void GetIteratorStats(std::vector<IteratorStats>* stats) const override;
 
  private:
   friend class Tablet;
