@@ -23,6 +23,7 @@
 #include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -47,7 +48,6 @@
 #include "kudu/consensus/log-test-base.h"
 #include "kudu/consensus/log.h"
 #include "kudu/consensus/log_anchor_registry.h"
-#include "kudu/consensus/log_reader.h"
 #include "kudu/consensus/log_util.h"
 #include "kudu/consensus/metadata.pb.h"
 #include "kudu/consensus/opid.pb.h"
@@ -56,8 +56,8 @@
 #include "kudu/consensus/ref_counted_replicate.h"
 #include "kudu/fs/data_dirs.h"
 #include "kudu/fs/fs_manager.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
+#include "kudu/gutil/strings/substitute.h"
 #include "kudu/rpc/result_tracker.h"
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/rowset.h"
@@ -99,7 +99,7 @@ namespace tablet {
 class BootstrapTest : public LogTestBase {
  protected:
 
-  void SetUp() OVERRIDE {
+  void SetUp() override {
     LogTestBase::SetUp();
     cmeta_manager_.reset(new ConsensusMetadataManager(fs_manager_.get()));
   }
@@ -250,7 +250,8 @@ TEST_F(BootstrapTest, TestBootstrapHighOpIdIndex) {
   ASSERT_OK(BuildLog());
   current_index_ = first_log_index;
   for (int i = 0; i < kNumEntries; i++) {
-    AppendReplicateBatchAndCommitEntryPairsToLog(1);
+    SCOPED_TRACE(Substitute("entry $0", i));
+    ASSERT_OK(AppendReplicateBatchAndCommitEntryPairsToLog(1));
   }
 
   // Kick off tablet bootstrap and ensure everything worked.
@@ -324,7 +325,7 @@ TEST_F(BootstrapTest, TestOrphanCommit) {
     ASSERT_OK(AppendCommit(opid));
     log::SegmentSequence segments;
     log_->reader()->GetSegmentsSnapshot(&segments);
-    fs_manager_->env()->DeleteFile(segments[0]->path());
+    ASSERT_OK(fs_manager_->GetEnv()->DeleteFile(segments[0]->path()));
 
     // Untrack the tablet in the data dir manager so upon the next call to
     // BootstrapTestTablet, the tablet metadata's data dir group can be loaded.
@@ -392,7 +393,7 @@ TEST_F(BootstrapTest, TestNonOrphansAfterOrphanCommit) {
 
   log::SegmentSequence segments;
   log_->reader()->GetSegmentsSnapshot(&segments);
-  fs_manager_->env()->DeleteFile(segments[0]->path());
+  ASSERT_OK(fs_manager_->GetEnv()->DeleteFile(segments[0]->path()));
 
   current_index_ += 2;
 
