@@ -115,7 +115,6 @@ using kudu::consensus::ConsensusBootstrapInfo;
 using kudu::consensus::MinimumOpId;
 using kudu::consensus::NO_OP;
 using kudu::consensus::OpId;
-using kudu::consensus::OpIdEquals;
 using kudu::consensus::OpIdToString;
 using kudu::consensus::OperationType;
 using kudu::consensus::OperationType_Name;
@@ -727,11 +726,11 @@ Status TabletBootstrap::PrepareRecoveryDir(bool* needs_recovery) {
     if (fs_manager->Exists(log_dir)) {
       LOG_WITH_PREFIX(INFO) << "Deleting old log files from previous recovery attempt in "
                             << log_dir;
-      RETURN_NOT_OK_PREPEND(fs_manager->env()->DeleteRecursively(log_dir),
+      RETURN_NOT_OK_PREPEND(fs_manager->GetEnv()->DeleteRecursively(log_dir),
                             "Could not recursively delete old log dir " + log_dir);
     }
 
-    RETURN_NOT_OK_PREPEND(env_util::CreateDirIfMissing(fs_manager->env(), log_dir),
+    RETURN_NOT_OK_PREPEND(env_util::CreateDirIfMissing(fs_manager->GetEnv(), log_dir),
                           "Failed to create log directory " + log_dir);
 
     *needs_recovery = true;
@@ -741,7 +740,7 @@ Status TabletBootstrap::PrepareRecoveryDir(bool* needs_recovery) {
   // If we made it here, there was no pre-existing recovery dir.
   // Now we look for log files in log_dir, and if we find any then we rename
   // the whole log_dir to a recovery dir and return needs_recovery = true.
-  RETURN_NOT_OK_PREPEND(env_util::CreateDirIfMissing(fs_manager->env(), log_dir),
+  RETURN_NOT_OK_PREPEND(env_util::CreateDirIfMissing(fs_manager->GetEnv(), log_dir),
                         "Failed to create log dir");
 
   vector<string> children;
@@ -764,10 +763,10 @@ Status TabletBootstrap::PrepareRecoveryDir(bool* needs_recovery) {
     // and then re-create the log directory.
     VLOG_WITH_PREFIX(1) << "Moving log directory " << log_dir << " to recovery directory "
                         << recovery_path << " in preparation for log replay";
-    RETURN_NOT_OK_PREPEND(fs_manager->env()->RenameFile(log_dir, recovery_path),
+    RETURN_NOT_OK_PREPEND(fs_manager->GetEnv()->RenameFile(log_dir, recovery_path),
                           Substitute("Could not move log directory $0 to recovery dir $1",
                                      log_dir, recovery_path));
-    RETURN_NOT_OK_PREPEND(fs_manager->env()->CreateDir(log_dir),
+    RETURN_NOT_OK_PREPEND(fs_manager->GetEnv()->CreateDir(log_dir),
                           "Failed to recreate log directory " + log_dir);
   }
   return Status::OK();
@@ -783,7 +782,7 @@ Status TabletBootstrap::OpenLogReaderInRecoveryDir() {
   // isn't fsynced() during writing, its contents are useless to us.
   scoped_refptr<LogIndex> log_index(nullptr);
   const string recovery_dir = fs_manager->GetTabletWalRecoveryDir(tablet_id);
-  RETURN_NOT_OK_PREPEND(LogReader::Open(fs_manager->env(), recovery_dir, log_index, tablet_id,
+  RETURN_NOT_OK_PREPEND(LogReader::Open(fs_manager->GetEnv(), recovery_dir, log_index, tablet_id,
                                         tablet_->GetMetricEntity().get(),
                                         file_cache_,
                                         &log_reader_),
@@ -967,7 +966,7 @@ Status TabletBootstrap::HandleReplicateMessage(ReplayState* state,
     const auto& existing_entry = existing_entry_iter->second;
 
     auto iter = state->pending_replicates.lower_bound(index);
-    DCHECK(OpIdEquals(iter->second->replicate().id(), existing_entry->replicate().id()));
+    DCHECK(iter->second->replicate().id() == existing_entry->replicate().id());
 
     const auto& last_entry = state->pending_replicates.rbegin()->second;
     VLOG_WITH_PREFIX(1) << "Overwriting operations starting at: "
@@ -1090,7 +1089,7 @@ Status TabletBootstrap::ApplyCommitMessage(const IOContext* io_context,
   if (PREDICT_TRUE(pending_replicate_entry)) {
     // We found a replicate with the same index, make sure it also has the same term.
     const auto& replicate = pending_replicate_entry->replicate();
-    if (PREDICT_FALSE(!OpIdEquals(committed_op_id, replicate.id()))) {
+    if (PREDICT_FALSE(committed_op_id != replicate.id())) {
       string error_msg = Substitute("Committed operation's OpId: $0 didn't match the"
           "commit message's committed OpId: $1. Pending operation: $2, Commit message: $3",
           SecureShortDebugString(replicate.id()),
