@@ -17,7 +17,6 @@
 
 #include "kudu/tablet/tablet_mm_ops.h"
 
-#include <mutex>
 #include <optional>
 #include <ostream>
 #include <type_traits>
@@ -119,8 +118,8 @@ int32_t TabletOpBase::priority() const {
   return priority;
 }
 
-bool TabletOpBase::DisableCompaction() const {
-  return tablet_->disable_compaction();
+bool TabletOpBase::compaction_enabled() const {
+  return tablet_->compaction_enabled();
 }
 
 ////////////////////////////////////////////////////////////
@@ -136,7 +135,7 @@ CompactRowSetsOp::CompactRowSetsOp(Tablet* tablet)
 }
 
 void CompactRowSetsOp::UpdateStats(MaintenanceOpStats* stats) {
-  if (PREDICT_FALSE(!FLAGS_enable_rowset_compaction || DisableCompaction())) {
+  if (PREDICT_FALSE(!FLAGS_enable_rowset_compaction || !compaction_enabled())) {
     KLOG_EVERY_N_SECS(WARNING, FLAGS_update_stats_log_throttling_interval_sec)
         << Substitute("Rowset compaction is disabled (check --enable_rowset_compaction "
            "and disable_compaction in extra_config for tablet:$0)", tablet_->tablet_id());
@@ -144,7 +143,7 @@ void CompactRowSetsOp::UpdateStats(MaintenanceOpStats* stats) {
     return;
   }
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
 
   double workload_score = FLAGS_enable_workload_score_for_perf_improvement_ops ?
                           tablet_->CollectAndUpdateWorkloadStats(MaintenanceOp::COMPACT_OP) : 0;
@@ -177,7 +176,7 @@ void CompactRowSetsOp::UpdateStats(MaintenanceOpStats* stats) {
 }
 
 bool CompactRowSetsOp::Prepare() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   // Invalidate the cached stats so that another section of the tablet can
   // be compacted concurrently.
   //
@@ -217,7 +216,7 @@ MinorDeltaCompactionOp::MinorDeltaCompactionOp(Tablet* tablet)
 }
 
 void MinorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
-  if (PREDICT_FALSE(!FLAGS_enable_minor_delta_compaction || DisableCompaction())) {
+  if (PREDICT_FALSE(!FLAGS_enable_minor_delta_compaction || !compaction_enabled())) {
     KLOG_EVERY_N_SECS(WARNING, FLAGS_update_stats_log_throttling_interval_sec)
         << Substitute("Minor delta compaction is disabled (check --enable_minor_delta_compaction "
            "and disable_compaction in extra_config for tablet:$0)", tablet_->tablet_id());
@@ -225,7 +224,7 @@ void MinorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
     return;
   }
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
 
   double workload_score = FLAGS_enable_workload_score_for_perf_improvement_ops ?
                           tablet_->CollectAndUpdateWorkloadStats(MaintenanceOp::COMPACT_OP) : 0;
@@ -263,7 +262,7 @@ void MinorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
 }
 
 bool MinorDeltaCompactionOp::Prepare() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   // Invalidate the cached stats so that another rowset in the tablet can
   // be delta compacted concurrently.
   //
@@ -301,7 +300,7 @@ MajorDeltaCompactionOp::MajorDeltaCompactionOp(Tablet* tablet)
 }
 
 void MajorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
-  if (PREDICT_FALSE(!FLAGS_enable_major_delta_compaction || DisableCompaction())) {
+  if (PREDICT_FALSE(!FLAGS_enable_major_delta_compaction || !compaction_enabled())) {
     KLOG_EVERY_N_SECS(WARNING, FLAGS_update_stats_log_throttling_interval_sec)
         << Substitute("Major delta compaction is disabled (check --enable_major_delta_compaction "
            "and disable_compaction in extra_config for tablet:$0)", tablet_->tablet_id());
@@ -309,7 +308,7 @@ void MajorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
     return;
   }
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
 
   double workload_score = FLAGS_enable_workload_score_for_perf_improvement_ops ?
                           tablet_->CollectAndUpdateWorkloadStats(MaintenanceOp::COMPACT_OP) : 0;
@@ -351,7 +350,7 @@ void MajorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
 }
 
 bool MajorDeltaCompactionOp::Prepare() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   // Invalidate the cached stats so that another rowset in the tablet can
   // be delta compacted concurrently.
   //
