@@ -232,13 +232,13 @@ Status TabletReplica::Start(
   DCHECK(log) << "A TabletReplica must be provided with a Log";
 
   {
-    std::lock_guard<simple_spinlock> state_change_guard(state_change_lock_);
+    std::lock_guard state_change_guard(state_change_lock_);
 
     scoped_refptr<MetricEntity> metric_entity;
     unique_ptr<PeerProxyFactory> peer_proxy_factory;
     unique_ptr<TimeManager> time_manager;
     {
-      std::lock_guard<simple_spinlock> l(lock_);
+      std::lock_guard l(lock_);
       CHECK_EQ(BOOTSTRAPPING, state_);
 
       tablet_ = DCHECK_NOTNULL(std::move(tablet));
@@ -297,7 +297,7 @@ Status TabletReplica::Start(
         metric_entity,
         mark_dirty_clbk_));
 
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
 
     // If an error has been set (e.g. due to a disk failure from a separate
     // thread), error out.
@@ -333,7 +333,7 @@ void TabletReplica::Stop() {
   }
 
   WaitUntilTxnCoordinatorTasksFinished();
-  std::lock_guard<simple_spinlock> l(state_change_lock_);
+  std::lock_guard l(state_change_lock_);
 
   // Even though Tablet::Shutdown() also unregisters its ops, we have to do it here
   // to ensure that any currently running operation finishes before we proceed with
@@ -368,7 +368,7 @@ void TabletReplica::Stop() {
 
   // Only mark the peer as STOPPED when all other components have shut down.
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     tablet_.reset();
     set_state(STOPPED);
   }
@@ -379,7 +379,7 @@ void TabletReplica::Stop() {
 void TabletReplica::Shutdown() {
   Stop();
   if (consensus_) consensus_->Shutdown();
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   if (state_ == SHUTDOWN || state_ == FAILED) return;
   if (!error_.ok()) {
     set_state(FAILED);
@@ -391,7 +391,7 @@ void TabletReplica::Shutdown() {
 void TabletReplica::WaitUntilStopped() {
   while (true) {
     {
-      std::lock_guard<simple_spinlock> lock(lock_);
+      std::lock_guard lock(lock_);
       if (state_ == STOPPED || state_ == SHUTDOWN || state_ == FAILED) {
         return;
       }
@@ -407,7 +407,7 @@ void TabletReplica::WaitUntilTxnCoordinatorTasksFinished() {
 
   while (true) {
     {
-      std::lock_guard<simple_spinlock> lock(lock_);
+      std::lock_guard lock(lock_);
       if (txn_coordinator_task_counter_ == 0) {
         return;
       }
@@ -487,7 +487,7 @@ void TabletReplica::set_state(TabletStatePB new_state) {
 
 Status TabletReplica::CheckRunning() const {
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     if (state_ != RUNNING) {
       return Status::IllegalState(Substitute("The tablet is not in a running state: $0",
                                              TabletStatePB_Name(state_)));
@@ -497,7 +497,7 @@ Status TabletReplica::CheckRunning() const {
 }
 
 bool TabletReplica::IsShuttingDown() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (state_ == STOPPING || state_ == STOPPED) {
     return true;
   }
@@ -513,7 +513,7 @@ Status TabletReplica::WaitUntilConsensusRunning(const MonoDelta& timeout) {
     bool has_consensus = false;
     TabletStatePB cached_state;
     {
-      std::lock_guard<simple_spinlock> lock(lock_);
+      std::lock_guard lock(lock_);
       cached_state = state_;
       if (consensus_) {
         has_consensus = true; // consensus_ is a set-once object.
@@ -551,7 +551,7 @@ Status TabletReplica::SubmitTxnWrite(
   const int64_t txn_id = op_state->request()->txn_id();
   shared_ptr<TxnOpDispatcher> dispatcher;
   {
-    std::lock_guard<simple_spinlock> guard(txn_op_dispatchers_lock_);
+    std::lock_guard guard(txn_op_dispatchers_lock_);
     dispatcher = LookupOrEmplace(
         &txn_op_dispatchers_,
         txn_id,
@@ -566,7 +566,7 @@ Status TabletReplica::UnregisterTxnOpDispatcher(int64_t txn_id,
   Status unregister_status;
   VLOG(3) << Substitute(
       "received request to unregister TxnOpDispatcher (txn ID $0)", txn_id);
-  std::lock_guard<simple_spinlock> guard(txn_op_dispatchers_lock_);
+  std::lock_guard guard(txn_op_dispatchers_lock_);
   auto it = txn_op_dispatchers_.find(txn_id);
   // It might happen that TxnOpDispatcher isn't there, and that's completely
   // fine. One possible scenario that might lead to such condition is:
@@ -630,7 +630,7 @@ Status TabletReplica::SubmitAlterSchema(unique_ptr<AlterSchemaOpState> state,
 void TabletReplica::GetTabletStatusPB(TabletStatusPB* status_pb_out) const {
   DCHECK(status_pb_out != nullptr);
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     status_pb_out->set_state(state_);
     status_pb_out->set_last_status(last_status_);
   }
@@ -653,44 +653,41 @@ void TabletReplica::GetTabletStatusPB(TabletStatusPB* status_pb_out) const {
   }
 }
 
-Status TabletReplica::RunLogGC() {
-  if (!CheckRunning().ok()) {
-    return Status::OK();
+void TabletReplica::RunLogGC() {
+  if (PREDICT_FALSE(!CheckRunning().ok())) {
+    return;
   }
   int32_t num_gced;
-  log::RetentionIndexes retention = GetRetentionIndexes();
-  Status s = log_->GC(retention, &num_gced);
-  if (!s.ok()) {
-    s = s.CloneAndPrepend("Unexpected error while running Log GC from TabletReplica");
-    LOG(ERROR) << s.ToString();
+  const auto s = log_->GC(GetRetentionIndexes(), &num_gced);
+  if (PREDICT_FALSE(!s.ok())) {
+    LOG(ERROR) << LogPrefix() << "unexpected error while running log GC: " << s.ToString();
   }
-  return Status::OK();
 }
 
 void TabletReplica::SetBootstrapping() {
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   set_state(BOOTSTRAPPING);
 }
 
 void TabletReplica::SetStatusMessage(string status) {
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   last_status_ = std::move(status);
 }
 
 string TabletReplica::last_status() const {
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   return last_status_;
 }
 
 void TabletReplica::SetError(const Status& error) {
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   CHECK(!error.ok());
   error_ = error;
   last_status_ = error.ToString();
 }
 
 string TabletReplica::HumanReadableState() const {
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   TabletDataState data_state = meta_->tablet_data_state();
   // If failed, any number of things could have gone wrong.
   if (state_ == FAILED) {
@@ -792,7 +789,7 @@ Status TabletReplica::GetGCableDataSize(int64_t* retention_size) const {
 
 Status TabletReplica::StartFollowerOp(const scoped_refptr<ConsensusRound>& round) {
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     if (state_ != RUNNING && state_ != BOOTSTRAPPING) {
       return Status::IllegalState(TabletStatePB_Name(state_));
     }
@@ -848,7 +845,7 @@ Status TabletReplica::StartFollowerOp(const scoped_refptr<ConsensusRound>& round
   state->set_consensus_round(round);
 
   shared_ptr<OpDriver> driver;
-  RETURN_NOT_OK(NewReplicaOpDriver(std::move(op), &driver));
+  RETURN_NOT_OK(NewFollowerOpDriver(std::move(op), &driver));
 
   // A weak pointer is required to avoid a refcount cycle.
   // TODO(aserbin): is the comment above still relevant?
@@ -899,7 +896,7 @@ void TabletReplica::FinishConsensusOnlyRound(ConsensusRound* round) {
     // pool token and this callback is invoked while the RaftConsensus lock is
     // held.
     CHECK_OK(prepare_pool_token_->Submit([this, ts] {
-      std::lock_guard<simple_spinlock> l(lock_);
+      std::lock_guard l(lock_);
       if (state_ == RUNNING || state_ == BOOTSTRAPPING) {
         tablet_->mvcc_manager()->AdjustNewOpLowerBound(Timestamp(ts));
       }
@@ -924,8 +921,8 @@ Status TabletReplica::NewLeaderOpDriver(unique_ptr<Op> op,
   return Status::OK();
 }
 
-Status TabletReplica::NewReplicaOpDriver(unique_ptr<Op> op,
-                                         shared_ptr<OpDriver>* driver) {
+Status TabletReplica::NewFollowerOpDriver(unique_ptr<Op> op,
+                                          shared_ptr<OpDriver>* driver) {
   auto op_driver = OpDriver::make_shared(
       &op_tracker_,
       consensus_.get(),
@@ -942,7 +939,7 @@ Status TabletReplica::NewReplicaOpDriver(unique_ptr<Op> op,
 void TabletReplica::RegisterMaintenanceOps(MaintenanceManager* maint_mgr) {
   // Taking state_change_lock_ ensures that we don't shut down concurrently with
   // this last start-up task.
-  std::lock_guard<simple_spinlock> state_change_lock(state_change_lock_);
+  std::lock_guard state_change_lock(state_change_lock_);
 
   if (state() != RUNNING) {
     LOG(WARNING) << "Not registering maintenance operations for " << tablet_
@@ -963,7 +960,7 @@ void TabletReplica::RegisterMaintenanceOps(MaintenanceManager* maint_mgr) {
 
   std::shared_ptr<Tablet> tablet;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     DCHECK(maintenance_ops_.empty());
     maintenance_ops_ = std::move(maintenance_ops);
     tablet = tablet_;
@@ -972,7 +969,7 @@ void TabletReplica::RegisterMaintenanceOps(MaintenanceManager* maint_mgr) {
 }
 
 void TabletReplica::CancelMaintenanceOpsForTests() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (auto& op : maintenance_ops_) {
     op->CancelAndDisable();
   }
@@ -982,7 +979,7 @@ void TabletReplica::UnregisterMaintenanceOps() {
   DCHECK(state_change_lock_.is_locked());
   decltype(maintenance_ops_) ops;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     ops = std::move(maintenance_ops_);
     maintenance_ops_.clear();
   }
@@ -1002,7 +999,7 @@ size_t TabletReplica::OnDiskSize() const {
   shared_ptr<Tablet> tablet;
   scoped_refptr<Log> log;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     tablet = tablet_;
     log = log_;
   }
@@ -1018,7 +1015,7 @@ size_t TabletReplica::OnDiskSize() const {
 Status TabletReplica::CountLiveRows(uint64_t* live_row_count) const {
   shared_ptr<Tablet> tablet;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     tablet = tablet_;
   }
 
@@ -1054,7 +1051,7 @@ void TabletReplica::UpdateTabletStats(vector<string>* dirty_tablets) {
   // a deadlock.
   RaftPeerPB::Role role = consensus_->role();
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (stats_pb_.on_disk_size() != pb.on_disk_size() ||
       stats_pb_.live_row_count() != pb.live_row_count()) {
     if (consensus::RaftPeerPB_Role_LEADER == role) {
@@ -1065,7 +1062,7 @@ void TabletReplica::UpdateTabletStats(vector<string>* dirty_tablets) {
 }
 
 ReportedTabletStatsPB TabletReplica::GetTabletStats() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return stats_pb_;
 }
 
@@ -1073,7 +1070,7 @@ bool TabletReplica::ShouldRunTxnCoordinatorStalenessTask() {
   if (!txn_coordinator_) {
     return false;
   }
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (state_ != RUNNING) {
     LOG(WARNING) << Substitute("The tablet is not running. State: $0",
                                TabletStatePB_Name(state_));
@@ -1087,7 +1084,7 @@ bool TabletReplica::ShouldRunTxnCoordinatorStateChangedTask() {
   if (!txn_coordinator_) {
     return false;
   }
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   // We only check if the tablet is shutting down here, since replica
   // state change can happen even when the tablet is not running yet.
   if (state_ == STOPPING || state_ == STOPPED) {
@@ -1101,7 +1098,7 @@ bool TabletReplica::ShouldRunTxnCoordinatorStateChangedTask() {
 
 void TabletReplica::DecreaseTxnCoordinatorTaskCounter() {
   DCHECK(txn_coordinator_);
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   txn_coordinator_task_counter_--;
   DCHECK_GE(txn_coordinator_task_counter_, 0);
 }
@@ -1172,7 +1169,7 @@ void TabletReplica::BeginTxnParticipantOp(int64_t txn_id, RegisteredTxnCallback
 void TabletReplica::MakeUnavailable(const Status& error) {
   std::shared_ptr<Tablet> tablet;
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     tablet = tablet_;
     for (auto& op : maintenance_ops_) {
       op->CancelAndDisable();
@@ -1191,10 +1188,12 @@ Status TabletReplica::TxnOpDispatcher::Dispatch(
     std::unique_ptr<WriteOpState> op,
     const std::function<Status(int64_t txn_id, RegisteredTxnCallback cb)>& scheduler) {
   const auto txn_id = op->request()->txn_id();
-  std::lock_guard<simple_spinlock> guard(lock_);
+  std::lock_guard guard(lock_);
   if (PREDICT_FALSE(unregistered_)) {
-    KLOG_EVERY_N_SECS(WARNING, 10) << Substitute(
-        "received request for unregistered TxnOpDispatcher (txn ID $0)", txn_id);
+    KLOG_EVERY_N_SECS(WARNING, 10)
+        << Substitute("received request for unregistered TxnOpDispatcher (txn ID $0)",
+                      txn_id)
+        << THROTTLE_MSG;
     // TODO(aserbin): Status::ServiceUnavailable() is more appropriate here?
     return Status::IllegalState(
         "tablet replica could not accept txn write operation");
@@ -1252,7 +1251,7 @@ Status TabletReplica::TxnOpDispatcher::Submit() {
   decltype(ops_queue_) failed_ops;
   Status failed_status;
   {
-    std::lock_guard<simple_spinlock> guard(lock_);
+    std::lock_guard guard(lock_);
     DCHECK(!preliminary_tasks_completed_);
     while (!ops_queue_.empty()) {
       auto op = std::move(ops_queue_.front());
@@ -1302,20 +1301,21 @@ Status TabletReplica::TxnOpDispatcher::Submit() {
 void TabletReplica::TxnOpDispatcher::Cancel(const Status& status,
                                             TabletServerErrorPB::Code code) {
   CHECK(!status.ok());
-  KLOG_EVERY_N_SECS(WARNING, 1) << Substitute("$0: cancelling pending write operations",
-                                              status.ToString());
+  KLOG_EVERY_N_SECS(WARNING, 1)
+      << Substitute("$0: cancelling pending write operations",
+                    status.ToString()) << THROTTLE_MSG;
   decltype(ops_queue_) ops;
   {
-    std::lock_guard<simple_spinlock> guard(lock_);
+    std::lock_guard guard(lock_);
     inflight_status_ = status;
     std::swap(ops, ops_queue_);
   }
 
-  RespondWithStatus(status, code, std::move(ops));
+  ignore_result(RespondWithStatus(status, code, std::move(ops)));
 }
 
 Status TabletReplica::TxnOpDispatcher::MarkUnregistered() {
-  std::lock_guard<simple_spinlock> guard(lock_);
+  std::lock_guard guard(lock_);
   unregistered_ = true;
 
   // If there are still pending write operations, return ServiceUnavailable()
