@@ -32,9 +32,11 @@
 #include "kudu/client/write_op.h"
 #include "kudu/util/mutex.h"
 #include "kudu/util/status.h"
-#include "kudu/util/threadpool.h"
 
 namespace kudu {
+class ThreadPool;
+class Throttler;
+
 namespace tools {
 
 // This class is not thread-safe.
@@ -46,6 +48,8 @@ class TableScanner {
                    std::nullopt,
                std::optional<std::string> dst_table_name = std::nullopt);
 
+  ~TableScanner();
+
   // Set output stream of this tool, or disable output if not set.
   // 'out' must remain valid for the lifetime of this class.
   void SetOutput(std::ostream* out);
@@ -102,6 +106,7 @@ class TableScanner {
   std::optional<std::string> dst_table_name_;
   int32_t scan_batch_size_;
   std::unique_ptr<ThreadPool> thread_pool_;
+  std::unique_ptr<Throttler> throttler_;
 
   // Protects output to 'out_' so that rows don't get interleaved.
   Mutex output_lock_;
