@@ -19,12 +19,12 @@
 
 #include <functional>
 #include <memory>
+#include <optional>
 #include <string>
 #include <unordered_map>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
-
+#include "kudu/gutil/strings/stringpiece.h"
 #include "kudu/util/status.h"
 
 namespace kudu {
@@ -167,9 +167,9 @@ struct ActionArgsDescriptor {
     // The gflag name.
     std::string name;
     // A default value to override the default gflag value.
-    boost::optional<std::string> default_value;
+    std::optional<std::string> default_value;
     // A description to override the gflag description.
-    boost::optional<std::string> description;
+    std::optional<std::string> description;
   };
 
   // Positional (required) command line arguments.
@@ -183,7 +183,7 @@ struct ActionArgsDescriptor {
 
   // Variable length command line argument. There may be at most one per
   // Action, and it's always found at the end of the command line.
-  boost::optional<Arg> variadic;
+  std::optional<Arg> variadic;
 };
 
 // Builds a new action (leaf) node.
@@ -241,9 +241,10 @@ class ActionBuilder {
   // The default value and description of the flag can be optionally overriden,
   // for cases where the values are action-dependent. Otherwise, the default
   // value and description from the gflag declaration will be used.
-  ActionBuilder& AddOptionalParameter(std::string param,
-                                      boost::optional<std::string> default_value = boost::none,
-                                      boost::optional<std::string> description = boost::none);
+  ActionBuilder& AddOptionalParameter(
+      std::string param,
+      std::optional<std::string> default_value = std::nullopt,
+      std::optional<std::string> description = std::nullopt);
 
   // Creates an action using builder state.
   virtual std::unique_ptr<Action> Build();
@@ -253,9 +254,9 @@ class ActionBuilder {
 
   std::string description_;
 
-  boost::optional<std::string> extra_description_;
+  std::optional<std::string> extra_description_;
 
-  boost::optional<std::string> program_name_;
+  std::optional<std::string> program_name_;
 
   ActionRunner runner_;
 
@@ -290,11 +291,11 @@ class Action {
 
   const std::string& description() const { return description_; }
 
-  const boost::optional<std::string>& extra_description() const {
+  const std::optional<std::string>& extra_description() const {
     return extra_description_;
   }
 
-  const boost::optional<std::string>& program_name() const {
+  const std::optional<std::string>& program_name() const {
     return program_name_;
   }
 
@@ -313,15 +314,21 @@ class Action {
 
   std::string description_;
 
-  boost::optional<std::string> extra_description_;
+  std::optional<std::string> extra_description_;
 
-  boost::optional<std::string> program_name_;
+  std::optional<std::string> program_name_;
 
   ActionRunner runner_;
 
   ActionArgsDescriptor args_;
 };
 
+// Append 'to_append' to 'dst', but hard-wrapped at 78 columns.
+// After any newline, 'continuation_indent' spaces are prepended.
+void AppendHardWrapped(StringPiece to_append,
+                       int continuation_indent,
+                       std::string* dst);
+
 // Returns new nodes for each major mode.
 std::unique_ptr<Mode> BuildClusterMode();
 std::unique_ptr<Mode> BuildDiagnoseMode();
@@ -334,10 +341,13 @@ std::unique_ptr<Mode> BuildPerfMode();
 std::unique_ptr<Mode> BuildRemoteReplicaMode();
 std::unique_ptr<Mode> BuildTableMode();
 std::unique_ptr<Mode> BuildTabletMode();
-std::unique_ptr<Mode> BuildTestMode();
 std::unique_ptr<Mode> BuildTxnMode();
 std::unique_ptr<Mode> BuildTServerMode();
 std::unique_ptr<Mode> BuildWalMode();
 
+#if defined(KUDU_CLI_TEST_TOOL_ENABLED)
+std::unique_ptr<Mode> BuildTestMode();
+#endif
+
 } // namespace tools
 } // namespace kudu
