@@ -172,7 +172,7 @@ class ScannerManager {
   // If true, removal thread should shut itself down. Protected
   // by 'shutdown_lock_' and 'shutdown_cv_'.
   bool shutdown_;
-  mutable Mutex shutdown_lock_;
+  Mutex shutdown_lock_;
   ConditionVariable shutdown_cv_;
 
   std::vector<ScannerMapStripe*> scanner_maps_;
@@ -407,6 +407,9 @@ class Scanner {
     return initted_.load(std::memory_order_acquire);
   }
 
+  // Store the scan request's timings in the tablet metrics.
+  void UpdateTabletMetrics(const CpuTimes& elapsed);
+
   // The unique ID of this scanner.
   const std::string id_;
 
