@@ -37,6 +37,7 @@
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/rpc/rpc_context.h"
+#include "kudu/rpc/transfer.h"
 #include "kudu/server/server_base.h"
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/tablet_replica.h"
@@ -153,7 +154,7 @@ void TabletCopyServiceImpl::BeginTabletCopySession(
   scoped_refptr<RemoteTabletCopySourceSession> session;
   bool new_session;
   {
-    MutexLock l(sessions_lock_);
+    std::lock_guard l(sessions_lock_);
     const SessionEntry* session_entry = FindOrNull(sessions_, session_id);
     new_session = session_entry == nullptr;
     if (new_session) {
@@ -188,7 +189,7 @@ void TabletCopyServiceImpl::BeginTabletCopySession(
   // fails, then remove it from the sessions map.
   Status status = session->Init();
   if (!status.ok()) {
-    MutexLock l(sessions_lock_);
+    std::lock_guard l(sessions_lock_);
     SessionEntry* session_entry = FindOrNull(sessions_, session_id);
     // Identity check the session to ensure that other interleaved threads have
     // not already removed the failed session, and replaced it with a new one.
@@ -204,13 +205,21 @@ void TabletCopyServiceImpl::BeginTabletCopySession(
   resp->set_responder_uuid(fs_manager_->uuid());
   resp->set_session_id(session_id);
   resp->set_session_idle_timeout_millis(FLAGS_tablet_copy_idle_timeout_sec * 1000);
-  resp->mutable_superblock()->CopyFrom(session->tablet_superblock());
   resp->mutable_initial_cstate()->CopyFrom(session->initial_cstate());
 
   for (const scoped_refptr<log::ReadableLogSegment>& segment : session->log_segments()) {
     resp->add_wal_segment_seqnos(segment->header().sequence_number());
   }
-
+  if (req->auto_download_superblock_in_batch() &&
+      (resp->ByteSizeLong() + session->tablet_superblock().ByteSizeLong() >
+      FLAGS_rpc_max_message_size)) {
+    LOG(WARNING) << "Superblock is very large, it maybe over the rpc messsage size, "
+                    "which is controlled by flag --rpc_max_message_size. Switch it into "
+                    "downloading superblock piece by piece.";
+    resp->set_superblock_is_too_large(true);
+  } else {
+    resp->mutable_superblock()->CopyFrom(session->tablet_superblock());
+  }
   // For testing: Close the session prematurely if unsafe gflag is set but
   // still respond as if it was opened.
   const auto timeout_prob = FLAGS_tablet_copy_early_session_timeout_prob;
@@ -218,7 +227,7 @@ void TabletCopyServiceImpl::BeginTabletCopySession(
     LOG_WITH_PREFIX(WARNING) << "Timing out tablet copy session due to flag "
                              << "--tablet_copy_early_session_timeout_prob "
                              << "being set to " << timeout_prob;
-    MutexLock l(sessions_lock_);
+    std::lock_guard l(sessions_lock_);
     TabletCopyErrorPB::Code app_error;
     WARN_NOT_OK(TabletCopyServiceImpl::DoEndTabletCopySessionUnlocked(session_id, &app_error),
                 Substitute("Unable to forcibly end tablet copy session $0", session_id));
@@ -235,7 +244,7 @@ void TabletCopyServiceImpl::CheckSessionActive(
 
   // Look up and validate tablet copy session.
   scoped_refptr<RemoteTabletCopySourceSession> session;
-  MutexLock l(sessions_lock_);
+  std::lock_guard l(sessions_lock_);
   TabletCopyErrorPB::Code app_error;
   Status status = FindSessionUnlocked(session_id, &app_error, &session);
   if (status.ok()) {
@@ -264,7 +273,7 @@ void TabletCopyServiceImpl::FetchData(const FetchDataRequestPB* req,
   // Look up and validate tablet copy session.
   scoped_refptr<RemoteTabletCopySourceSession> session;
   {
-    MutexLock l(sessions_lock_);
+    std::lock_guard l(sessions_lock_);
     TabletCopyErrorPB::Code app_error = TabletCopyErrorPB::UNKNOWN_ERROR;
     RPC_RETURN_NOT_OK(FindSessionUnlocked(session_id, &app_error, &session),
                       app_error, "No such session", context);
@@ -299,6 +308,10 @@ void TabletCopyServiceImpl::FetchData(const FetchDataRequestPB* req,
     RPC_RETURN_NOT_OK(session->GetBlockPiece(block_id, offset, client_maxlen,
                                              data, &total_data_length, &error_code),
                       error_code, "Unable to get piece of data block", context);
+  } else if (data_id.type() == DataIdPB::SUPER_BLOCK) {
+    RPC_RETURN_NOT_OK(session->GetSuperBlockPiece(offset, client_maxlen, data,
+                                                  &total_data_length, &error_code),
+                      error_code, "Unable to get piece of super block", context);
   } else {
     // Fetching a log segment chunk.
     uint64_t segment_seqno = data_id.wal_segment_seqno();
@@ -324,7 +337,7 @@ void TabletCopyServiceImpl::EndTabletCopySession(
         EndTabletCopySessionResponsePB* /*resp*/,
         rpc::RpcContext* context) {
   {
-    MutexLock l(sessions_lock_);
+    std::lock_guard l(sessions_lock_);
     TabletCopyErrorPB::Code app_error = TabletCopyErrorPB::UNKNOWN_ERROR;
     LOG_WITH_PREFIX(INFO) << "Request end of tablet copy session " << req->session_id()
                           << " received from " << context->requestor_string();
@@ -339,7 +352,7 @@ void TabletCopyServiceImpl::Shutdown() {
   session_expiration_thread_->Join();
 
   // Destroy all tablet copy sessions.
-  MutexLock l(sessions_lock_);
+  std::lock_guard l(sessions_lock_);
   auto iter = sessions_.cbegin();
   while (iter != sessions_.cend()) {
     const string& session_id = iter->first;
@@ -371,6 +384,10 @@ Status TabletCopyServiceImpl::FindSessionUnlocked(
 Status TabletCopyServiceImpl::ValidateFetchRequestDataId(
         const DataIdPB& data_id,
         TabletCopyErrorPB::Code* app_error) {
+  if (data_id.type() == DataIdPB::SUPER_BLOCK) {
+    return Status::OK();
+  }
+
   if (PREDICT_FALSE(data_id.has_block_id() && data_id.has_wal_segment_seqno())) {
     *app_error = TabletCopyErrorPB::INVALID_TABLET_COPY_REQUEST;
     return Status::InvalidArgument(
@@ -411,6 +428,7 @@ Status TabletCopyServiceImpl::DoEndTabletCopySessionUnlocked(
   sessions_lock_.AssertAcquired();
   scoped_refptr<RemoteTabletCopySourceSession> session;
   RETURN_NOT_OK(FindSessionUnlocked(session_id, app_error, &session));
+  session->UpdateTabletMetrics();
   // Remove the session from the map.
   // It will get destroyed once there are no outstanding refs.
   LOG_WITH_PREFIX(INFO) << "ending tablet copy session " << session_id << " on tablet "
@@ -421,7 +439,7 @@ Status TabletCopyServiceImpl::DoEndTabletCopySessionUnlocked(
 
 void TabletCopyServiceImpl::EndExpiredSessions() {
   do {
-    MutexLock l(sessions_lock_);
+    std::lock_guard l(sessions_lock_);
     const MonoTime now = MonoTime::Now();
 
     vector<SessionEntry> expired_session_entries;
