@@ -24,6 +24,7 @@
 #include <optional>
 #include <ostream>
 #include <set>
+#include <shared_mutex>
 #include <string>
 #include <type_traits>
 #include <utility>
@@ -68,6 +69,7 @@
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/flag_validators.h"
 #include "kudu/util/logging.h"
+#include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/net/net_util.h"
 #include "kudu/util/scoped_cleanup.h"
@@ -265,6 +267,20 @@ METRIC_DEFINE_gauge_int64(server, tablets_opening_time_startup,
                           "Time taken to start the tablets during server startup",
                           kudu::MetricLevel::kDebug);
 
+METRIC_DEFINE_histogram(server, create_tablet_run_time,
+                        "Create Tablet Operation Run Time",
+                        kudu::MetricUnit::kMicroseconds,
+                        "The runtime of the create tablet operation.",
+                        kudu::MetricLevel::kDebug,
+                        10000000, 2);
+
+METRIC_DEFINE_histogram(server, delete_tablet_run_time,
+                        "Delete Tablet Operation Run Time",
+                        kudu::MetricUnit::kMicroseconds,
+                        "The runtime of the delete tablet operation.",
+                        kudu::MetricLevel::kDebug,
+                        10000000, 2);
+
 DECLARE_int32(heartbeat_interval_ms);
 
 using kudu::consensus::ConsensusMetadata;
@@ -299,6 +315,7 @@ using std::make_shared;
 using std::nullopt;
 using std::optional;
 using std::set;
+using std::shared_lock;
 using std::shared_ptr;
 using std::string;
 using std::vector;
@@ -377,6 +394,11 @@ TSTabletManager::TSTabletManager(TabletServer* server)
 
   tablets_num_opened_startup_ = METRIC_tablets_num_opened_startup.Instantiate(
       server->metric_entity(), 0);
+
+  create_tablet_run_time_ =
+      METRIC_create_tablet_run_time.Instantiate(server->metric_entity());
+  delete_tablet_run_time_ =
+      METRIC_delete_tablet_run_time.Instantiate(server->metric_entity());
 }
 
 // Base class for tasks submitted against TSTabletManager threadpools whose
@@ -573,7 +595,7 @@ Status TSTabletManager::Init(Timer* start_tablets,
                                                registered_count, metas.size());
       scoped_refptr<TransitionInProgressDeleter> deleter;
       {
-        std::lock_guard<RWMutex> lock(lock_);
+        std::lock_guard lock(lock_);
         CHECK_OK(StartTabletStateTransitionUnlocked(meta->tablet_id(), "opening tablet", &deleter));
       }
 
@@ -593,7 +615,7 @@ Status TSTabletManager::Init(Timer* start_tablets,
   }
 
   {
-    std::lock_guard<RWMutex> lock(lock_);
+    std::lock_guard lock(lock_);
     state_ = MANAGER_RUNNING;
   }
 
@@ -631,12 +653,13 @@ Status TSTabletManager::CreateNewTablet(const string& table_id,
 
   // Set the initial opid_index for a RaftConfigPB to -1.
   config.set_opid_index(consensus::kInvalidOpIdIndex);
+  MonoTime time_started = MonoTime::Now();
 
   scoped_refptr<TransitionInProgressDeleter> deleter;
   {
     // acquire the lock in exclusive mode as we'll add a entry to the
     // transition_in_progress_ set if the lookup fails.
-    std::lock_guard<RWMutex> lock(lock_);
+    std::lock_guard lock(lock_);
     TRACE("Acquired tablet manager lock");
 
     // Sanity check that the tablet isn't already registered.
@@ -684,6 +707,10 @@ Status TSTabletManager::CreateNewTablet(const string& table_id,
   if (replica) {
     *replica = new_replica;
   }
+
+  create_tablet_run_time_->Increment(
+      (MonoTime::Now() - time_started).ToMicroseconds());
+
   return Status::OK();
 }
 
@@ -809,7 +836,7 @@ void TSTabletManager::RunTabletCopy(
   bool replacing_tablet = false;
   scoped_refptr<TransitionInProgressDeleter> deleter;
   {
-    std::lock_guard<RWMutex> lock(lock_);
+    std::lock_guard lock(lock_);
     if (LookupTabletUnlocked(tablet_id, &old_replica)) {
       meta = old_replica->tablet_metadata();
       replacing_tablet = true;
@@ -1034,7 +1061,7 @@ Status TSTabletManager::BeginReplicaStateTransition(
     TabletServerErrorPB::Code* error_code) {
   // Acquire the lock in exclusive mode as we'll add a entry to the
   // transition_in_progress_ map.
-  std::lock_guard<RWMutex> lock(lock_);
+  std::lock_guard lock(lock_);
   TRACE("Acquired tablet manager lock");
   RETURN_NOT_OK(CheckRunningUnlocked(error_code));
 
@@ -1116,6 +1143,7 @@ Status TSTabletManager::DeleteTablet(
   }
 
   TRACE("Deleting tablet $0", tablet_id);
+  MonoTime time_started = MonoTime::Now();
 
   if (PREDICT_FALSE(FLAGS_delete_tablet_inject_latency_ms > 0)) {
     LOG(WARNING) << "Injecting " << FLAGS_delete_tablet_inject_latency_ms
@@ -1137,6 +1165,8 @@ Status TSTabletManager::DeleteTablet(
   // If a tablet is already tombstoned, then a request to tombstone
   // the same tablet should become a no-op.
   if (delete_type == TABLET_DATA_TOMBSTONED && data_state == TABLET_DATA_TOMBSTONED) {
+    delete_tablet_run_time_->Increment(
+        (MonoTime::Now() - time_started).ToMicroseconds());
     return Status::OK();
   }
 
@@ -1190,11 +1220,13 @@ Status TSTabletManager::DeleteTablet(
   // Only DELETED tablets are fully shut down and removed from the tablet map.
   if (delete_type == TABLET_DATA_DELETED) {
     replica->Shutdown();
-    std::lock_guard<RWMutex> lock(lock_);
+    std::lock_guard lock(lock_);
     RETURN_NOT_OK(CheckRunningUnlocked(error_code));
     CHECK_EQ(1, tablet_map_.erase(tablet_id)) << tablet_id;
     InsertOrDie(&perm_deleted_tablet_ids_, tablet_id);
   }
+  delete_tablet_run_time_->Increment(
+      (MonoTime::Now() - time_started).ToMicroseconds());
 
   return Status::OK();
 }
@@ -1453,7 +1485,7 @@ void TSTabletManager::IncrementTabletsProcessed(int tablets_total,
 
 void TSTabletManager::Shutdown() {
   {
-    std::lock_guard<RWMutex> lock(lock_);
+    std::lock_guard lock(lock_);
     switch (state_) {
       case MANAGER_QUIESCING: {
         VLOG(1) << "Tablet manager shut down already in progress..";
@@ -1528,7 +1560,7 @@ void TSTabletManager::Shutdown() {
   }
 
   {
-    std::lock_guard<RWMutex> l(lock_);
+    std::lock_guard l(lock_);
     // We don't expect anyone else to be modifying the map after we start the
     // shut down process.
     CHECK_EQ(tablet_map_.size(), replicas_to_shutdown.size())
@@ -1542,7 +1574,7 @@ void TSTabletManager::Shutdown() {
 void TSTabletManager::RegisterTablet(const string& tablet_id,
                                      const scoped_refptr<TabletReplica>& replica,
                                      RegisterTabletReplicaMode mode) {
-  std::lock_guard<RWMutex> lock(lock_);
+  std::lock_guard lock(lock_);
   // If we are replacing a tablet replica, we delete the existing one first.
   if (mode == REPLACEMENT_REPLICA && tablet_map_.erase(tablet_id) != 1) {
     LOG(FATAL) << "Unable to remove previous tablet replica " << tablet_id << ": not registered!";
@@ -1926,8 +1958,9 @@ Status TSTabletManager::DeleteTabletData(
   return Status::OK();
 }
 
-void TSTabletManager::FailTabletsInDataDir(const string& uuid) {
-  DataDirManager* dd_manager = fs_manager_->dd_manager();
+void TSTabletManager::FailTabletsInDataDir(const string& uuid,
+                                           const string& tenant_id) {
+  scoped_refptr<DataDirManager> dd_manager = fs_manager_->dd_manager(tenant_id);
   int uuid_idx;
   CHECK(dd_manager->FindUuidIndexByUuid(uuid, &uuid_idx))
       << Substitute("No data directory found with UUID $0", uuid);
@@ -1936,7 +1969,8 @@ void TSTabletManager::FailTabletsInDataDir(const string& uuid) {
     return;
   }
   // Fail the directory to prevent other tablets from being placed in it.
-  dd_manager->MarkDirFailed(uuid_idx);
+  WARN_NOT_OK(dd_manager->MarkDirFailed(uuid_idx),
+              "error marking data directory as failed");
   set<string> tablets = dd_manager->FindTabletsByDirUuidIdx(uuid_idx);
   LOG(INFO) << Substitute("Data dir $0 has $1 tablets", uuid, tablets.size());
   for (const string& tablet_id : dd_manager->FindTabletsByDirUuidIdx(uuid_idx)) {
@@ -1986,7 +2020,7 @@ void TSTabletManager::FailTabletAndScheduleShutdown(const string& tablet_id) {
 
 int TSTabletManager::RefreshTabletStateCacheAndReturnCount(tablet::TabletStatePB st) {
   MonoDelta period = MonoDelta::FromMilliseconds(FLAGS_tablet_state_walk_min_period_ms);
-  std::lock_guard<RWMutex> lock(lock_);
+  std::lock_guard lock(lock_);
   if (last_walked_ + period < MonoTime::Now()) {
     // Old cache: regenerate counts.
     tablet_state_counts_.clear();
@@ -2062,7 +2096,7 @@ Status TSTabletManager::SchedulePreliminaryTasksForTxnWrite(
 
 Status TSTabletManager::ScheduleAbortTxn(int64_t txn_id, const string& user) {
   {
-    std::lock_guard<simple_spinlock> l(txn_aborts_lock_);
+    std::lock_guard l(txn_aborts_lock_);
     if (!InsertIfNotPresent(&txn_aborts_in_progress_, txn_id)) {
       return Status::OK();
     }
@@ -2078,7 +2112,7 @@ Status TSTabletManager::ScheduleAbortTxn(int64_t txn_id, const string& user) {
         if (s.IsTimedOut()) {
           // Presumably this was a transient error. Try again.
           {
-            std::lock_guard<simple_spinlock> l(txn_aborts_lock_);
+            std::lock_guard l(txn_aborts_lock_);
             txn_aborts_in_progress_.erase(txn_id);
           }
           WARN_NOT_OK(ScheduleAbortTxn(txn_id, user),
@@ -2086,13 +2120,13 @@ Status TSTabletManager::ScheduleAbortTxn(int64_t txn_id, const string& user) {
           return;
         }
         WARN_NOT_OK(s, Substitute("Error aborting transaction $0 as user $1", txn_id, user));
-        std::lock_guard<simple_spinlock> l(txn_aborts_lock_);
+        std::lock_guard l(txn_aborts_lock_);
         txn_aborts_in_progress_.erase(txn_id);
       });
 }
 
 void TSTabletManager::SetNextUpdateTimeForTests() {
-  std::lock_guard<rw_spinlock> l(lock_update_);
+  std::lock_guard l(lock_update_);
   next_update_time_ = MonoTime::Now();
 }
 
@@ -2102,7 +2136,7 @@ TransitionInProgressDeleter::TransitionInProgressDeleter(
 
 void TransitionInProgressDeleter::Destroy() {
   CHECK(!is_destroyed_);
-  std::lock_guard<RWMutex> lock(*lock_);
+  std::lock_guard lock(*lock_);
   CHECK(in_progress_->erase(entry_));
   is_destroyed_ = true;
 }
