@@ -23,9 +23,11 @@
 #include <map>
 #include <memory>
 #include <optional>
+#include <shared_mutex>
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
+#include <type_traits>
 #include <vector>
 
 #include <gtest/gtest_prod.h>
@@ -103,7 +105,7 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // Construct the tablet manager.
   explicit TSTabletManager(TabletServer* server);
 
-  virtual ~TSTabletManager();
+  ~TSTabletManager() override;
 
   // Load all tablet metadata blocks from disk, and open their respective tablets.
   // Starts the timer, 'start_tablets' and populates the 'tablets_total'. The subsequent
@@ -176,7 +178,7 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
                                   scoped_refptr<tablet::TabletReplica>* replica) const
                                   override;
 
-  virtual const NodeInstancePB& NodeInstance() const override;
+  const NodeInstancePB& NodeInstance() const override;
 
   // Initiate tablet copy of the specified tablet on the tablet_copy_pool_.
   // See the StartTabletCopy() RPC declaration in consensus.proto for details.
@@ -243,7 +245,7 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   void FailTabletAndScheduleShutdown(const std::string& tablet_id);
 
   // Forces shutdown of the tablet replicas in the data dir corresponding to 'uuid'.
-  void FailTabletsInDataDir(const std::string& uuid);
+  void FailTabletsInDataDir(const std::string& uuid, const std::string& tenant_id);
 
   // Refresh the cached counts of tablet states, if the cache is old enough,
   // and return the count for tablet state 'st'.
@@ -386,7 +388,7 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
                                  int64_t last_logged_term);
 
   TSTabletManagerStatePB state() const {
-    shared_lock<RWMutex> l(lock_);
+    std::shared_lock<RWMutex> l(lock_);
     return state_;
   }
 
@@ -499,6 +501,10 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // ensures we do not attempt to collect metrics while calling the destructor.
   FunctionGaugeDetacher metric_detacher_;
 
+  // Track the creation and deletion time of tablets on the node.
+  scoped_refptr<Histogram> create_tablet_run_time_;
+  scoped_refptr<Histogram> delete_tablet_run_time_;
+
   DISALLOW_COPY_AND_ASSIGN(TSTabletManager);
 };
 
