@@ -39,10 +39,12 @@
 #include <cstdint>
 #include <cstdlib>
 #include <functional>
+#include <map>
 #include <memory>
 #include <ostream>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <unordered_set>
 #include <utility>
 #include <vector>
@@ -60,6 +62,7 @@
 #include "kudu/gutil/strings/util.h"
 #include "kudu/util/array_view.h" // IWYU pragma: keep
 #include "kudu/util/env_util.h"
+#include "kudu/util/errno.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/path_util.h"
@@ -80,7 +83,6 @@ DECLARE_int32(env_inject_short_read_bytes);
 DECLARE_int32(env_inject_short_write_bytes);
 DECLARE_int32(encryption_key_length);
 DECLARE_string(env_inject_eio_globs);
-DECLARE_string(encryption_server_key);
 
 namespace kudu {
 
@@ -98,7 +100,7 @@ static const uint64_t kTwoMb = 2 * kOneMb;
 
 class TestEnv : public KuduTest {
  public:
-  virtual void SetUp() OVERRIDE {
+  void SetUp() override {
     KuduTest::SetUp();
     CheckFallocateSupport();
   }
@@ -132,7 +134,10 @@ class TestEnv : public KuduTest {
       }
     }
 
-    RETRY_ON_EINTR(err, close(fd));
+    if (PREDICT_FALSE(close(fd) != 0)) {
+      const int err = errno;
+      LOG(WARNING) << Substitute("error closing fd $0: $1", fd, ErrnoToString(err));
+    }
 #endif
 
     checked = true;
@@ -246,8 +251,7 @@ bool TestEnv::fallocate_punch_hole_supported_ = false;
 
 TEST_F(TestEnv, TestPreallocate) {
   if (!fallocate_supported_) {
-    LOG(INFO) << "fallocate not supported, skipping test";
-    return;
+    GTEST_SKIP() << "fallocate not supported, skipping test";
   }
   LOG(INFO) << "Testing PreAllocate()";
   string test_path = GetTestPath("test_env_wf");
@@ -259,11 +263,11 @@ TEST_F(TestEnv, TestPreallocate) {
   ASSERT_OK(file->Sync());
 
   // the writable file size should report 0
-  ASSERT_EQ(file->Size(), 0);
+  ASSERT_EQ(0, file->Size());
   // but the real size of the file on disk should report 1MB
   uint64_t size;
   ASSERT_OK(env_->GetFileSize(test_path, &size));
-  ASSERT_EQ(size, kOneMb);
+  ASSERT_EQ(kOneMb, size);
 
   // write 1 MB
   uint8_t scratch[kOneMb];
@@ -272,7 +276,7 @@ TEST_F(TestEnv, TestPreallocate) {
   ASSERT_OK(file->Sync());
 
   // the writable file size should now report 1 MB
-  ASSERT_EQ(file->Size(), kOneMb);
+  ASSERT_EQ(kOneMb, file->Size());
   ASSERT_OK(file->Close());
   // and the real size for the file on disk should match only the
   // written size
@@ -285,8 +289,7 @@ TEST_F(TestEnv, TestPreallocate) {
 // be smaller than the mmapped regions size).
 TEST_F(TestEnv, TestConsecutivePreallocate) {
   if (!fallocate_supported_) {
-    LOG(INFO) << "fallocate not supported, skipping test";
-    return;
+    GTEST_SKIP() << "fallocate not supported, skipping test";
   }
   LOG(INFO) << "Testing consecutive PreAllocate()";
   string test_path = GetTestPath("test_env_wf");
@@ -298,11 +301,11 @@ TEST_F(TestEnv, TestConsecutivePreallocate) {
   ASSERT_OK(file->Sync());
 
   // the writable file size should report 0
-  ASSERT_EQ(file->Size(), 0);
+  ASSERT_EQ(0, file->Size());
   // but the real size of the file on disk should report 64 MBs
   uint64_t size;
   ASSERT_OK(env_->GetFileSize(test_path, &size));
-  ASSERT_EQ(size, 64 * kOneMb);
+  ASSERT_EQ(64 * kOneMb, size);
 
   // write 1 MB
   uint8_t scratch[kOneMb];
@@ -311,7 +314,7 @@ TEST_F(TestEnv, TestConsecutivePreallocate) {
   ASSERT_OK(file->Sync());
 
   // the writable file size should now report 1 MB
-  ASSERT_EQ(kOneMb, file->Size());
+  ASSERT_EQ(file->Size(), kOneMb);
   ASSERT_OK(env_->GetFileSize(test_path, &size));
   ASSERT_EQ(64 * kOneMb, size);
 
@@ -330,7 +333,7 @@ TEST_F(TestEnv, TestConsecutivePreallocate) {
   ASSERT_OK(file->Sync());
 
   // the writable file size should now report 2 MB
-  ASSERT_EQ(file->Size(), 2 * kOneMb);
+  ASSERT_EQ(2 * kOneMb, file->Size());
   // while the real file size should reamin at 128 MBs
   ASSERT_OK(env_->GetFileSize(test_path, &size));
   ASSERT_EQ(128 * kOneMb, size);
@@ -345,8 +348,7 @@ TEST_F(TestEnv, TestConsecutivePreallocate) {
 
 TEST_F(TestEnv, TestHolePunch) {
   if (!fallocate_punch_hole_supported_) {
-    LOG(INFO) << "hole punching not supported, skipping test";
-    return;
+    GTEST_SKIP() << "hole punching not supported, skipping test";
   }
   string test_path = GetTestPath("test_env_wf");
   unique_ptr<RWFile> file;
@@ -382,8 +384,7 @@ TEST_F(TestEnv, TestHolePunchBenchmark) {
   const int kHoleSize = 10 * kOneMb;
   const int kNumRuns = 1000;
   if (!fallocate_punch_hole_supported_) {
-    LOG(INFO) << "hole punching not supported, skipping test";
-    return;
+    GTEST_SKIP() << "hole punching not supported, skipping test";
   }
   Random r(SeedRandom());
 
@@ -531,8 +532,8 @@ TEST_F(TestEnv, TestReadVFully) {
 
   // Verify that Read fully reads the whole requested data.
   ASSERT_OK(file->ReadV(0, results));
-  ASSERT_EQ(result1, "abcde");
-  ASSERT_EQ(result2, "12345");
+  ASSERT_EQ("abcde", result1);
+  ASSERT_EQ("12345", result2);
 
   // Turn short reads off again
   FLAGS_env_inject_short_read_bytes = 0;
@@ -625,10 +626,9 @@ TEST_F(TestEnv, TestOverwrite) {
 }
 
 TEST_F(TestEnv, TestReopen) {
-  LOG(INFO) << "Testing reopening behavior";
-  string test_path = GetTestPath("test_env_wf");
-  string first = "The quick brown fox";
-  string second = "jumps over the lazy dog";
+  const string test_path = GetTestPath("test_env_wf");
+  const string first = "The quick brown fox";
+  const string second = "jumps over the lazy dog";
 
   // Create the file and write to it.
   shared_ptr<WritableFile> writer;
@@ -889,8 +889,8 @@ TEST_F(TestEnv, TestGetFileModifiedTime) {
   // HFS has 1 second mtime granularity.
   AssertEventually([&] {
     int64_t after_time;
-    writer->Append(" ");
-    writer->Sync();
+    ASSERT_OK(writer->Append(" "));
+    ASSERT_OK(writer->Sync());
     ASSERT_OK(env_->GetFileModifiedTime(writer->filename(), &after_time));
     ASSERT_LT(initial_time, after_time);
   }, MonoDelta::FromSeconds(5));
@@ -910,7 +910,7 @@ TEST_F(TestEnv, TestRWFile) {
   uint8_t scratch[kTestData.length()];
   Slice result(scratch, kTestData.length());
   ASSERT_OK(file->Read(0, result));
-  ASSERT_EQ(result, kTestData);
+  ASSERT_EQ(kTestData, result);
   uint64_t sz;
   ASSERT_OK(file->Size(&sz));
   ASSERT_EQ(kTestData.length(), sz);
@@ -924,8 +924,8 @@ TEST_F(TestEnv, TestRWFile) {
   Slice result2(scratch2, size2);
   vector<Slice> results = { result1, result2 };
   ASSERT_OK(file->ReadV(0, results));
-  ASSERT_EQ(result1, "abc");
-  ASSERT_EQ(result2, "de");
+  ASSERT_EQ("abc", result1);
+  ASSERT_EQ("de", result2);
 
   // Write past the end of the file and rewrite some of the interior.
   ASSERT_OK(file->Write(kTestData.length() * 2, kTestData));
@@ -937,7 +937,7 @@ TEST_F(TestEnv, TestRWFile) {
   ASSERT_OK(file->Read(0, result3));
 
   // Retest.
-  ASSERT_EQ(result3, kNewTestData);
+  ASSERT_EQ(kNewTestData, result3);
   ASSERT_OK(file->Size(&sz));
   ASSERT_EQ(kNewTestData.length(), sz);
 
@@ -952,7 +952,7 @@ TEST_F(TestEnv, TestRWFile) {
   uint8_t scratch4[kNewTestData.length()];
   Slice result4(scratch4, kNewTestData.length());
   ASSERT_OK(file->Read(0, result4));
-  ASSERT_EQ(result4, kNewTestData);
+  ASSERT_EQ(kNewTestData, result4);
 
   // Test CREATE_OR_OPEN semantics on a new file.
   const string bar_path = GetTestPath("bar");
@@ -978,7 +978,7 @@ TEST_F(TestEnv, TestCanonicalize) {
     ASSERT_EQ(test_dir_, result);
   }
 
-  string dir = GetTestPath("some_dir");
+  const string dir = GetTestPath("some_dir");
   ASSERT_OK(env_->CreateDir(dir));
   string result;
   ASSERT_OK(env_->Canonicalize(dir + "/", &result));
