@@ -66,7 +66,7 @@ class Env {
     MUST_EXIST
   };
 
-  Env() { }
+  Env() = default;
   virtual ~Env();
 
   // Return a default environment suitable for the current operating
@@ -80,6 +80,9 @@ class Env {
   // environment.  Unlike the default env, this is not owned by Kudu, and
   // must be destroyed when not used anymore.
   static std::unique_ptr<Env> NewEnv();
+  // Same as the usage of "NewEnv()", it also needs to be destroyed when
+  // no longer used.
+  static std::shared_ptr<Env> NewSharedEnv();
 
   // Create a brand new sequentially-readable file with the specified name.
   // On success, stores a pointer to the new file in *result and returns OK.
@@ -393,6 +396,9 @@ class Env {
   // Set the raw server encryption key. The key size is in bits.
   virtual void SetEncryptionKey(const uint8_t* key, size_t key_size) = 0;
 
+  // Used for manipulating the proc filesystem
+  virtual Status EchoToFile(const char* file_path, const char* data_ptr, int data_size) = 0;
+
  private:
   DISALLOW_COPY_AND_ASSIGN(Env);
 };
@@ -426,7 +432,7 @@ class Fifo : public File {
 
   // Initializes the default environment with encryption enabled using the
   // given AES key.
-  static Status InitializeEncryptedEnv(int key_size, uint8_t* server_key);
+  static Status InitializeEncryptedEnv(int key_size, uint8_t* encryption_key);
 
   // Returns the write fd, set when opened for writes. The fifo must have been
   // opened for writes before calling.
@@ -729,7 +735,7 @@ class RWFile : public File {
 // Identifies a locked file.
 class FileLock {
  public:
-  FileLock() { }
+  FileLock() = default;
   virtual ~FileLock();
  private:
   DISALLOW_COPY_AND_ASSIGN(FileLock);
