@@ -22,9 +22,9 @@
 #include <cstring>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -310,10 +310,18 @@ class Descriptor<RWFile> : public RWFile {
     return opened.file()->GetExtentMap(out);
   }
 
+  bool IsEncrypted() const override {
+    return true;
+  }
+
   const string& filename() const override {
     return base_.filename();
   }
 
+  size_t GetEncryptionHeaderSize() const override {
+    return base_.env()->GetEncryptionHeaderSize();
+  }
+
  private:
   friend class ::kudu::FileCache;
 
@@ -342,6 +350,7 @@ class Descriptor<RWFile> : public RWFile {
     // The file was evicted, reopen it.
     RWFileOptions opts;
     opts.mode = Mode;
+    opts.is_sensitive = true;
     unique_ptr<RWFile> f;
     RETURN_NOT_OK(base_.env()->NewRWFile(opts, base_.filename(), &f));
 
@@ -391,6 +400,10 @@ class Descriptor<RandomAccessFile> : public RandomAccessFile {
     return base_.filename();
   }
 
+  size_t GetEncryptionHeaderSize() const override {
+    return base_.env()->GetEncryptionHeaderSize();
+  }
+
   size_t memory_footprint() const override {
     // Normally we would use kudu_malloc_usable_size(this). However, that's
     // not safe because 'this' was allocated via std::make_shared(), which
@@ -440,7 +453,9 @@ class Descriptor<RandomAccessFile> : public RandomAccessFile {
 
     // The file was evicted, reopen it.
     unique_ptr<RandomAccessFile> f;
-    RETURN_NOT_OK(base_.env()->NewRandomAccessFile(base_.filename(), &f));
+    RandomAccessFileOptions opts;
+    opts.is_sensitive = true;
+    RETURN_NOT_OK(base_.env()->NewRandomAccessFile(opts, base_.filename(), &f));
 
     // The cache will take ownership of the newly opened file.
     ScopedOpenedDescriptor<RandomAccessFile> opened(
@@ -472,7 +487,7 @@ FileCache::FileCache(const string& cache_name,
       running_(1) {
   if (entity) {
     unique_ptr<FileCacheMetrics> metrics(new FileCacheMetrics(entity));
-    cache_->SetMetrics(std::move(metrics));
+    cache_->SetMetrics(std::move(metrics), Cache::ExistingMetricsPolicy::kKeep);
   }
   LOG(INFO) << Substitute("Constructed file cache $0 with capacity $1",
                           cache_name, max_open_files);
@@ -498,7 +513,7 @@ Status FileCache::DoOpenFile(const string& file_name,
   shared_ptr<internal::Descriptor<RWFile>> d;
   bool cd;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     d = FindDescriptorUnlocked(file_name, FindMode::CREATE_IF_NOT_EXIST,
                                &rwf_descs_, &cd);
     DCHECK(d);
@@ -526,7 +541,7 @@ Status FileCache::DoOpenFile(const string& file_name,
   shared_ptr<internal::Descriptor<RandomAccessFile>> d;
   bool cd;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     d = FindDescriptorUnlocked(file_name, FindMode::CREATE_IF_NOT_EXIST,
                                &raf_descs_, &cd);
     DCHECK(d);
@@ -608,7 +623,7 @@ Status FileCache::DeleteFile(const string& file_name) {
   // descriptor per file name, we can short circuit the search if we find a
   // descriptor in the first map.
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     bool ignored;
     {
       auto d = FindDescriptorUnlocked(file_name, FindMode::DONT_CREATE,
@@ -653,7 +668,7 @@ void FileCache::Invalidate(const string& file_name) {
   shared_ptr<internal::Descriptor<RWFile>> rwf_desc;
   shared_ptr<internal::Descriptor<RandomAccessFile>> raf_desc;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     bool ignored;
     rwf_desc = FindDescriptorUnlocked(file_name, FindMode::CREATE_IF_NOT_EXIST,
                                       &rwf_descs_, &ignored);
@@ -676,14 +691,14 @@ void FileCache::Invalidate(const string& file_name) {
   // duration of this method, and no other methods erase strong references from
   // the maps.
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     CHECK_EQ(1, rwf_descs_.erase(file_name));
     CHECK_EQ(1, raf_descs_.erase(file_name));
   }
 }
 
 size_t FileCache::NumDescriptorsForTests() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return rwf_descs_.size() + raf_descs_.size();
 }
 
@@ -695,7 +710,7 @@ string FileCache::ToDebugString() const {
   DescriptorMap<RWFile> rwfs_copy;
   DescriptorMap<RandomAccessFile> rafs_copy;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     rwfs_copy = rwf_descs_;
     rafs_copy = raf_descs_;
   }
@@ -785,7 +800,7 @@ void FileCache::ExpireDescriptorsFromMap(DescriptorMap<FileType>* descs) {
 void FileCache::RunDescriptorExpiry() {
   while (!running_.WaitFor(MonoDelta::FromMilliseconds(
       FLAGS_file_cache_expiry_period_ms))) {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     ExpireDescriptorsFromMap(&rwf_descs_);
     ExpireDescriptorsFromMap(&raf_descs_);
   }
