@@ -43,14 +43,14 @@ TAG_FLAG(metrics_retirement_age_ms, advanced);
 // More complex applications will define other entities.
 METRIC_DEFINE_entity(server);
 
-namespace kudu {
-
 using std::string;
 using std::unordered_set;
 using std::vector;
 using strings::Substitute;
 using strings::SubstituteAndAppend;
 
+namespace kudu {
+
 template<typename Collection>
 void WriteMetricsToJson(JsonWriter* writer,
                         const Collection& metrics,
@@ -231,7 +231,7 @@ void MetricEntity::CheckInstantiation(const MetricPrototype* proto) const {
 }
 
 scoped_refptr<Metric> MetricEntity::FindOrNull(const MetricPrototype& prototype) const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return FindPtrOrNull(metric_map_, &prototype);
 }
 
@@ -303,7 +303,7 @@ Status MetricEntity::GetMetricsAndAttrs(const MetricFilters& filters,
 
   {
     // Snapshot the metrics in this registry (not guaranteed to be a consistent snapshot)
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     *attrs = attributes_;
     *metrics = metric_map_;
   }
@@ -404,49 +404,38 @@ Status MetricEntity::WriteAsJson(JsonWriter* writer, const MetricJsonOptions& op
   return Status::OK();
 }
 
-Status MetricEntity::WriteAsPrometheus(PrometheusWriter* writer) const {
+Status MetricEntity::WriteAsPrometheus(
+    PrometheusWriter* writer, const MetricPrometheusOptions& opts) const {
   static const string kIdMaster = "kudu.master";
   static const string kIdTabletServer = "kudu.tabletserver";
 
-  if (strcmp(prototype_->name(), "server") != 0) {
-    // Only server-level metrics are emitted in Prometheus format as of now,
-    // non-server metric entities are currently silently skipped.
-    //
-    // TODO(KUDU-3563): output tablet-level metrics in Prometheus format as well
-    return Status::OK();
-  }
-
-  // Empty filters result in getting all the metrics for this MetricEntity.
-  //
-  // TODO(aserbin): instead of hard-coding, pass MetricFilters as a parameter
-  MetricFilters filters;
-  filters.entity_level = "debug";
-
   MetricMap metrics;
   AttributeMap attrs;
-  const auto s = GetMetricsAndAttrs(filters, &metrics, &attrs);
+  const auto s = GetMetricsAndAttrs(opts.filters, &metrics, &attrs);
   if (s.IsNotFound()) {
     // Status::NotFound is returned when this entity has been filtered, treat it
     // as OK, and skip printing it.
     return Status::OK();
   }
   RETURN_NOT_OK(s);
-
-  if (id_ == kIdMaster) {
-    // Prefix all master metrics with 'kudu_master_'.
-    static const string kMasterPrefix = "kudu_master_";
-    WriteMetricsPrometheus(writer, metrics, kMasterPrefix);
-    return Status::OK();
+  if (strcmp(prototype_->name(), "server") == 0) {
+    if (id_ == kIdMaster) {
+      // Prefix all master metrics with 'kudu_master_'.
+      static const string kMasterPrefix = "kudu_master_";
+      WriteMetricsPrometheus(writer, metrics, kMasterPrefix);
+      return Status::OK();
+    }
+    if (id_ == kIdTabletServer) {
+      // Prefix all tablet server metrics with 'kudu_tserver_'.
+      static const string kTabletServerPrefix = "kudu_tserver_";
+      WriteMetricsPrometheus(writer, metrics, kTabletServerPrefix);
+      return Status::OK();
+    }
+    return Status::NotSupported(Substitute("$0: unexpected server-level metric entity", id_));
   }
-  if (id_ == kIdTabletServer) {
-    // Prefix all tablet server metrics with 'kudu_tserver_'.
-    static const string kTabletServerPrefix = "kudu_tserver_";
-    WriteMetricsPrometheus(writer, metrics, kTabletServerPrefix);
-    return Status::OK();
-  }
-
-  return Status::NotSupported(
-      Substitute("$0: unexpected server-level metric entity", id_));
+  const string prefix = Substitute("kudu_$0_$1_", prototype_->name(), id_);
+  WriteMetricsPrometheus(writer, metrics, prefix);
+  return Status::OK();
 }
 
 Status MetricEntity::CollectTo(MergedEntityMetrics* collections,
@@ -498,7 +487,7 @@ Status MetricEntity::CollectTo(MergedEntityMetrics* collections,
 void MetricEntity::RetireOldMetrics() {
   MonoTime now(MonoTime::Now());
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (auto it = metric_map_.begin(); it != metric_map_.end();) {
     const scoped_refptr<Metric>& metric = it->second;
 
@@ -540,17 +529,17 @@ void MetricEntity::RetireOldMetrics() {
 }
 
 void MetricEntity::NeverRetire(const scoped_refptr<Metric>& metric) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   never_retire_metrics_.push_back(metric);
 }
 
 void MetricEntity::SetAttributes(const AttributeMap& attrs) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   attributes_ = attrs;
 }
 
 void MetricEntity::SetAttribute(const string& key, const string& val) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   attributes_[key] = val;
 }
 
@@ -567,7 +556,7 @@ MetricRegistry::~MetricRegistry() {
 Status MetricRegistry::WriteAsJson(JsonWriter* writer, const MetricJsonOptions& opts) const {
   EntityMap entities;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     entities = entities_;
   }
 
@@ -597,14 +586,15 @@ Status MetricRegistry::WriteAsJson(JsonWriter* writer, const MetricJsonOptions&
   return Status::OK();
 }
 
-Status MetricRegistry::WriteAsPrometheus(PrometheusWriter* writer) const {
+Status MetricRegistry::WriteAsPrometheus(
+    PrometheusWriter* writer, const MetricPrometheusOptions& opts) const {
   EntityMap entities;
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     entities = entities_;
   }
   for (const auto& e : entities) {
-    WARN_NOT_OK(e.second->WriteAsPrometheus(writer),
+    WARN_NOT_OK(e.second->WriteAsPrometheus(writer, opts),
                 Substitute("Failed to write entity $0 as Prometheus", e.second->id()));
   }
 
@@ -614,7 +604,7 @@ Status MetricRegistry::WriteAsPrometheus(PrometheusWriter* writer) const {
 }
 
 void MetricRegistry::RetireOldMetrics() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (auto it = entities_.begin(); it != entities_.end();) {
     it->second->RetireOldMetrics();
 
@@ -640,17 +630,17 @@ MetricPrototypeRegistry* MetricPrototypeRegistry::get() {
 }
 
 void MetricPrototypeRegistry::AddMetric(const MetricPrototype* prototype) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   metrics_.push_back(prototype);
 }
 
 void MetricPrototypeRegistry::AddEntity(const MetricEntityPrototype* prototype) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   entities_.push_back(prototype);
 }
 
 void MetricPrototypeRegistry::WriteAsJson(JsonWriter* writer) const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   MetricJsonOptions opts;
   opts.include_schema_info = true;
   writer->StartObject();
@@ -689,7 +679,7 @@ void MetricPrototypeRegistry::WriteAsJson() const {
 }
 
 void MetricPrototypeRegistry::WriteAsXML() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   std::cout << "<?xml version=\"1.0\"?>" << "\n";
   // Add a root node for the document.
   std::cout << "<AllMetrics>" << "\n";
@@ -780,7 +770,7 @@ scoped_refptr<MetricEntity> MetricRegistry::FindOrCreateEntity(
     const MetricEntityPrototype* prototype,
     const string& id,
     const MetricEntity::AttributeMap& initial_attrs) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   scoped_refptr<MetricEntity> e = FindPtrOrNull(entities_, id);
   if (!e) {
     e = new MetricEntity(prototype, id, initial_attrs);
@@ -859,7 +849,7 @@ StringGauge::StringGauge(const GaugePrototype<string>* proto,
       unique_values_(std::move(initial_unique_values)) {}
 
 scoped_refptr<Metric> StringGauge::snapshot() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   auto p = new StringGauge(down_cast<const GaugePrototype<string>*>(prototype_),
                            value_,
                            unique_values_);
@@ -870,7 +860,7 @@ scoped_refptr<Metric> StringGauge::snapshot() const {
 }
 
 string StringGauge::value() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (PREDICT_TRUE(unique_values_.empty())) {
     return value_;
   }
@@ -884,14 +874,14 @@ void StringGauge::FillUniqueValuesUnlocked() {
 }
 
 unordered_set<string> StringGauge::unique_values() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   FillUniqueValuesUnlocked();
   return unique_values_;
 }
 
 void StringGauge::set_value(const string& value) {
   UpdateModificationEpoch();
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   value_ = value;
   unique_values_.clear();
 }
@@ -909,7 +899,7 @@ void StringGauge::MergeFrom(const scoped_refptr<Metric>& other) {
   scoped_refptr<StringGauge> other_ptr = down_cast<StringGauge*>(other.get());
   auto other_values = other_ptr->unique_values();
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   FillUniqueValuesUnlocked();
   unique_values_.insert(other_values.begin(), other_values.end());
 }
@@ -923,14 +913,15 @@ void StringGauge::WriteValue(JsonWriter* writer) const {
 // (see https://prometheus.io/docs/instrumenting/exposition_formats/).
 // DCHECK() is added to make sure this method is not called from anywhere,
 // but overriding it is necessary since Gauge::WriteValue() is a pure virtual one.
-// An alternative could be defining a empty implementation for Gauge::WriteValue()
+// An alternative could be defining an empty implementation for Gauge::WriteValue()
 // virtual method and not adding this empty override here.
-void StringGauge::WriteValue(PrometheusWriter* writer, const std::string& prefix) const {
+void StringGauge::WriteValue(PrometheusWriter* /*writer*/,
+                             const string& /*prefix*/) const {
   DCHECK(false);
 }
 
 Status StringGauge::WriteAsPrometheus(PrometheusWriter* /*writer*/,
-                                      const std::string& /*prefix*/) const {
+                                      const string& /*prefix*/) const {
   // Prometheus doesn't support string gauges.
   // This function ensures that output written to Prometheus is empty.
   return Status::OK();
@@ -941,7 +932,7 @@ Status StringGauge::WriteAsPrometheus(PrometheusWriter* /*writer*/,
 //
 
 scoped_refptr<Metric> MeanGauge::snapshot() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   auto p = new MeanGauge(down_cast<const GaugePrototype<double>*>(prototype_));
   p->set_value(total_sum_, total_count_);
   p->m_epoch_.store(m_epoch_);
@@ -951,23 +942,23 @@ scoped_refptr<Metric> MeanGauge::snapshot() const {
 }
 
 double MeanGauge::value() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return total_count_ > 0 ? total_sum_ / total_count_
                           : 0.0;
 }
 
 double MeanGauge::total_sum() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return total_sum_;
 }
 
 double MeanGauge::total_count() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return total_count_;
 }
 
 void MeanGauge::set_value(double total_sum, double total_count) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   total_sum_ = total_sum;
   total_count_ = total_count;
 }
@@ -983,7 +974,7 @@ void MeanGauge::MergeFrom(const scoped_refptr<Metric>& other) {
 
   UpdateModificationEpoch();
   scoped_refptr<MeanGauge> other_ptr = down_cast<MeanGauge*>(other.get());
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   total_sum_ += other_ptr->total_sum();
   total_count_ += other_ptr->total_count();
 }
@@ -1178,6 +1169,7 @@ Status Histogram::GetHistogramSnapshotPB(HistogramSnapshotPB* snapshot_pb,
     snapshot_pb->set_percentile_99_9(0);
     snapshot_pb->set_percentile_99_99(0);
     snapshot_pb->set_max(0);
+    snapshot_pb->set_last(0);
   } else {
     HdrHistogram snapshot(*histogram_);
     snapshot_pb->set_total_count(snapshot.TotalCount());
@@ -1190,6 +1182,7 @@ Status Histogram::GetHistogramSnapshotPB(HistogramSnapshotPB* snapshot_pb,
     snapshot_pb->set_percentile_99_9(snapshot.ValueAtPercentile(99.9));
     snapshot_pb->set_percentile_99_99(snapshot.ValueAtPercentile(99.99));
     snapshot_pb->set_max(snapshot.MaxValue());
+    snapshot_pb->set_last(snapshot.LastValue());
 
     if (opts.include_raw_histograms) {
       RecordedValuesIterator iter(&snapshot);
