@@ -233,7 +233,6 @@
 #include <functional>
 #include <limits>
 #include <memory>
-#include <mutex>
 #include <ostream>
 #include <string>
 #include <unordered_map>
@@ -250,7 +249,6 @@
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/substitute.h"
-#include "kudu/util/atomic.h"
 #include "kudu/util/hdr_histogram.h"
 #include "kudu/util/jsonwriter.h" // IWYU pragma: keep
 #include "kudu/util/locks.h"
@@ -514,6 +512,17 @@ struct MetricJsonOptions {
   MetricMergeRules merge_rules;
 };
 
+// Options to control the behavior of the metric end-points producing Kudu
+// metrics in the Prometheus format.
+struct MetricPrometheusOptions {
+  // Metrics are filtered per information in the 'filters' field,
+  // see the in-line MetricFilters's documentation for more details.
+  //
+  // NOTE: for Prometheus format metrics, the only applicable filtering
+  //       is by 'MetricFilters::entity_level' field.
+  MetricFilters filters;
+};
+
 class MetricEntityPrototype {
  public:
   explicit MetricEntityPrototype(const char* name);
@@ -588,8 +597,7 @@ class MetricPrototype {
                                 const std::string& prefix) const;
  protected:
   explicit MetricPrototype(CtorArgs args);
-  virtual ~MetricPrototype() {
-  }
+  virtual ~MetricPrototype() = default;
 
   const CtorArgs args_;
 
@@ -670,7 +678,8 @@ class MetricEntity : public RefCountedThreadSafe<MetricEntity> {
   // See MetricRegistry::WriteAsJson()
   Status WriteAsJson(JsonWriter* writer, const MetricJsonOptions& opts) const;
 
-  Status WriteAsPrometheus(PrometheusWriter* writer) const;
+  Status WriteAsPrometheus(PrometheusWriter* writer,
+                           const MetricPrometheusOptions& opts) const;
 
   // Collect metrics of this entity to 'collections'. Metrics will be filtered by 'filters',
   // and will be merged under the rule of 'merge_rules'.
@@ -700,19 +709,19 @@ class MetricEntity : public RefCountedThreadSafe<MetricEntity> {
   void SetAttribute(const std::string& key, const std::string& val);
 
   int num_metrics() const {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     return metric_map_.size();
   }
 
   // Mark this entity as unpublished. This will cause the registry to retire its metrics
   // and unregister it.
   void Unpublish() {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     published_ = false;
   }
 
   bool published() {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     return published_;
   }
 
@@ -887,7 +896,8 @@ class MetricRegistry {
   Status WriteAsJson(JsonWriter* writer, const MetricJsonOptions& opts) const;
 
   // Writes metrics in this registry to given Prometheus 'writer'.
-  Status WriteAsPrometheus(PrometheusWriter* writer) const;
+  Status WriteAsPrometheus(PrometheusWriter* writer,
+                           const MetricPrometheusOptions& opts) const;
   // For each registered entity, retires orphaned metrics. If an entity has no more
   // metrics and there are no external references, entities are removed as well.
   //
@@ -896,12 +906,12 @@ class MetricRegistry {
 
   // Return the number of entities in this registry.
   int num_entities() const {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     return entities_.size();
   }
 
  private:
-  typedef std::unordered_map<std::string, scoped_refptr<MetricEntity> > EntityMap;
+  typedef std::unordered_map<std::string, scoped_refptr<MetricEntity>> EntityMap;
   EntityMap entities_;
 
   mutable simple_spinlock lock_;
@@ -1008,12 +1018,11 @@ class GaugePrototype : public MetricPrototype {
     return gauge;
   }
 
-  virtual MetricType::Type type() const OVERRIDE {
+  MetricType::Type type() const override {
     if (args_.flags_ & EXPOSE_AS_COUNTER) {
       return MetricType::kCounter;
-    } else {
-      return MetricType::kGauge;
     }
+    return MetricType::kGauge;
   }
 
   void WriteHelpAndType(PrometheusWriter* writer,
@@ -1035,9 +1044,9 @@ class Gauge : public Metric {
     : Metric(prototype) {
   }
   ~Gauge() override {}
-  Status WriteAsJson(JsonWriter* w, const MetricJsonOptions& opts) const OVERRIDE;
+  Status WriteAsJson(JsonWriter* w, const MetricJsonOptions& opts) const override;
 
-  Status WriteAsPrometheus(PrometheusWriter* w, const std::string& prefix) const OVERRIDE;
+  Status WriteAsPrometheus(PrometheusWriter* w, const std::string& prefix) const override;
  protected:
   virtual void WriteValue(JsonWriter* writer) const = 0;
   virtual void WriteValue(PrometheusWriter* writer, const std::string& prefix) const = 0;
@@ -1052,10 +1061,10 @@ class StringGauge : public Gauge {
               std::string initial_value,
               std::unordered_set<std::string> initial_unique_values
                   = std::unordered_set<std::string>());
-  scoped_refptr<Metric> snapshot() const OVERRIDE;
+  scoped_refptr<Metric> snapshot() const override;
   std::string value() const;
   void set_value(const std::string& value);
-  virtual bool IsUntouched() const override {
+  bool IsUntouched() const override {
     return false;
   }
   void MergeFrom(const scoped_refptr<Metric>& other) override;
@@ -1087,7 +1096,7 @@ class MeanGauge : public Gauge {
   double total_count() const;
   double total_sum() const;
   void set_value(double total_sum, double total_count);
-  virtual bool IsUntouched() const override {
+  bool IsUntouched() const override {
     return false;
   }
   void MergeFrom(const scoped_refptr<Metric>& other) override;
@@ -1143,19 +1152,19 @@ class AtomicGauge : public Gauge {
     return scoped_refptr<Metric>(p);
   }
   T value() const {
-    return static_cast<T>(value_.Load(kMemOrderRelease));
+    return static_cast<T>(value_.load(std::memory_order_acquire));
   }
   void set_value(const T& value) {
     UpdateModificationEpoch();
-    value_.Store(static_cast<int64_t>(value), kMemOrderNoBarrier);
+    value_.store(static_cast<int64_t>(value), std::memory_order_relaxed);
   }
   void Increment() {
     UpdateModificationEpoch();
-    value_.IncrementBy(1, kMemOrderNoBarrier);
+    value_.fetch_add(1, std::memory_order_relaxed);
   }
   void IncrementBy(int64_t amount) {
     UpdateModificationEpoch();
-    value_.IncrementBy(amount, kMemOrderNoBarrier);
+    value_.fetch_add(amount, std::memory_order_relaxed);
   }
   void Decrement() {
     IncrementBy(-1);
@@ -1163,7 +1172,7 @@ class AtomicGauge : public Gauge {
   void DecrementBy(int64_t amount) {
     IncrementBy(-amount);
   }
-  virtual bool IsUntouched() const override {
+  bool IsUntouched() const override {
     return false;
   }
   void MergeFrom(const scoped_refptr<Metric>& other) override {
@@ -1191,7 +1200,7 @@ class AtomicGauge : public Gauge {
     }
   }
  protected:
-  void WriteValue(JsonWriter* writer) const OVERRIDE {
+  void WriteValue(JsonWriter* writer) const override {
     writer->Value(value());
   }
 
@@ -1203,7 +1212,7 @@ class AtomicGauge : public Gauge {
                                 value());
   }
  private:
-  AtomicInt<int64_t> value_;
+  std::atomic<int64_t> value_;
   MergeType type_;
 
   DISALLOW_COPY_AND_ASSIGN(AtomicGauge);
@@ -1286,11 +1295,11 @@ class FunctionGauge : public Gauge {
   }
 
   T value() const {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     return function_();
   }
 
-  void WriteValue(JsonWriter* writer) const OVERRIDE {
+  void WriteValue(JsonWriter* writer) const override {
     writer->Value(value());
   }
 
@@ -1306,7 +1315,7 @@ class FunctionGauge : public Gauge {
   // This should be used during destruction. If you want a settable
   // Gauge, use a normal Gauge instead of a FunctionGauge.
   void DetachToConstant(T v) {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     function_ = [v]() { return v; };
   }
 
@@ -1338,7 +1347,7 @@ class FunctionGauge : public Gauge {
     detacher->OnDestructor([self]() { self->DetachToCurrentValue(); });
   }
 
-  virtual bool IsUntouched() const override {
+  bool IsUntouched() const override {
     return false;
   }
 
@@ -1395,7 +1404,7 @@ class CounterPrototype : public MetricPrototype {
   }
   scoped_refptr<Counter> Instantiate(const scoped_refptr<MetricEntity>& entity);
 
-  virtual MetricType::Type type() const OVERRIDE { return MetricType::kCounter; }
+  MetricType::Type type() const override { return MetricType::kCounter; }
 
  private:
   DISALLOW_COPY_AND_ASSIGN(CounterPrototype);
@@ -1420,9 +1429,9 @@ class Counter : public Metric {
   int64_t value() const;
   void Increment();
   void IncrementBy(int64_t amount);
-  Status WriteAsJson(JsonWriter* w, const MetricJsonOptions& opts) const OVERRIDE;
+  Status WriteAsJson(JsonWriter* w, const MetricJsonOptions& opts) const override;
 
-  Status WriteAsPrometheus(PrometheusWriter* w, const std::string& prefix) const OVERRIDE;
+  Status WriteAsPrometheus(PrometheusWriter* w, const std::string& prefix) const override;
 
   bool IsUntouched() const override {
     return value() == 0;
@@ -1466,7 +1475,7 @@ class HistogramPrototype : public MetricPrototype {
 
   uint64_t max_trackable_value() const { return max_trackable_value_; }
   int num_sig_digits() const { return num_sig_digits_; }
-  virtual MetricType::Type type() const OVERRIDE { return MetricType::kHistogram; }
+  MetricType::Type type() const override { return MetricType::kHistogram; }
 
  private:
   const uint64_t max_trackable_value_;
@@ -1496,9 +1505,9 @@ class Histogram : public Metric {
   // or IncrementBy()).
   uint64_t TotalCount() const;
 
-  Status WriteAsJson(JsonWriter* w, const MetricJsonOptions& opts) const OVERRIDE;
+  Status WriteAsJson(JsonWriter* w, const MetricJsonOptions& opts) const override;
 
-  Status WriteAsPrometheus(PrometheusWriter* w, const std::string& prefix) const OVERRIDE;
+  Status WriteAsPrometheus(PrometheusWriter* w, const std::string& prefix) const override;
 
   // Returns a snapshot of this histogram including the bucketed values and counts.
   Status GetHistogramSnapshotPB(HistogramSnapshotPB* snapshot_pb,
@@ -1513,7 +1522,7 @@ class Histogram : public Metric {
   uint64_t MaxValueForTests() const;
   double MeanValueForTests() const;
 
-  virtual bool IsUntouched() const override {
+  bool IsUntouched() const override {
     return TotalCount() == 0;
   }
 
@@ -1563,7 +1572,7 @@ class ScopedLatencyMetric {
 inline scoped_refptr<Counter> MetricEntity::FindOrCreateCounter(
     const CounterPrototype* proto) {
   CheckInstantiation(proto);
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   scoped_refptr<Counter> m = down_cast<Counter*>(FindPtrOrNull(metric_map_, proto).get());
   if (!m) {
     m = new Counter(proto);
@@ -1575,7 +1584,7 @@ inline scoped_refptr<Counter> MetricEntity::FindOrCreateCounter(
 inline scoped_refptr<Histogram> MetricEntity::FindOrCreateHistogram(
     const HistogramPrototype* proto) {
   CheckInstantiation(proto);
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   scoped_refptr<Histogram> m = down_cast<Histogram*>(FindPtrOrNull(metric_map_, proto).get());
   if (!m) {
     m = new Histogram(proto);
@@ -1590,7 +1599,7 @@ inline scoped_refptr<AtomicGauge<T> > MetricEntity::FindOrCreateGauge(
     const T& initial_value,
     MergeType type) {
   CheckInstantiation(proto);
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   scoped_refptr<AtomicGauge<T> > m = down_cast<AtomicGauge<T>*>(
       FindPtrOrNull(metric_map_, proto).get());
   if (!m) {
@@ -1603,7 +1612,7 @@ inline scoped_refptr<AtomicGauge<T> > MetricEntity::FindOrCreateGauge(
 inline scoped_refptr<MeanGauge> MetricEntity::FindOrCreateMeanGauge(
     const GaugePrototype<double>* proto) {
   CheckInstantiation(proto);
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   scoped_refptr<MeanGauge> m = down_cast<MeanGauge*>(
       FindPtrOrNull(metric_map_, proto).get());
   if (!m) {
@@ -1619,7 +1628,7 @@ inline scoped_refptr<FunctionGauge<T> > MetricEntity::FindOrCreateFunctionGauge(
     std::function<T()> function,
     MergeType type) {
   CheckInstantiation(proto);
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   scoped_refptr<FunctionGauge<T> > m = down_cast<FunctionGauge<T>*>(
       FindPtrOrNull(metric_map_, proto).get());
   if (!m) {
