@@ -513,6 +513,10 @@ build_gperftools() {
     $GPERFTOOLS_SOURCE/configure $cfg_options
   fixup_libtool
   make -j$PARALLEL $EXTRA_MAKEFLAGS install
+  # Add tcmalloc_guard.h because TCMallocGuard is useful for fine-grained
+  # control over the initialization of libtcmalloc internals, and that's
+  # helpful in the context of addressing KUDU-2439 and KUDU-3635.
+  install -m 0444 $GPERFTOOLS_SOURCE/src/tcmalloc_guard.h $PREFIX/include/gperftools
   popd
 }
 
@@ -1159,7 +1163,7 @@ build_gumbo_query() {
   cmake \
     -DCMAKE_BUILD_TYPE=release \
     -DCMAKE_INSTALL_PREFIX=$PREFIX \
-    -DCMAKE_CXX_FLAGS="$EXTRA_CXXFLAGS" \
+    -DCMAKE_CXX_FLAGS="$EXTRA_CXXFLAGS -I$PREFIX/include" \
     -DCMAKE_EXE_LINKER_FLAGS="$EXTRA_LDFLAGS $EXTRA_LIBS" \
     -DCMAKE_MODULE_LINKER_FLAGS="$EXTRA_LDFLAGS $EXTRA_LIBS" \
     -DCMAKE_SHARED_LINKER_FLAGS="$EXTRA_LDFLAGS $EXTRA_LIBS -L$PREFIX/lib -Wl,-rpath,$PREFIX/lib" \
